@@ -30,7 +30,7 @@
             Philipp Tomsich
             Christian Thalinger
 
-   $Id: cacaoh.c 4921 2006-05-15 14:24:36Z twisti $
+   $Id: cacaoh.c 5809 2006-10-20 13:09:54Z twisti $
 
 */
 
@@ -147,6 +147,11 @@ static void version(void)
 }
 
 
+/* forward declarations *******************************************************/
+
+static JavaVMInitArgs *cacaoh_options_prepare(int argc, char **argv);
+
+
 /* main ************************************************************************
 
    Main program.
@@ -185,17 +190,17 @@ int main(int argc, char **argv)
 	if (cp) {
 		bootclasspath = MNEW(char, strlen(cp) + strlen("0"));
 		strcpy(bootclasspath, cp);
-
-	} else {
-		cplen = strlen(CACAO_VM_ZIP_PATH) +
+	}
+	else {
+		cplen = strlen(CACAO_VM_ZIP) +
 			strlen(":") +
-			strlen(CLASSPATH_GLIBJ_ZIP_PATH) +
+			strlen(CLASSPATH_GLIBJ_ZIP) +
 			strlen("0");
 
 		bootclasspath = MNEW(char, cplen);
-		strcat(bootclasspath, CACAO_VM_ZIP_PATH);
+		strcat(bootclasspath, CACAO_VM_ZIP);
 		strcat(bootclasspath, ":");
-		strcat(bootclasspath, CLASSPATH_GLIBJ_ZIP_PATH);
+		strcat(bootclasspath, CLASSPATH_GLIBJ_ZIP);
 	}
 
 
@@ -206,8 +211,8 @@ int main(int argc, char **argv)
 	if (cp) {
 		classpath = MNEW(char, strlen(cp) + strlen("0"));
 		strcat(classpath, cp);
-
-	} else {
+	}
+	else {
 		classpath = MNEW(char, strlen(".") + strlen("0"));
 		strcpy(classpath, ".");
 	}
@@ -224,7 +229,7 @@ int main(int argc, char **argv)
 
 	/* parse the options ******************************************************/
 
-	vm_args = options_prepare(argc, argv);
+	vm_args = cacaoh_options_prepare(argc, argv);
 
 	while ((opt = options_get(opts, vm_args)) != OPT_DONE) {
 		switch (opt) {
@@ -279,14 +284,17 @@ int main(int argc, char **argv)
 		log_text("Java - header-generator started"); 
 	}
 	
+#if defined(ENABLE_THREADS)
+	/* pre-initialize some core thread stuff, like the stopworldlock,
+	   thus this has to happen _before_ gc_init()!!! */
+
+	threads_preinit();
+#endif
+
 	/* initialize the garbage collector */
 
 	gc_init(heapmaxsize, heapstartsize);
 
-#if defined(ENABLE_THREADS)
-	threads_preinit();
-#endif
-
 	/* initialize the string hashtable stuff: lock (must be done
 	   _after_ threads_preinit) */
 
@@ -371,6 +379,29 @@ int main(int argc, char **argv)
 }
 
 
+/* cacaoh_options_prepare ******************************************************
+
+   Prepare the JavaVMInitArgs.
+
+*******************************************************************************/
+
+static JavaVMInitArgs *cacaoh_options_prepare(int argc, char **argv)
+{
+	JavaVMInitArgs *vm_args;
+	s4              i;
+
+	vm_args = NEW(JavaVMInitArgs);
+
+	vm_args->nOptions = argc - 1;
+	vm_args->options  = MNEW(JavaVMOption, argc);
+
+	for (i = 1; i < argc; i++)
+		vm_args->options[i - 1].optionString = argv[i];
+
+	return vm_args;
+}
+
+
 /*
  * These are local overrides for various environment variables in Emacs.
  * Please do not remove this and leave it at the end of the file, where
