@@ -31,7 +31,7 @@
             Christian Thalinger
 			Edwin Steiner
 
-   $Id: headers.c 4938 2006-05-22 09:06:44Z twisti $
+   $Id: headers.c 5868 2006-10-30 11:21:36Z edwin $
 
 */
 
@@ -69,6 +69,11 @@
 #include "vm/jit/asmpart.h"
 
 
+/* Invocation API variables ***************************************************/
+
+_Jv_JavaVM *_Jv_jvm;                    /* denotes a Java VM                  */
+char       *_Jv_bootclasspath;
+
 #if defined(ENABLE_INTRP)
 /* dummy interpreter stack to keep the compiler happy */
 
@@ -104,6 +109,16 @@ java_objectheader *vm_call_method(methodinfo *m, java_objectheader *o, ...)
 
 void vm_abort(const char *text, ...)
 {
+	va_list ap;
+
+	/* print the log message */
+
+	va_start(ap, text);
+	vfprintf(stderr, text, ap);
+	va_end(ap);
+
+	/* now abort the VM */
+
 	abort();
 }
 
@@ -139,12 +154,13 @@ void *asm_switchstackandcall(void *stack, void *func, void **stacktopsave, void
 
 void asm_handle_builtin_exception(classinfo *c) {}
 
-
 #if defined(ENABLE_JIT)
+void asm_abstractmethoderror(void) {}
 void asm_getclassvalues_atomic(vftbl_t *super, vftbl_t *sub, castinfo *out) {}
 #endif
 
 #if defined(ENABLE_INTRP)
+void intrp_asm_abstractmethoderror(void) {}
 void intrp_asm_getclassvalues_atomic(vftbl_t *super, vftbl_t *sub, castinfo *out) {}
 #endif
 
@@ -174,10 +190,12 @@ bool typeinfo_init_class(typeinfo *info,classref_or_classinfo c)
 }
 
 void typeinfo_print(FILE *file,typeinfo *info,int indent) {}
+void typeinfo_print_short(FILE *file,typeinfo *info) {}
 
 void exceptions_print_exception(java_objectheader *xptr) {}
+void stacktrace_dump_trace(threadobject *thread) {}
 void stacktrace_print_trace(java_objectheader *xptr) {}
-
+java_objectarray *stacktrace_getClassContext() { return NULL; }
 void code_free_code_of_method(methodinfo *m) {}
 
 
@@ -189,13 +207,9 @@ void throw_main_exception_exit(void) { assert(0); }
 void throw_exception(void) { assert(0); }
 void throw_exception_exit(void) { assert(0); }
 
-java_objectheader *new_verifyerror(methodinfo *m, const char *message)
+void exceptions_throw_verifyerror(methodinfo *m, const char *message)
 {
 	assert(0);
-
-	/* keep compiler happy */
-
-	return NULL;
 }
 
 java_objectheader *new_exception_throwable(const char *classname, java_lang_Throwable *throwable)
@@ -282,6 +296,14 @@ java_objectheader *new_exception_javastring(const char *classname,
 }
 
 
+void exceptions_throw_abstractmethoderror(void)
+{
+	fprintf(stderr, "java.lang.AbstractMethodError\n");
+
+	exit(1);
+}
+
+
 java_objectheader *new_classformaterror(classinfo *c, const char *message, ...)
 {
 	va_list ap;
@@ -375,6 +397,12 @@ java_objectheader *exceptions_new_nosuchmethoderror(classinfo *c,
 }
 
 
+void exceptions_throw_nosuchmethoderror(classinfo *c, utf *name, utf *desc)
+{
+	(void) exceptions_new_nosuchmethoderror(c, name, desc);
+}
+
+
 java_objectheader *new_internalerror(const char *message, ...)
 {
 	va_list ap;
@@ -414,7 +442,31 @@ java_objectheader *new_unsupportedclassversionerror(classinfo *c, const char *me
 }
 
 
-java_objectheader *new_illegalmonitorstateexception(void)
+void exceptions_throw_arrayindexoutofboundsexception(void)
+{
+	fprintf(stderr, "%s", string_java_lang_ArrayIndexOutOfBoundsException);
+	exit(1);
+}
+
+
+java_objectheader *exceptions_new_arraystoreexception(void)
+{
+	fprintf(stderr, "%s", string_java_lang_ArrayStoreException);
+	exit(1);
+
+	/* keep compiler happy */
+
+	return NULL;
+}
+
+
+void exceptions_throw_arraystoreexception(void)
+{
+	(void) exceptions_new_arraystoreexception();
+}
+
+
+java_objectheader *exceptions_throw_illegalmonitorstateexception(void)
 {
 	fprintf(stderr, "%s", string_java_lang_IllegalMonitorStateException);
 	exit(1);
@@ -480,6 +532,8 @@ void print_dynamic_super_statistics(void) {}
 
 /************************ global variables **********************/
 
+#define ACC_NATIVELY_OVERLOADED    0x10000000
+
 chain *ident_chain;     /* chain with method and field names in current class */
 FILE *file = NULL;
 static u4 outputsize;
@@ -513,7 +567,7 @@ static void addoutputsize (int len)
 	u4 newsize,i;
 	if (!dopadding) return;
 
-	newsize = ALIGN(outputsize, len);
+	newsize = MEMORY_ALIGN(outputsize, len);
 	
 	for (i = outputsize; i < newsize; i++) fprintf(file, "   u1 pad%d\n", (int) i);
 	outputsize = newsize;
@@ -706,11 +760,9 @@ void printmethod(methodinfo *m)
 	fprintf(file, "_");
 	printID(m->name);
 
-	/* ATTENTION: We use the methodinfo's isleafmethod variable as
-	   nativelyoverloaded, so we can save some space during
-	   runtime. */
+	/* ATTENTION: We use a dummy flag here. */
 
-	if (m->isleafmethod)
+	if (m->flags & ACC_NATIVELY_OVERLOADED)
 		printOverloadPart(m->descriptor);
 
 	fprintf(file, "(JNIEnv *env");
@@ -832,11 +884,9 @@ void headerfile_generate(classinfo *c, char *opt_directory)
 		if (!(m->flags & ACC_NATIVE))
 			continue;
 
-		/* We use the methodinfo's isleafmethod variable as
-		   nativelyoverloaded, so we can save some space during
-		   runtime. */
+		/* We use a dummy flag here. */
 
-		if (!m->isleafmethod) {
+		if (!(m->flags & ACC_NATIVELY_OVERLOADED)) {
 			nativelyoverloaded = false;
 
 			for (j = i + 1; j < c->methodscount; j++) {
@@ -846,13 +896,14 @@ void headerfile_generate(classinfo *c, char *opt_directory)
 					continue;
 
 				if (m->name == m2->name) {
-					m2->isleafmethod = true;
-					nativelyoverloaded = true;
+					m2->flags          |= ACC_NATIVELY_OVERLOADED;
+					nativelyoverloaded  = true;
 				}
 			}
 		}
 
-		m->isleafmethod = nativelyoverloaded;
+		if (nativelyoverloaded == true)
+			m->flags |= ACC_NATIVELY_OVERLOADED;
 	}
 
 	for (i = 0; i < c->methodscount; i++) {
@@ -887,6 +938,15 @@ void print_classname(classinfo *clazz)
 	}
 } 
 
+/* jvmti releated functions ************************************************/
+
+#if defined(ENABLE_JVMTI)
+void jvmti_ThreadStartEnd(int ev) {;}
+void jvmti_ClassLoadPrepare(bool prepared, classinfo *c) {;}
+void jvmti_MonitorContendedEntering(bool entered, jobject obj) {;}
+#endif
+
+
 
 /*
  * These are local overrides for various environment variables in Emacs.
