@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: boehm.h 4921 2006-05-15 14:24:36Z twisti $
+   $Id: boehm.h 5189 2006-07-31 12:25:09Z twisti $
 
 */
 
@@ -47,12 +47,13 @@
 void  gc_init(u4 heapmaxsize, u4 heapstartsize);
 
 void *heap_alloc_uncollectable(u4 bytelength);
-void *heap_allocate(u4 bytelength, bool references, methodinfo *finalizer);
+void *heap_allocate(u4 bytelength, u4 references, methodinfo *finalizer);
 void  heap_free(void *p);
 
 void  gc_call(void);
 s8    gc_get_heap_size(void);
 s8    gc_get_free_bytes(void);
+s8    gc_get_total_bytes(void);
 s8    gc_get_max_heap_size(void);
 void  gc_invoke_finalizers(void);
 void  gc_finalize_all(void);
