@@ -29,11 +29,13 @@
    Changes: Christian Thalinger
    			Edwin Steiner
 
-   $Id: memory.c 4957 2006-05-26 11:48:10Z edwin $
+   $Id: memory.c 5868 2006-10-30 11:21:36Z edwin $
 
 */
 
 
+#include "config.h"
+
 #include <assert.h>
 #include <errno.h>
 #include <stdio.h>
@@ -48,7 +50,6 @@
 # include <sys/types.h>
 #endif
 
-#include "config.h"
 #include "vm/types.h"
 
 #include "arch.h"
@@ -57,7 +58,10 @@
 #include "native/native.h"
 
 #if defined(ENABLE_THREADS)
+# include "threads/native/lock.h"
 # include "threads/native/threads.h"
+#else
+# include "threads/none/lock.h"
 #endif
 
 #include "toolbox/logging.h"
@@ -66,6 +70,7 @@
 #include "vm/options.h"
 #include "vm/statistics.h"
 #include "vm/stringlocal.h"
+#include "vm/vm.h"
 
 
 /*******************************************************************************
@@ -76,7 +81,7 @@
 *******************************************************************************/
 
 #if !defined(ENABLE_THREADS)
-static dumpinfo _no_threads_dumpinfo;
+static dumpinfo_t _no_threads_dumpinfo;
 #endif
 
 #if defined(ENABLE_THREADS)
@@ -88,13 +93,14 @@ static dumpinfo _no_threads_dumpinfo;
 
 /* global code memory variables ***********************************************/
 
-#define DEFAULT_CODEMEM_SIZE    128 * 1024  /* defaulting to 128kB            */
+#define DEFAULT_CODE_MEMORY_SIZE    128 * 1024 /* defaulting to 128kB         */
 
 #if defined(ENABLE_THREADS)
-static java_objectheader *codememlock = NULL;
+static java_objectheader *lock_code_memory = NULL;
 #endif
-static int                codememsize = 0;
-static void              *codememptr  = NULL;
+static void              *code_memory      = NULL;
+static int                code_memory_size = 0;
+static int                pagesize         = 0;
 
 
 /* memory_init *****************************************************************
@@ -106,11 +112,15 @@ static void              *codememptr  = NULL;
 bool memory_init(void)
 {
 #if defined(ENABLE_THREADS)
-	codememlock = NEW(java_objectheader);
+	lock_code_memory = NEW(java_objectheader);
 
-	lock_init_object_lock(codememlock);
+	lock_init_object_lock(lock_code_memory);
 #endif
 
+	/* get the pagesize of this architecture */
+
+	pagesize = getpagesize();
+
 	/* everything's ok */
 
 	return true;
@@ -132,7 +142,7 @@ static void *memory_checked_alloc(s4 size)
 
 	void *p = calloc(size, 1);
 
-	if (!p)
+	if (p == NULL)
 		exceptions_throw_outofmemory_exit();
 
 	return p;
@@ -141,82 +151,107 @@ static void *memory_checked_alloc(s4 size)
 
 /* memory_cnew *****************************************************************
 
-   Allocates memory from the heap, aligns it to architecutres PAGESIZE
-   and make the memory read-, write-, and executeable.
+   Allocates memory from the heap via mmap and make the memory read-,
+   write-, and executeable.
 
 *******************************************************************************/
 
 void *memory_cnew(s4 size)
 {
 	void *p;
-	int   pagesize;
 
-#if defined(ENABLE_THREADS)
-	builtin_monitorenter(codememlock);
-#endif
+	LOCK_MONITOR_ENTER(lock_code_memory);
 
-	size = ALIGN(size, ALIGNSIZE);
+	size = MEMORY_ALIGN(size, ALIGNSIZE);
 
 	/* check if enough memory is available */
 
-	if (size > codememsize) {
+	if (size > code_memory_size) {
 		/* set default code size */
 
-		codememsize = DEFAULT_CODEMEM_SIZE;
+		code_memory_size = DEFAULT_CODE_MEMORY_SIZE;
 
 		/* do we need more? */
 
-		if (size > codememsize)
-			codememsize = size;
+		if (size > code_memory_size)
+			code_memory_size = size;
 
-		/* get the pagesize of this architecture */
+		/* align the size of the memory to be allocated */
 
-		pagesize = getpagesize();
-
-		/* allocate normal heap memory */
-
-		if ((p = memory_checked_alloc(codememsize + pagesize - 1)) == NULL)
-			return NULL;
+		code_memory_size = MEMORY_ALIGN(code_memory_size, pagesize);
 
 #if defined(ENABLE_STATISTICS)
 		if (opt_stat) {
-			codememusage += codememsize + pagesize - 1;
+			codememusage += code_memory_size;
 
 			if (codememusage > maxcodememusage)
 				maxcodememusage = codememusage;
 		}
 #endif
 
-		/* align the memory allocated to a multiple of PAGESIZE,
-		   mprotect requires this */
+		/* allocate the memory */
 
-		p = (void *) (((ptrint) p + pagesize - 1) & ~(pagesize - 1));
+#if defined(MAP_ANONYMOUS) || defined(MAP_ANON)
+		p = mmap(NULL,
+				 (size_t) code_memory_size,
+				 PROT_READ | PROT_WRITE | PROT_EXEC,
+				 MAP_PRIVATE |
+# if defined(MAP_ANONYMOUS)
+				 MAP_ANONYMOUS,
+# elif defined(MAP_ANON)
+				 MAP_ANON,
+# else
+				 0,
+# endif
+				 -1, 	 
+				 (off_t) 0);
 
-		/* make the memory read-, write-, and executeable */
+# if defined(MAP_FAILED)
+		if (p == MAP_FAILED)
+# else
+		if (p == (void *) -1)
+# endif
+			vm_abort("mmap failed: %s", strerror(errno));
 
-		if (mprotect(p, codememsize, PROT_READ | PROT_WRITE | PROT_EXEC) == -1)
-			throw_cacao_exception_exit(string_java_lang_InternalError,
-									   strerror(errno));
+#else
+		/* This works a least on IRIX. */
+
+		p = memory_checked_alloc(code_memory_size);
+#endif
 
 		/* set global code memory pointer */
 
-		codememptr = p;
+		code_memory = p;
 	}
 
 	/* get a memory chunk of the allocated memory */
 
-	p = codememptr;
-	codememptr = (void *) ((ptrint) codememptr + size);
-	codememsize -= size;
+	p = code_memory;
 
-#if defined(ENABLE_THREADS)
-	builtin_monitorexit(codememlock);
-#endif
+	code_memory       = (void *) ((ptrint) code_memory + size);
+	code_memory_size -= size;
+
+	LOCK_MONITOR_EXIT(lock_code_memory);
 
 	return p;
 }
 
 
+/* memory_cfree ****************************************************************
+
+   Frees the code memory pointed to.
+
+   ATTENTION: This function currently does NOTHING!  Because we don't
+   have a memory management for code memory.
+
+*******************************************************************************/
+
+void memory_cfree(void *p, s4 size)
+{
+	/* do nothing */
+}
+
+
 void *mem_alloc(s4 size)
 {
 	if (size == 0)
@@ -253,7 +288,7 @@ void *mem_realloc(void *src, s4 len1, s4 len2)
 
 	dst = realloc(src, len2);
 
-	if (!dst)
+	if (dst == NULL)
 		exceptions_throw_outofmemory_exit();
 
 	return dst;
@@ -318,8 +353,8 @@ void *dump_alloc(s4 size)
 
 #else /* !defined(DISABLE_DUMP) */
 
-	void     *m;
-	dumpinfo *di;
+	void       *m;
+	dumpinfo_t *di;
 
 	/* If no threads are used, the dumpinfo structure is a static structure   */
 	/* defined at the top of this file.                                       */
@@ -329,15 +364,15 @@ void *dump_alloc(s4 size)
 	if (size == 0)
 		return NULL;
 
-	size = ALIGN(size, ALIGNSIZE);
+	size = MEMORY_ALIGN(size, ALIGNSIZE);
 
 	if (di->useddumpsize + size > di->allocateddumpsize) {
-		dumpblock *newdumpblock;
+		dumpblock_t *newdumpblock;
 		s4         newdumpblocksize;
 
 		/* allocate a new dumplist structure */
 
-		newdumpblock = memory_checked_alloc(sizeof(dumpblock));
+		newdumpblock = memory_checked_alloc(sizeof(dumpblock_t));
 
 		/* If requested size is greater than the default, make the new dump   */
 		/* block as big as the size requested. Else use the default size.     */
@@ -445,23 +480,22 @@ void dump_release(s4 size)
 
 #else /* !defined(DISABLE_DUMP) */
 
-	dumpinfo *di;
+	dumpinfo_t *di;
 
 	/* If no threads are used, the dumpinfo structure is a static structure   */
 	/* defined at the top of this file.                                       */
 
 	di = DUMPINFO;
 
-	if (size < 0 || size > di->useddumpsize)
-		throw_cacao_exception_exit(string_java_lang_InternalError,
-								   "Illegal dump release size %d", size);
+	if ((size < 0) || (size > di->useddumpsize))
+		vm_abort("Illegal dump release size: %d", size);
 
 	/* reset the used dump size to the size specified */
 
 	di->useddumpsize = size;
 
 	while (di->currentdumpblock && di->allocateddumpsize - di->currentdumpblock->size >= di->useddumpsize) {
-		dumpblock *tmp = di->currentdumpblock;
+		dumpblock_t *tmp = di->currentdumpblock;
 
 		di->allocateddumpsize -= tmp->size;
 		di->currentdumpblock = tmp->prev;
@@ -498,14 +532,14 @@ s4 dump_size(void)
 
 #else /* !defined(DISABLE_DUMP) */
 
-	dumpinfo *di;
+	dumpinfo_t *di;
 
 	/* If no threads are used, the dumpinfo structure is a static structure   */
 	/* defined at the top of this file.                                       */
 
 	di = DUMPINFO;
 
-	if (!di)
+	if (di == NULL)
 		return 0;
 
 	return di->useddumpsize;
