@@ -28,7 +28,7 @@
 
    Changes: Christian Thalinger
 
-   $Id: memory.h 4840 2006-04-25 17:45:02Z edwin $
+   $Id: memory.h 5868 2006-10-30 11:21:36Z edwin $
 
 */
 
@@ -38,8 +38,8 @@
 
 /* forward typedefs ***********************************************************/
 
-typedef struct dumpblock dumpblock;
-typedef struct dumpinfo dumpinfo;
+typedef struct dumpblock_t dumpblock_t;
+typedef struct dumpinfo_t  dumpinfo_t;
 
 #include "config.h"
 
@@ -47,6 +47,34 @@ typedef struct dumpinfo dumpinfo;
 
 #include "vm/types.h"
 
+
+/* ATTENTION: We need to define dumpblock_t and dumpinfo_t before
+   internal includes, as we need dumpinfo_t as nested structure in
+   threadobject. */
+
+/* dumpblock ******************************************************************/
+
+#define DUMPBLOCKSIZE    2 << 13    /* 2 * 8192 bytes */
+#define ALIGNSIZE        8
+
+struct dumpblock_t {
+	dumpblock_t *prev;
+	u1          *dumpmem;
+	s4           size;
+};
+
+
+/* dumpinfo *******************************************************************/
+
+struct dumpinfo_t {
+	dumpblock_t *currentdumpblock;
+	s4           allocateddumpsize;
+	s4           useddumpsize;
+};
+
+
+/* internal includes **********************************************************/
+
 #include "mm/boehm.h"
 
 
@@ -90,8 +118,8 @@ These macros do the same except they operate on the dump area:
 
 Some more macros:
 
-	ALIGN (pos, size) ... make pos divisible by size. always returns an
-						  address >= pos.
+	MEMORY_ALIGN (pos, size) ... make pos divisible by size. always returns an
+                                 address >= pos.
 	                      
 	
 	OFFSET (s,el) ....... returns the offset of 'el' in structure 's' in bytes.
@@ -101,31 +129,8 @@ Some more macros:
 
 */
 
-
-#define DUMPBLOCKSIZE    2 << 13    /* 2 * 8192 bytes */
-#define ALIGNSIZE        8
-
-
-/* dumpblock ******************************************************************/
-
-struct dumpblock {
-	dumpblock *prev;
-	u1        *dumpmem;
-	s4         size;
-};
-
-
-/* dumpinfo *******************************************************************/
-
-struct dumpinfo {
-	dumpblock *currentdumpblock;
-	s4         allocateddumpsize;
-	s4         useddumpsize;
-};
-
-
-#define ALIGN(pos,size)       ((((pos) + (size) - 1) / (size)) * (size))
-#define PADDING(pos,size)     (ALIGN((pos),(size)) - (pos))
+#define MEMORY_ALIGN(pos,size) ((((pos) + (size) - 1) / (size)) * (size))
+#define PADDING(pos,size)     (MEMORY_ALIGN((pos),(size)) - (pos))
 #define OFFSET(s,el)          ((s4) ((ptrint) &(((s*) 0)->el)))
 
 #if !defined(DISABLE_GC)
@@ -163,7 +168,7 @@ struct dumpinfo {
 #define MMOVE(dest,src,type,num) memmove((dest), (src), sizeof(type) * (num))
 
 #define CNEW(type,num)        ((type *) memory_cnew(sizeof(type) * (num)))
-#define CFREE(ptr,num)        MFREE(ptr,u1,num)
+#define CFREE(ptr,num)        memory_cfree((ptr),(num))
 
 
 /* GC macros ******************************************************************/
@@ -184,6 +189,7 @@ struct dumpinfo {
 bool memory_init(void);
 
 void *memory_cnew(s4 size);
+void  memory_cfree(void *p, s4 size);
 
 void *mem_alloc(s4 size);
 void  mem_free(void *m, s4 size);
