@@ -30,7 +30,7 @@
 
    Changes: Christian Thalinger
 
-   $Id: native.c 4921 2006-05-15 14:24:36Z twisti $
+   $Id: native.c 5810 2006-10-20 13:54:54Z twisti $
 
 */
 
@@ -38,6 +38,7 @@
 #include "config.h"
 
 #include <assert.h>
+#include <ctype.h>
 
 #if !defined(WITH_STATIC_CLASSPATH)
 # include <ltdl.h>
@@ -49,6 +50,13 @@
 #include "native/jni.h"
 #include "native/native.h"
 #include "native/include/java_lang_Throwable.h"
+
+#if defined(ENABLE_THREADS)
+# include "threads/native/lock.h"
+#else
+# include "threads/none/lock.h"
+#endif
+
 #include "toolbox/logging.h"
 #include "vm/builtin.h"
 #include "vm/exceptions.h"
@@ -62,6 +70,10 @@
 #include "vm/jit/asmpart.h"
 #include "vm/jit/jit.h"
 
+#if defined(ENABLE_JVMTI)
+#include "native/jvmti/cacaodbg.h"
+#endif
+
 
 /* include table of native functions ******************************************/
 
@@ -72,6 +84,9 @@
 
 #include "native/include/gnu_classpath_VMStackWalker.h"
 #include "native/include/gnu_classpath_VMSystemProperties.h"
+#include "native/include/gnu_java_lang_management_VMClassLoadingMXBeanImpl.h"
+#include "native/include/gnu_java_lang_management_VMMemoryMXBeanImpl.h"
+#include "native/include/gnu_java_lang_management_VMRuntimeMXBeanImpl.h"
 #include "native/include/java_lang_Class.h"
 #include "native/include/java_lang_Object.h"
 #include "native/include/java_lang_VMClass.h"
@@ -82,11 +97,13 @@
 #include "native/include/java_lang_VMSystem.h"
 #include "native/include/java_lang_VMThread.h"
 #include "native/include/java_lang_VMThrowable.h"
+#include "native/include/java_lang_management_VMManagementFactory.h"
 #include "native/include/java_lang_reflect_Constructor.h"
 #include "native/include/java_lang_reflect_Field.h"
 #include "native/include/java_lang_reflect_Method.h"
 #include "native/include/java_lang_reflect_VMProxy.h"
 #include "native/include/java_security_VMAccessController.h"
+
 #if defined(ENABLE_JVMTI)
 #include "native/include/gnu_classpath_jdwp_event_EventRequest.h"
 #include "native/include/java_nio_ByteBuffer.h"
@@ -117,16 +134,40 @@
 
 #include "native/nativetable.inc"
 
-#else /* defined(WITH_STATIC_CLASSPATH) */
+#elif !defined(ENABLE_LIBJVM)
 
-/* Ensure that symbols for functions implemented within CACAO are used
-   and exported to dlopen. */
+/* dummynativetable ************************************************************
 
-static functionptr dummynativetable[] = {
+   Ensure that symbols for functions implemented within CACAO are used
+   and exported to dlopen.
+
+   ATTENTION: Don't make this table static!!!  Otherwise the compiler
+   may optimize it away!
+
+*******************************************************************************/
+
+functionptr dummynativetable[] = {
 	(functionptr) Java_gnu_classpath_VMStackWalker_getClassContext,
+	(functionptr) Java_gnu_classpath_VMStackWalker_getCallingClass,
+	(functionptr) Java_gnu_classpath_VMStackWalker_getCallingClassLoader,
+	(functionptr) Java_gnu_classpath_VMStackWalker_firstNonNullClassLoader,
 
 	(functionptr) Java_gnu_classpath_VMSystemProperties_preInit,
 
+	(functionptr) Java_gnu_java_lang_management_VMClassLoadingMXBeanImpl_getLoadedClassCount,
+	(functionptr) Java_gnu_java_lang_management_VMClassLoadingMXBeanImpl_getUnloadedClassCount,
+	(functionptr) Java_gnu_java_lang_management_VMClassLoadingMXBeanImpl_isVerbose,
+	(functionptr) Java_gnu_java_lang_management_VMClassLoadingMXBeanImpl_setVerbose,
+
+	(functionptr) Java_gnu_java_lang_management_VMMemoryMXBeanImpl_getHeapMemoryUsage,
+	(functionptr) Java_gnu_java_lang_management_VMMemoryMXBeanImpl_getNonHeapMemoryUsage,
+	(functionptr) Java_gnu_java_lang_management_VMMemoryMXBeanImpl_getObjectPendingFinalizationCount,
+	(functionptr) Java_gnu_java_lang_management_VMMemoryMXBeanImpl_isVerbose,
+	(functionptr) Java_gnu_java_lang_management_VMMemoryMXBeanImpl_setVerbose,
+
+	(functionptr) Java_gnu_java_lang_management_VMRuntimeMXBeanImpl_getInputArguments,
+	(functionptr) Java_gnu_java_lang_management_VMRuntimeMXBeanImpl_getStartTime,
+
 	(functionptr) Java_java_lang_VMClass_isInstance,
 	(functionptr) Java_java_lang_VMClass_isAssignableFrom,
 	(functionptr) Java_java_lang_VMClass_isInterface,
@@ -193,6 +234,10 @@ static functionptr dummynativetable[] = {
 	(functionptr) Java_java_lang_VMThrowable_fillInStackTrace,
 	(functionptr) Java_java_lang_VMThrowable_getStackTrace,
 
+	(functionptr) Java_java_lang_management_VMManagementFactory_getMemoryPoolNames,
+	(functionptr) Java_java_lang_management_VMManagementFactory_getMemoryManagerNames,
+	(functionptr) Java_java_lang_management_VMManagementFactory_getGarbageCollectorNames,
+
 	(functionptr) Java_java_lang_reflect_Constructor_getModifiersInternal,
 	(functionptr) Java_java_lang_reflect_Constructor_constructNative,
 
@@ -228,6 +273,7 @@ static functionptr dummynativetable[] = {
 	(functionptr) Java_java_lang_reflect_VMProxy_generateProxyClass,
 
 	(functionptr) Java_java_security_VMAccessController_getStack,
+
 #if defined(ENABLE_JVMTI)
 	(functionptr) Java_gnu_classpath_jdwp_VMVirtualMachine_suspendThread,
 	(functionptr) Java_gnu_classpath_jdwp_VMVirtualMachine_resumeThread,
@@ -258,7 +304,7 @@ static functionptr dummynativetable[] = {
 
 };
 
-#endif /* defined(WITH_STATIC_CLASSPATH) */
+#endif /* defined(ENABLE_LIBJVM) */
 
 
 /* tables for methods *********************************************************/
@@ -282,36 +328,34 @@ static lt_dlhandle mainhandle;
 #endif
 
 
-/* native_loadclasses **********************************************************
+/* native_init *****************************************************************
 
-   Load classes required for native methods.
+   Initializes the native subsystem.
 
 *******************************************************************************/
 
 bool native_init(void)
 {
 #if !defined(WITH_STATIC_CLASSPATH)
-	void *p;
-
-	/* We need to access the dummy native table, not only to remove a
-	   warning but to be sure that the table is not optimized away
-	   (gcc does this since 3.4). */
-
-	p = &dummynativetable;
-
 	/* initialize libltdl */
 
-	if (lt_dlinit()) {
-		/* XXX how can we throw an exception here? */
-		log_text(lt_dlerror());
+	if (lt_dlinit())
+		vm_abort("lt_dlinit failed: %s\n", lt_dlerror());
 
-		return false;
-	}
+	/* Get the handle for the main program or for the libjvm.so,
+	   depends on the configuration. */
 
-	/* get the handle for the main program */
+# if defined(ENABLE_LIBJVM)
+	/* First try to open where dlopen searches, e.g. LD_LIBRARY_PATH.
+	   If not found, try the absolute path. */
 
+	if (!(mainhandle = lt_dlopenext("libjvm")))
+		if (!(mainhandle = lt_dlopenext(cacao_libjvm)))
+			vm_abort("lt_dlopenext failed: %s\n", lt_dlerror());
+# else
 	if (!(mainhandle = lt_dlopen(NULL)))
-		return false;
+		vm_abort("lt_dlopen failed: %s\n", lt_dlerror());
+# endif
 
 	/* initialize library hashtable, 10 entries should be enough */
 
@@ -341,9 +385,7 @@ void native_hashtable_library_add(utf *filename, java_objectheader *loader,
 	u4   key;                           /* hashkey                            */
 	u4   slot;                          /* slot in hashtable                  */
 
-#if defined(ENABLE_THREADS)
-	builtin_monitorenter(hashtable_library->header);
-#endif
+	LOCK_MONITOR_ENTER(hashtable_library->header);
 
 	/* normally addresses are aligned to 4, 8 or 16 bytes */
 
@@ -386,9 +428,7 @@ void native_hashtable_library_add(utf *filename, java_objectheader *loader,
 
 	while (ne) {
 		if (ne->name == filename) {
-#if defined(ENABLE_THREADS)
-			builtin_monitorexit(hashtable_library->header);
-#endif
+			LOCK_MONITOR_EXIT(hashtable_library->header);
 
 			return;
 		}
@@ -408,9 +448,7 @@ void native_hashtable_library_add(utf *filename, java_objectheader *loader,
 	ne->hashlink = le->namelink;
 	le->namelink = ne;
 
-#if defined(ENABLE_THREADS)
-	builtin_monitorexit(hashtable_library->header);
-#endif
+	LOCK_MONITOR_EXIT(hashtable_library->header);
 }
 #endif /* !defined(WITH_STATIC_CLASSPATH) */
 
@@ -631,6 +669,68 @@ static char *native_make_overloaded_function(char *name, utf *desc)
 }
 
 
+/* native_insert_char **********************************************************
+
+   Inserts the passed UTF character into the native method name.  If
+   necessary it is escaped properly.
+
+*******************************************************************************/
+
+static s4 native_insert_char(char *name, u4 pos, u2 c)
+{
+	s4 val;
+	s4 i;
+
+	switch (c) {
+	case '/':
+	case '.':
+		/* replace '/' or '.' with '_' */
+		name[pos] = '_';
+		break;
+
+	case '_':
+		/* escape sequence for '_' is '_1' */
+		name[pos]   = '_';
+		name[++pos] = '1';
+		break;
+
+	case ';':
+		/* escape sequence for ';' is '_2' */
+		name[pos]   = '_';
+		name[++pos] = '2';
+		break;
+
+	case '[':
+		/* escape sequence for '[' is '_1' */
+		name[pos]   = '_';
+		name[++pos] = '3';
+		break;
+
+	default:
+		if (isalnum(c))
+			name[pos] = c;
+		else {
+			/* unicode character */
+			name[pos]   = '_';
+			name[++pos] = '0';
+
+			for (i = 0; i < 4; ++i) {
+				val = c & 0x0f;
+				name[pos + 4 - i] = (val > 10) ? ('a' + val - 10) : ('0' + val);
+				c >>= 4;
+			}
+
+			pos += 4;
+		}
+		break;
+	}
+
+	/* return the new buffer index */
+
+	return pos;
+}
+
+
 /* native_resolve_function *****************************************************
 
    Resolves a native function, maybe from a dynamic library.
@@ -639,19 +739,19 @@ static char *native_make_overloaded_function(char *name, utf *desc)
 
 functionptr native_resolve_function(methodinfo *m)
 {
-	lt_ptr                     sym;
-	char                      *name;
-	char                      *newname;
-	s4                         namelen;
-	char                      *utf_ptr;
-	char                      *utf_endptr;
-	s4                         dumpsize;
+	lt_ptr                          sym;
+	char                           *name;
+	char                           *newname;
+	s4                              namelen;
+	char                           *utf_ptr;
+	char                           *utf_endptr;
+	u2                              c;
+	u4                              pos;
+	s4                              dumpsize;
 	hashtable_library_loader_entry *le;
 	hashtable_library_name_entry   *ne;
-	u4                         key;     /* hashkey                            */
-	u4                         slot;    /* slot in hashtable                  */
-	u4                         i;
-
+	u4                              key;    /* hashkey                        */
+	u4                              slot;   /* slot in hashtable              */
 
 	/* verbose output */
 
@@ -663,28 +763,15 @@ functionptr native_resolve_function(methodinfo *m)
 		printf(" ... ");
 	}
 		
-	/* calculate length of native function name */
+	/* Calculate length of native function name.  We multiply the
+	   class and method name length by 6 as this is the maxium
+	   escape-sequence that can be generated (unicode). */
 
-	namelen = strlen("Java_") + utf_get_number_of_u2s(m->class->name) + strlen("_") +
-		utf_get_number_of_u2s(m->name) + strlen("0");
-
-	/* check for underscores in class name */
-
-	utf_ptr = m->class->name->text;
-	utf_endptr = UTF_END(m->class->name);
-
-	while (utf_ptr < utf_endptr)
-		if (utf_nextu2(&utf_ptr) == '_')
-			namelen++;
-
-	/* check for underscores in method name */
-
-	utf_ptr = m->name->text;
-	utf_endptr = UTF_END(m->name);
-
-	while (utf_ptr < utf_endptr)
-		if (utf_nextu2(&utf_ptr) == '_')
-			namelen++;
+	namelen = strlen("Java_") +
+		utf_get_number_of_u2s(m->class->name) * 6 +
+		strlen("_") +
+		utf_get_number_of_u2s(m->name) * 6 +
+		strlen("0");
 
 	/* allocate memory */
 
@@ -692,49 +779,38 @@ functionptr native_resolve_function(methodinfo *m)
 
 	name = DMNEW(char, namelen);
 
-
 	/* generate name of native functions */
 
 	strcpy(name, "Java_");
-	i = strlen("Java_");
+	pos = strlen("Java_");
 
-	utf_ptr = m->class->name->text;
+	utf_ptr    = m->class->name->text;
 	utf_endptr = UTF_END(m->class->name);
 
-	for (; utf_ptr < utf_endptr; utf_ptr++, i++) {
-		name[i] = *utf_ptr;
-
-		/* escape sequence for '_' is '_1' */
-
-		if (name[i] == '_')
-			name[++i] = '1';
-
-		/* replace '/' with '_' */
-
-		if (name[i] == '/')
-			name[i] = '_';
+	for (; utf_ptr < utf_endptr; utf_ptr++, pos++) {
+		c   = *utf_ptr;
+		pos = native_insert_char(name, pos, c);
 	}
 
 	/* seperator between class and method */
 
-	name[i++] = '_';
+	name[pos++] = '_';
 
-	utf_ptr = m->name->text;
+	utf_ptr    = m->name->text;
 	utf_endptr = UTF_END(m->name);
 
-	for (; utf_ptr < utf_endptr; utf_ptr++, i++) {
-		name[i] = *utf_ptr;
-
-		/* escape sequence for '_' is '_1' */
-
-		if (name[i] == '_')
-			name[++i] = '1';
+	for (; utf_ptr < utf_endptr; utf_ptr++, pos++) {
+		c   = *utf_ptr;
+		pos = native_insert_char(name, pos, c);
 	}
 
 	/* close string */
 
-	name[i] = '\0';
+	name[pos] = '\0';
 
+	/* check for an buffer overflow */
+
+	assert(pos <= namelen);
 
 	/* generate overloaded function (having the types in it's name)           */
 
@@ -790,6 +866,11 @@ functionptr native_resolve_function(methodinfo *m)
 	}
 
 
+#if defined(ENABLE_JVMTI)
+	/* fire Native Method Bind event */
+	if (jvmti) jvmti_NativeMethodBind(m, sym, &sym);
+#endif
+
 	/* no symbol found? throw exception */
 
 	if (sym == NULL) {
