@@ -60,6 +60,10 @@
 #include "machine-instr.h"
 #endif
 
+#if defined(ENABLE_JVMTI)
+#include "native/jvmti/cacaodbg.h"
+#endif
+
 
 /******************************************************************************/
 /* DEBUGGING MACROS                                                           */
@@ -779,13 +783,25 @@ static void lock_inflate(threadobject *t, java_objectheader *o, lock_record_t *l
       t............the current thread
 	  o............the object of which to enter the monitor
 
+   RETURN VALUE:
+      true.........the lock has been successfully acquired
+	  false........an exception has been thrown
+
 *******************************************************************************/
 
-void lock_monitor_enter(threadobject *t, java_objectheader *o)
+bool lock_monitor_enter(java_objectheader *o)
 {
+	threadobject *t;
 	/* CAUTION: This code assumes that ptrint is unsigned! */
-	ptrint lockword;
-	ptrint thinlock;
+	ptrint        lockword;
+	ptrint        thinlock;
+
+	if (o == NULL) {
+		exceptions_throw_nullpointerexception();
+		return false;
+	}
+
+	t = THREADOBJECT;
 
 	thinlock = t->thinlock;
 
@@ -795,7 +811,7 @@ void lock_monitor_enter(threadobject *t, java_objectheader *o)
 		/* success. we locked it */
 		/* The Java Memory Model requires a memory barrier here: */
 		MEMORY_BARRIER();
-		return;
+		return true;
 	}
 
 	/* next common case: recursive lock with small recursion count */
@@ -813,7 +829,7 @@ void lock_monitor_enter(threadobject *t, java_objectheader *o)
 			o->monitorPtr = (lock_record_t *) (lockword + THIN_LOCK_COUNT_INCR);
 
 			/* success. we locked it */
-			return;
+			return true;
 		}
 		else {
 			lock_record_t *lr;
@@ -825,7 +841,7 @@ void lock_monitor_enter(threadobject *t, java_objectheader *o)
 			lock_inflate(t, o, lr);
 			lr->count++;
 
-			return;
+			return true;
 		}
 	}
 
@@ -841,7 +857,7 @@ void lock_monitor_enter(threadobject *t, java_objectheader *o)
 			/* check for recursive entering */
 			if (lr->owner == t) {
 				lr->count++;
-				return;
+				return true;
 			}
 
 			/* acquire the mutex of the lock record */
@@ -850,7 +866,7 @@ void lock_monitor_enter(threadobject *t, java_objectheader *o)
 
 			assert(lr->count == 0);
 
-			return;
+			return true;
 		}
 
 		/****** inflation path ******/
@@ -859,10 +875,20 @@ void lock_monitor_enter(threadobject *t, java_objectheader *o)
 
 		lr = lock_hashtable_get_lock_record(t, o);
 
+#if defined(ENABLE_JVMTI)
+        /* Monitor Contended Enter */
+		jvmti_MonitorContendedEntering(false, o);
+#endif
 		/* enter the monitor */
 
 		lock_record_enter(t, lr);
 
+
+#if defined(ENABLE_JVMTI)
+		/* Monitor Contended Entered */
+		jvmti_MonitorContendedEntering(true, o);
+#endif
+
 		/* inflation loop */
 
 		while (IS_THIN_LOCK(lockword = (ptrint) o->monitorPtr)) {
@@ -892,7 +918,7 @@ void lock_monitor_enter(threadobject *t, java_objectheader *o)
 
 		/* we own the inflated lock now */
 
-		return;
+		return true;
 	}
 }
 
@@ -915,10 +941,18 @@ void lock_monitor_enter(threadobject *t, java_objectheader *o)
 
 *******************************************************************************/
 
-bool lock_monitor_exit(threadobject *t, java_objectheader *o)
+bool lock_monitor_exit(java_objectheader *o)
 {
-	ptrint lockword;
-	ptrint thinlock;
+	threadobject *t;
+	ptrint        lockword;
+	ptrint        thinlock;
+
+	if (o == NULL) {
+		exceptions_throw_nullpointerexception();
+		return false;
+	}
+
+	t = THREADOBJECT;
 
 	/* We don't have to worry about stale values here, as any stale value */
 	/* will indicate that we don't own the lock.                          */
@@ -984,7 +1018,7 @@ bool lock_monitor_exit(threadobject *t, java_objectheader *o)
 		/* will be != t and thus fail this check.                             */
 
 		if (lr->owner != t) {
-			*exceptionptr = new_illegalmonitorstateexception();
+			exceptions_throw_illegalmonitorstateexception();
 			return false;
 		}
 
@@ -1007,7 +1041,8 @@ bool lock_monitor_exit(threadobject *t, java_objectheader *o)
 
 	/* legal thin lock cases have been handled above, so this is an error */
 
-	*exceptionptr = new_illegalmonitorstateexception();
+	exceptions_throw_illegalmonitorstateexception();
+
 	return false;
 }
 
@@ -1141,7 +1176,7 @@ static void lock_monitor_wait(threadobject *t, java_objectheader *o, s8 millis,
 		lr = GET_FAT_LOCK(lockword);
 
 		if (lr->owner != t) {
-			*exceptionptr = new_illegalmonitorstateexception();
+			exceptions_throw_illegalmonitorstateexception();
 			return;
 		}
 	}
@@ -1149,7 +1184,7 @@ static void lock_monitor_wait(threadobject *t, java_objectheader *o, s8 millis,
 		/* it's a thin lock */
 
 		if (LOCK_WORD_WITHOUT_COUNT(lockword) != t->thinlock) {
-			*exceptionptr = new_illegalmonitorstateexception();
+			exceptions_throw_illegalmonitorstateexception();
 			return;
 		}
 
@@ -1239,7 +1274,7 @@ static void lock_monitor_notify(threadobject *t, java_objectheader *o, bool one)
 		lr = GET_FAT_LOCK(lockword);
 
 		if (lr->owner != t) {
-			*exceptionptr = new_illegalmonitorstateexception();
+			exceptions_throw_illegalmonitorstateexception();
 			return;
 		}
 	}
@@ -1247,7 +1282,7 @@ static void lock_monitor_notify(threadobject *t, java_objectheader *o, bool one)
 		/* it's a thin lock */
 
 		if (LOCK_WORD_WITHOUT_COUNT(lockword) != t->thinlock) {
-			*exceptionptr = new_illegalmonitorstateexception();
+			exceptions_throw_illegalmonitorstateexception();
 			return;
 		}
 
