@@ -29,7 +29,7 @@
    Changes: Christian Thalinger
    			Edwin Steiner
 
-   $Id: threads.c 4958 2006-05-26 11:57:20Z twisti $
+   $Id: threads.c 5870 2006-10-30 12:27:59Z twisti $
 
 */
 
@@ -90,6 +90,9 @@
 # include "boehm-gc/include/gc.h"
 #endif
 
+#if defined(ENABLE_JVMTI)
+#include "native/jvmti/cacaodbg.h"
+#endif
 
 #if defined(__DARWIN__)
 /* Darwin has no working semaphore implementation.  This one is taken
@@ -198,7 +201,7 @@ static s4 threads_table_add(threadobject *thread);
 static void threads_table_remove(threadobject *thread);
 static void threads_calc_absolute_time(struct timespec *tm, s8 millis, s4 nanos);
 
-#if !defined(NDEBUG)
+#if !defined(NDEBUG) && 0
 static void threads_table_dump(FILE *file);
 #endif
 
@@ -209,6 +212,9 @@ static void threads_table_dump(FILE *file);
 /* the main thread                                                            */
 threadobject *mainthreadobj;
 
+static methodinfo *method_thread_init;
+static methodinfo *method_threadgroup_add;
+
 /* the thread object of the current thread                                    */
 /* This is either a thread-local variable defined with __thread, or           */
 /* a thread-specific value stored with key threads_current_threadobject_key.  */
@@ -222,7 +228,7 @@ pthread_key_t threads_current_threadobject_key;
 static threads_table_t threads_table;
 
 /* global compiler mutex                                                      */
-static pthread_mutex_rec_t compiler_mutex;
+static pthread_mutex_t compiler_mutex;
 
 /* global mutex for changing the thread list                                  */
 static pthread_mutex_t threadlistlock;
@@ -251,56 +257,6 @@ pthread_mutex_t _mb_lock = PTHREAD_MUTEX_INITIALIZER;
 #endif
 
 
-/******************************************************************************/
-/* Recursive Mutex Implementation for Darwin                                  */
-/******************************************************************************/
-
-#if defined(MUTEXSIM)
-
-/* We need this for older MacOSX (10.1.x) */
-
-void pthread_mutex_init_rec(pthread_mutex_rec_t *m)
-{
-	pthread_mutex_init(&m->mutex, NULL);
-	m->count = 0;
-}
-
-void pthread_mutex_destroy_rec(pthread_mutex_rec_t *m)
-{
-	pthread_mutex_destroy(&m->mutex);
-}
-
-void pthread_mutex_lock_rec(pthread_mutex_rec_t *m)
-{
-	for (;;) {
-		if (!m->count)
-		{
-			pthread_mutex_lock(&m->mutex);
-			m->owner = pthread_self();
-			m->count++;
-			break;
-		}
-		else {
-			if (m->owner != pthread_self()) {
-				pthread_mutex_lock(&m->mutex);
-			}
-			else {
-				m->count++;
-				break;
-			}
-		}
-	}
-}
-
-void pthread_mutex_unlock_rec(pthread_mutex_rec_t *m)
-{
-	if (!--m->count)
-		pthread_mutex_unlock(&m->mutex);
-}
-
-#endif /* defined(MUTEXSIM) */
-
-
 /* threads_sem_init ************************************************************
  
    Initialize a semaphore. Checks against errors and interruptions.
@@ -410,7 +366,7 @@ static void threads_set_thread_priority(pthread_t tid, int priority)
 
 void compiler_lock(void)
 {
-	pthread_mutex_lock_rec(&compiler_mutex);
+	pthread_mutex_lock(&compiler_mutex);
 }
 
 
@@ -422,7 +378,7 @@ void compiler_lock(void)
 
 void compiler_unlock(void)
 {
-	pthread_mutex_unlock_rec(&compiler_mutex);
+	pthread_mutex_unlock(&compiler_mutex);
 }
 
 
@@ -491,34 +447,36 @@ static void threads_cast_darwinstop(void)
 	do {
 		if (tobj != self)
 		{
-			thread_state_flavor_t flavor = PPC_THREAD_STATE;
-			mach_msg_type_number_t thread_state_count = PPC_THREAD_STATE_COUNT;
+			thread_state_flavor_t flavor = MACHINE_THREAD_STATE;
+			mach_msg_type_number_t thread_state_count = MACHINE_THREAD_STATE_COUNT;
+#if defined(__I386__)
+			i386_thread_state_t thread_state;
+#else
 			ppc_thread_state_t thread_state;
+#endif
 			mach_port_t thread = tobj->mach_thread;
 			kern_return_t r;
 
 			r = thread_suspend(thread);
-			if (r != KERN_SUCCESS) {
-				log_text("thread_suspend failed");
-				assert(0);
-			}
 
-			r = thread_get_state(thread, flavor,
-				(natural_t*)&thread_state, &thread_state_count);
-			if (r != KERN_SUCCESS) {
-				log_text("thread_get_state failed");
-				assert(0);
-			}
+			if (r != KERN_SUCCESS)
+				vm_abort("thread_suspend failed");
 
-			thread_restartcriticalsection(&thread_state);
+			r = thread_get_state(thread, flavor, (natural_t *) &thread_state,
+								 &thread_state_count);
 
-			r = thread_set_state(thread, flavor,
-				(natural_t*)&thread_state, thread_state_count);
-			if (r != KERN_SUCCESS) {
-				log_text("thread_set_state failed");
-				assert(0);
-			}
+			if (r != KERN_SUCCESS)
+				vm_abort("thread_get_state failed");
+
+			thread_restartcriticalsection((ucontext_t *) &thread_state);
+
+			r = thread_set_state(thread, flavor, (natural_t *) &thread_state,
+								 thread_state_count);
+
+			if (r != KERN_SUCCESS)
+				vm_abort("thread_set_state failed");
 		}
+
 		tobj = tobj->next;
 	} while (tobj != mainthreadobj);
 }
@@ -535,11 +493,11 @@ static void threads_cast_darwinresume(void)
 			kern_return_t r;
 
 			r = thread_resume(thread);
-			if (r != KERN_SUCCESS) {
-				log_text("thread_resume failed");
-				assert(0);
-			}
+
+			if (r != KERN_SUCCESS)
+				vm_abort("thread_resume failed");
 		}
+
 		tobj = tobj->next;
 	} while (tobj != mainthreadobj);
 }
@@ -557,16 +515,24 @@ static void threads_cast_irixresume(void)
 
 void threads_cast_stopworld(void)
 {
+#if !defined(__DARWIN__) && !defined(__CYGWIN__)
 	int count, i;
+#endif
+
 	lock_stopworld(STOPWORLD_FROM_CLASS_NUMBERING);
 	pthread_mutex_lock(&threadlistlock);
+
 #if defined(__DARWIN__)
 	threads_cast_darwinstop();
+#elif defined(__CYGWIN__)
+	/* TODO */
+	assert(0);
 #else
 	count = threads_cast_sendsignals(GC_signum1(), 0);
-	for (i=0; i<count; i++)
+	for (i = 0; i < count; i++)
 		threads_sem_wait(&suspend_ack);
 #endif
+
 	pthread_mutex_unlock(&threadlistlock);
 }
 
@@ -577,6 +543,9 @@ void threads_cast_startworld(void)
 	threads_cast_darwinresume();
 #elif defined(__MIPS__)
 	threads_cast_irixresume();
+#elif defined(__CYGWIN__)
+	/* TODO */
+	assert(0);
 #else
 	threads_cast_sendsignals(GC_signum2(), -1);
 #endif
@@ -602,6 +571,9 @@ static void threads_sigsuspend_handler(ucontext_t *ctx)
 	threads_sem_post(&suspend_ack);
 	pthread_cond_wait(&suspend_cond, &suspend_ack_lock);
 	pthread_mutex_unlock(&suspend_ack_lock);
+#elif defined(__CYGWIN__)
+	/* TODO */
+	assert(0);
 #else
 	threads_sem_post(&suspend_ack);
 
@@ -624,6 +596,7 @@ int cacao_suspendhandler(ucontext_t *ctx)
 }
 #endif
 
+
 /* threads_set_current_threadobject ********************************************
 
    Set the current thread object.
@@ -662,30 +635,25 @@ threadobject *threads_get_current_threadobject(void)
 
    Do some early initialization of stuff required.
 
+   ATTENTION: Do NOT use any Java heap allocation here, as gc_init()
+   is called AFTER this function!
+
 *******************************************************************************/
 
 void threads_preinit(void)
 {
-#ifndef MUTEXSIM
 	pthread_mutexattr_t mutexattr;
 	pthread_mutexattr_init(&mutexattr);
 	pthread_mutexattr_settype(&mutexattr, PTHREAD_MUTEX_RECURSIVE);
 	pthread_mutex_init(&compiler_mutex, &mutexattr);
 	pthread_mutexattr_destroy(&mutexattr);
-#else
-	pthread_mutex_init_rec(&compiler_mutex);
-#endif
 
 	pthread_mutex_init(&threadlistlock, NULL);
 	pthread_mutex_init(&stopworldlock, NULL);
 
-	/* Allocate something so the garbage collector's signal handlers
-	   are installed. */
-	heap_allocate(1, false, NULL);
-
 	mainthreadobj = NEW(threadobject);
-	mainthreadobj->tid = pthread_self();
-	mainthreadobj->index = 1;
+	mainthreadobj->tid      = pthread_self();
+	mainthreadobj->index    = 1;
 	mainthreadobj->thinlock = lock_pre_compute_thinlock(mainthreadobj->index);
 	
 #if !defined(HAVE___THREAD)
@@ -719,7 +687,6 @@ bool threads_init(void)
 	java_lang_Thread      *mainthread;
 	java_lang_ThreadGroup *threadgroup;
 	threadobject          *tempthread;
-	methodinfo            *method;
 
 	tempthread = mainthreadobj;
 
@@ -733,6 +700,28 @@ bool threads_init(void)
 
 	class_java_lang_VMThread->instancesize = sizeof(threadobject);
 
+	/* get methods we need in this file */
+
+	method_thread_init =
+		class_resolveclassmethod(class_java_lang_Thread,
+								 utf_init,
+								 utf_new_char("(Ljava/lang/VMThread;Ljava/lang/String;IZ)V"),
+								 class_java_lang_Thread,
+								 true);
+
+	if (method_thread_init == NULL)
+		return false;
+
+	method_threadgroup_add =
+		class_resolveclassmethod(class_java_lang_ThreadGroup,
+								 utf_new_char("addThread"),
+								 utf_new_char("(Ljava/lang/Thread;)V"),
+								 class_java_lang_ThreadGroup,
+								 true);
+
+	if (method_threadgroup_add == NULL)
+		return false;
+
 	/* create a VMThread */
 
 	mainthreadobj = (threadobject *) builtin_new(class_java_lang_VMThread);
@@ -753,12 +742,16 @@ bool threads_init(void)
 
 	threads_table_add(mainthreadobj);
 
+	/* mark main thread as Java thread */
+
+	mainthreadobj->flags = THREAD_FLAG_JAVA;
+
 #if defined(ENABLE_INTRP)
 	/* create interpreter stack */
 
 	if (opt_intrp) {
 		MSET(intrp_main_stack, 0, u1, opt_stacksize);
-		mainthreadobj->_global_sp = intrp_main_stack + opt_stacksize;
+		mainthreadobj->_global_sp = (Cell*) (intrp_main_stack + opt_stacksize);
 	}
 #endif
 
@@ -783,17 +776,8 @@ bool threads_init(void)
 
 	/* call Thread.<init>(Ljava/lang/VMThread;Ljava/lang/String;IZ)V */
 
-	method = class_resolveclassmethod(class_java_lang_Thread,
-									  utf_init,
-									  utf_new_char("(Ljava/lang/VMThread;Ljava/lang/String;IZ)V"),
-									  class_java_lang_Thread,
-									  true);
-
-	if (method == NULL)
-		return false;
-
-	(void) vm_call_method(method, (java_objectheader *) mainthread,
-						  mainthreadobj, threadname, 5, false);
+	(void) vm_call_method(method_thread_init, (java_objectheader *) mainthread,
+						  mainthreadobj, threadname, NORM_PRIORITY, false);
 
 	if (*exceptionptr)
 		return false;
@@ -802,22 +786,14 @@ bool threads_init(void)
 
 	/* add mainthread to ThreadGroup */
 
-	method = class_resolveclassmethod(class_java_lang_ThreadGroup,
-									  utf_new_char("addThread"),
-									  utf_new_char("(Ljava/lang/Thread;)V"),
-									  class_java_lang_ThreadGroup,
-									  true);
-
-	if (method == NULL)
-		return false;
-
-	(void) vm_call_method(method, (java_objectheader *) threadgroup,
+	(void) vm_call_method(method_threadgroup_add, 
+						  (java_objectheader *) threadgroup,
 						  mainthread);
 
 	if (*exceptionptr)
 		return false;
 
-	threads_set_thread_priority(pthread_self(), 5);
+	threads_set_thread_priority(pthread_self(), NORM_PRIORITY);
 
 	/* initialize the thread attribute object */
 
@@ -990,6 +966,9 @@ void threads_init_threadobject(java_lang_VMThread *t)
 
    Thread startup function called by pthread_create.
 
+   Thread which have a startup.function != NULL are marked as internal
+   threads. All other threads are threated as normal Java threads.
+
    NOTE: This function is not called directly by pthread_create. The Boehm GC
          inserts its own GC_start_routine in between, which then calls
 		 threads_startup.
@@ -1076,71 +1055,60 @@ static void *threads_startup_thread(void *t)
 	/* set interpreter stack */
 
 	if (opt_intrp)
-		thread->_global_sp = (void *) (intrp_thread_stack + opt_stacksize);
+		thread->_global_sp = (Cell *) (intrp_thread_stack + opt_stacksize);
 #endif
 
-
-
 #if defined(ENABLE_JVMTI)
-	/* breakpoint for thread start event */
- 	__asm__("threadstart:");
-#endif
+	/* fire thread start event */
 
+	if (jvmti) 
+		jvmti_ThreadStartEnd(JVMTI_EVENT_THREAD_START);
+#endif
 
 	/* find and run the Thread.run()V method if no other function was passed */
 
 	if (function == NULL) {
+		/* this is a normal Java thread */
+
+		thread->flags |= THREAD_FLAG_JAVA;
+
 		method = class_resolveclassmethod(thread->o.header.vftbl->class,
 										  utf_run,
 										  utf_void__void,
 										  thread->o.header.vftbl->class,
 										  true);
 
-		if (!method)
+		if (method == NULL)
 			throw_exception();
 
-		(void) vm_call_method(method, (java_objectheader *) thread);
+		/* increase total started thread count */
 
+		_Jv_jvm->total_started_thread_count++;
+
+		(void) vm_call_method(method, (java_objectheader *) thread);
 	}
 	else {
+		/* this is an internal thread */
+
+		thread->flags |= THREAD_FLAG_INTERNAL;
+
+		/* increase total started thread count */
+
+		_Jv_jvm->total_started_thread_count++;
+
 		/* call passed function, e.g. finalizer_thread */
 
 		(function)();
 	}
 
 #if defined(ENABLE_JVMTI)
-	/* breakpoint for thread end event */
- 	__asm__("threadend:");
+	/* fire thread end event */
+
+	if (jvmti)
+		jvmti_ThreadStartEnd(JVMTI_EVENT_THREAD_END);
 #endif
 
-
-	/* Allow lock record pools to be used by other threads. They
-	   cannot be deleted so we'd better not waste them. */
-
-	/* XXX We have to find a new way to free lock records */
-	/*     with the new locking algorithm.                */
-	/* lock_record_free_pools(thread->ee.lockrecordpools); */
-
-	/* remove thread from thread list and threads table, do this inside a lock */
-
-	pthread_mutex_lock(&threadlistlock);
-
-	thread->next->prev = thread->prev;
-	thread->prev->next = thread->next;
-
-	threads_table_remove(thread);
-
-	pthread_mutex_unlock(&threadlistlock);
-
-	/* reset thread id (lock on joinmutex? TWISTI) */
-
-	pthread_mutex_lock(&thread->joinmutex);
-	thread->tid = 0;
-	pthread_mutex_unlock(&thread->joinmutex);
-
-	/* tell everyone that a thread has finished */
-
-	pthread_cond_broadcast(&thread->joincond);
+	threads_detach_thread(thread);
 
 	return NULL;
 }
@@ -1208,6 +1176,174 @@ void threads_start_thread(java_lang_Thread *t, functionptr function)
 }
 
 
+/* threads_attach_current_thread ***********************************************
+
+   Attaches the current thread to the VM.  Used in JNI.
+
+*******************************************************************************/
+
+bool threads_attach_current_thread(JavaVMAttachArgs *vm_aargs, bool isdaemon)
+{
+	threadobject          *thread;
+	java_lang_Thread      *t;
+	utf                   *u;
+	java_lang_String      *s;
+	java_lang_ThreadGroup *group;
+	java_objectheader     *o;
+
+	/* create a java.lang.VMThread object */
+
+	thread = (threadobject *) builtin_new(class_java_lang_VMThread);
+
+	if (thread == NULL)
+		return false;
+
+	threads_init_threadobject(&thread->o);
+	threads_set_current_threadobject(thread);
+	lock_init_execution_env(thread);
+
+	/* insert the thread into the threadlist and the threads table */
+
+	pthread_mutex_lock(&threadlistlock);
+
+	thread->prev        = mainthreadobj;
+	thread->next        = mainthreadobj->next;
+	mainthreadobj->next = thread;
+	thread->next->prev  = thread;
+
+	threads_table_add(thread);
+
+	pthread_mutex_unlock(&threadlistlock);
+
+	/* mark main thread as Java thread */
+
+	thread->flags = THREAD_FLAG_JAVA;
+
+#if defined(ENABLE_INTRP)
+	/* create interpreter stack */
+
+	if (opt_intrp) {
+		MSET(intrp_main_stack, 0, u1, opt_stacksize);
+		thread->_global_sp = (Cell *) (intrp_main_stack + opt_stacksize);
+	}
+#endif
+
+	/* create a java.lang.Thread object */
+
+	t = (java_lang_Thread *) builtin_new(class_java_lang_Thread);
+
+	if (t == NULL)
+		return false;
+
+	thread->o.thread = t;
+
+	if (vm_aargs != NULL) {
+		u     = utf_new_char(vm_aargs->name);
+		group = (java_lang_ThreadGroup *) vm_aargs->group;
+	}
+	else {
+		u     = utf_null;
+		group = mainthreadobj->o.thread->group;
+	}
+
+	s = javastring_new(u);
+
+	o = (java_objectheader *) t;
+
+	(void) vm_call_method(method_thread_init, o, thread, s, NORM_PRIORITY,
+						  isdaemon);
+
+	if (*exceptionptr)
+		return false;
+
+	/* store the thread group in the object */
+
+	t->group = group;
+
+	o = (java_objectheader *) group;
+
+	(void) vm_call_method(method_threadgroup_add, o, t);
+
+	if (*exceptionptr)
+		return false;
+
+	return true;
+}
+
+
+/* threads_detach_thread *******************************************************
+
+   Detaches the passed thread from the VM.  Used in JNI.
+
+*******************************************************************************/
+
+bool threads_detach_thread(threadobject *thread)
+{
+	java_lang_Thread      *t;
+	java_lang_ThreadGroup *group;
+	methodinfo            *m;
+	java_objectheader     *o;
+
+	/* Allow lock record pools to be used by other threads. They
+	   cannot be deleted so we'd better not waste them. */
+
+	/* XXX We have to find a new way to free lock records */
+	/*     with the new locking algorithm.                */
+	/* lock_record_free_pools(thread->ee.lockrecordpools); */
+
+	/* XXX implement uncaught exception stuff (like JamVM does) */
+
+	/* remove thread from the thread group */
+
+	t     = thread->o.thread;
+	group = t->group;
+
+	/* XXX TWISTI: should all threads be in a ThreadGroup? */
+
+	if (group != NULL) {
+		m = class_resolveclassmethod(group->header.vftbl->class,
+									 utf_removeThread,
+									 utf_java_lang_Thread__V,
+									 class_java_lang_ThreadGroup,
+									 true);
+
+		if (m == NULL)
+			return false;
+
+		o = (java_objectheader *) group;
+
+		(void) vm_call_method(m, o, t);
+
+		if (*exceptionptr)
+			return false;
+	}
+
+	/* remove thread from thread list and threads table, do this
+	   inside a lock */
+
+	pthread_mutex_lock(&threadlistlock);
+
+	thread->next->prev = thread->prev;
+	thread->prev->next = thread->next;
+
+	threads_table_remove(thread);
+
+	pthread_mutex_unlock(&threadlistlock);
+
+	/* reset thread id (lock on joinmutex? TWISTI) */
+
+	pthread_mutex_lock(&thread->joinmutex);
+	thread->tid = 0;
+	pthread_mutex_unlock(&thread->joinmutex);
+
+	/* tell everyone that a thread has finished */
+
+	pthread_cond_broadcast(&thread->joincond);
+
+	return true;
+}
+
+
 /* threads_find_non_daemon_thread **********************************************
 
    Helper function used by threads_join_all_threads for finding
@@ -1443,7 +1579,7 @@ static void threads_calc_absolute_time(struct timespec *tm, s8 millis, s4 nanos)
 }
 
 
-/* threads_interrupt_thread ****************************************************
+/* threads_thread_interrupt ****************************************************
 
    Interrupt the given thread.
 
@@ -1455,17 +1591,26 @@ static void threads_calc_absolute_time(struct timespec *tm, s8 millis, s4 nanos)
 
 *******************************************************************************/
 
-void threads_interrupt_thread(java_lang_VMThread *thread)
+void threads_thread_interrupt(java_lang_VMThread *thread)
 {
-	threadobject *t = (threadobject*) thread;
+	threadobject *t;
 
-	/* signal the thread a "waitcond" and tell it that it has been */
-	/* interrupted                                                 */
+	t = (threadobject *) thread;
+
+	/* Signal the thread a "waitcond" and tell it that it has been
+	   interrupted. */
 
 	pthread_mutex_lock(&t->waitmutex);
+
+	/* Interrupt blocking system call using a signal. */
+
+	pthread_kill(t->tid, SIGHUP);
+
 	if (t->sleeping)
 		pthread_cond_signal(&t->waitcond);
+
 	t->interrupted = true;
+
 	pthread_mutex_unlock(&t->waitmutex);
 }
 
@@ -1540,7 +1685,7 @@ void threads_sleep(s8 millis, s4 nanos)
 }
 
 
-/* threads_yield *****************************************************************
+/* threads_yield ***************************************************************
 
    Yield to the scheduler.
 
@@ -1552,7 +1697,7 @@ void threads_yield(void)
 }
 
 
-/* threads_java_lang_Thread_set_priority ***********************************************************
+/* threads_java_lang_Thread_set_priority ***************************************
 
    Set the priority for the given java.lang.Thread.
 
@@ -1581,12 +1726,14 @@ void threads_java_lang_Thread_set_priority(java_lang_Thread *t, s4 priority)
 
 void threads_dump(void)
 {
-	threadobject       *tobj;
+	threadobject       *thread;
 	java_lang_VMThread *vmt;
 	java_lang_Thread   *t;
 	utf                *name;
 
-	tobj = mainthreadobj;
+	thread = mainthreadobj;
+
+	/* XXX we should stop the world here */
 
 	printf("Full thread dump CACAO "VERSION":\n");
 
@@ -1595,12 +1742,12 @@ void threads_dump(void)
 	do {
 		/* get thread objects */
 
-		vmt = &tobj->o;
+		vmt = &thread->o;
 		t   = vmt->thread;
 
 		/* the thread may be currently in initalization, don't print it */
 
-		if (t) {
+		if (t != NULL) {
 			/* get thread name */
 
 			name = javastring_toutf(t->name, false);
@@ -1613,22 +1760,18 @@ void threads_dump(void)
 				printf("daemon ");
 
 #if SIZEOF_VOID_P == 8
-			printf("prio=%d tid=0x%016lx\n", t->priority, tobj->tid);
+			printf("prio=%d tid=0x%016lx\n", t->priority, (long) thread->tid);
 #else
-			printf("prio=%d tid=0x%08lx\n", t->priority, tobj->tid);
+			printf("prio=%d tid=0x%08lx\n", t->priority, (long) thread->tid);
 #endif
 
-			/* send SIGUSR1 to thread to print stacktrace */
+			/* dump trace of thread */
 
-			pthread_kill(tobj->tid, SIGUSR1);
-
-			/* sleep this thread a bit, so the signal can reach the thread */
-
-			threads_sleep(10, 0);
+			stacktrace_dump_trace(thread);
 		}
 
-		tobj = tobj->next;
-	} while (tobj && (tobj != mainthreadobj));
+		thread = thread->next;
+	} while ((thread != NULL) && (thread != mainthreadobj));
 }
 
 
@@ -1641,7 +1784,7 @@ void threads_dump(void)
 
 ******************************************************************************/
 
-#if !defined(NDEBUG)
+#if !defined(NDEBUG) && 0
 static void threads_table_dump(FILE *file)
 {
 	s4 i;
@@ -1660,7 +1803,7 @@ static void threads_table_dump(FILE *file)
 		fprintf(file, "%4d: ", i);
 
 		if (index < size) {
-			fprintf(file, "free, nextfree = %d\n", index);
+			fprintf(file, "free, nextfree = %d\n", (int) index);
 		}
 		else {
 			fprintf(file, "thread %p\n", (void*) threads_table.table[i].thread);
@@ -1673,20 +1816,6 @@ static void threads_table_dump(FILE *file)
 }
 #endif
 
-
-#if defined(ENABLE_JVMTI)
-/* jvmti_get_threads_breakpoints ***********************************************
-
-   gets all breakpoints from this file
-
-*******************************************************************************/
-
-void jvmti_get_threads_breakpoints(void **brks) {
-	__asm__ ("movl $threadstart,%0;" :"=m"(brks[0]));
-	__asm__ ("movl $threadend,%0;" :"=m"(brks[1]));
-}
-#endif
-
 /*
  * These are local overrides for various environment variables in Emacs.
  * Please do not remove this and leave it at the end of the file, where
