@@ -29,7 +29,7 @@
 
    Changes: Christian Thalinger
 
-   $Id: threads.h 4953 2006-05-25 12:28:51Z twisti $
+   $Id: threads.h 5870 2006-10-30 12:27:59Z twisti $
 
 */
 
@@ -37,6 +37,13 @@
 #ifndef _THREADS_H
 #define _THREADS_H
 
+/* forward typedefs ***********************************************************/
+
+typedef struct threadobject          threadobject;
+typedef union  threads_table_entry_t threads_table_entry_t;
+typedef struct threads_table_t       threads_table_t;
+
+
 #include "config.h"
 
 #include <pthread.h>
@@ -44,9 +51,6 @@
 
 #include "vm/types.h"
 
-#include "config.h"
-#include "vm/types.h"
-
 #include "mm/memory.h"
 #include "native/jni.h"
 #include "native/include/java_lang_Object.h" /* required by java/lang/VMThread*/
@@ -56,12 +60,13 @@
 
 #include "threads/native/lock.h"
 
+#if defined(ENABLE_INTRP)
+#include "vm/jit/intrp/intrp.h"
+#endif
+
 #if defined(__DARWIN__)
 # include <mach/mach.h>
 
-/* We need to emulate recursive mutexes. */
-# define MUTEXSIM
-
 typedef struct {
 	pthread_mutex_t mutex;
 	pthread_cond_t cond;
@@ -73,11 +78,11 @@ typedef struct {
 #endif
 
 
-/* forward typedefs ***********************************************************/
+/* thread priorities **********************************************************/
 
-typedef struct threadobject          threadobject;
-typedef union  threads_table_entry_t threads_table_entry_t;
-typedef struct threads_table_t       threads_table_t;
+#define MIN_PRIORITY     1
+#define NORM_PRIORITY    5
+#define MAX_PRIORITY     10
 
 
 /* current threadobject *******************************************************/
@@ -133,6 +138,10 @@ struct threads_table_t {
 
 *******************************************************************************/
 
+#define THREAD_FLAG_JAVA        0x01    /* a normal Java thread               */
+#define THREAD_FLAG_INTERNAL    0x02    /* CACAO internal thread              */
+
+
 struct threadobject {
 	java_lang_VMThread    o;            /* the java.lang.VMThread object      */
 
@@ -143,9 +152,10 @@ struct threadobject {
 
 	ptrint                thinlock;     /* pre-computed thin lock value       */
 
-	s4                    index;        /* thread index, startin with 1       */
+	s4                    index;        /* thread index, starting with 1      */
+	u1                    flags;        /* flag field                         */
 
-	pthread_t             tid;               /* pthread id                    */
+	pthread_t             tid;          /* pthread id                         */
 
 #if defined(__DARWIN__)
 	mach_port_t           mach_thread;       /* Darwin thread id              */
@@ -162,15 +172,17 @@ struct threadobject {
 	bool                  signaled;
 	bool                  sleeping;
 
+	u1                   *pc;           /* current PC (used for profiling)    */
+
 	java_objectheader    *_exceptionptr;     /* current exception             */
 	stackframeinfo       *_stackframeinfo;   /* current native stackframeinfo */
 	localref_table       *_localref_table;   /* JNI local references          */
 
 #if defined(ENABLE_INTRP)
-	u1                   *_global_sp;        /* stack pointer for interpreter */
+	Cell                 *_global_sp;        /* stack pointer for interpreter */
 #endif
 
-	dumpinfo              dumpinfo;     /* dump memory info structure         */
+	dumpinfo_t            dumpinfo;     /* dump memory info structure         */
 };
 
 
@@ -194,6 +206,9 @@ void threads_init_threadobject(java_lang_VMThread *);
 
 void threads_start_thread(java_lang_Thread *t, functionptr function);
 
+bool threads_attach_current_thread(JavaVMAttachArgs *vm_aargs, bool isdaemon);
+bool threads_detach_thread(threadobject *thread);
+
 void threads_join_all_threads(void);
 
 void threads_sleep(s8 millis, s4 nanos);
@@ -201,7 +216,7 @@ void threads_yield(void);
 
 bool threads_wait_with_timeout_relative(threadobject *t, s8 millis, s4 nanos);
 
-void threads_interrupt_thread(java_lang_VMThread *);
+void threads_thread_interrupt(java_lang_VMThread *);
 bool threads_check_if_interrupted_and_reset(void);
 bool threads_thread_has_been_interrupted(java_lang_VMThread *);
 
@@ -212,36 +227,6 @@ void threads_cast_startworld(void);
 
 void threads_dump(void);
 
-#if defined(ENABLE_JVMTI)
-void jvmti_get_threads_breakpoints(void **brks);
-#endif
-/******************************************************************************/
-/* Recursive Mutex Implementation for Darwin                                  */
-/******************************************************************************/
-
-#if defined(MUTEXSIM)
-
-/* We need this for older MacOSX (10.1.x) */
-
-typedef struct {
-	pthread_mutex_t mutex;
-	pthread_t owner;
-	int count;
-} pthread_mutex_rec_t;
-
-void pthread_mutex_init_rec(pthread_mutex_rec_t *m);
-void pthread_mutex_destroy_rec(pthread_mutex_rec_t *m);
-void pthread_mutex_lock_rec(pthread_mutex_rec_t *m);
-void pthread_mutex_unlock_rec(pthread_mutex_rec_t *m);
-
-#else /* !defined(MUTEXSIM) */
-
-#define pthread_mutex_lock_rec pthread_mutex_lock
-#define pthread_mutex_unlock_rec pthread_mutex_unlock
-#define pthread_mutex_rec_t pthread_mutex_t
-
-#endif /* defined(MUTEXSIM) */
-
 #endif /* _THREADS_H */
 
 
