@@ -28,11 +28,13 @@
 
    Changes:
 
-   $Id: util.c 4960 2006-05-26 12:19:43Z edwin $
+   $Id: util.c 5252 2006-08-18 13:07:21Z twisti $
 
 */
 
 
+#include "config.h"
+
 #include <assert.h>
 #include <errno.h>
 #include <stdarg.h>
@@ -43,9 +45,10 @@
 #include "mm/memory.h"
 #include "vm/exceptions.h"
 #include "vm/stringlocal.h"
+#include "vm/vm.h"
 
 
-/* getcwd **********************************************************************
+/* _Jv_getcwd ******************************************************************
 
    Return the current working directory.
 
@@ -73,8 +76,7 @@ char *_Jv_getcwd(void)
 		/* too small buffer or a more serious problem */
 
 		if (errno != ERANGE)
-			throw_cacao_exception_exit(string_java_lang_InternalError,
-									   strerror(errno));
+			vm_abort("getcwd failed: %s", strerror(errno));
 
 		/* double the buffer size */
 
