@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: access.c 4435 2006-02-04 23:59:54Z twisti $
+   $Id: access.c 5846 2006-10-28 13:02:19Z edwin $
 
 */
 
@@ -42,22 +42,10 @@
 #include "vm/access.h"
 #include "vm/builtin.h"
 #include "vm/class.h"
+#include "vm/exceptions.h"
+#include "vm/stringlocal.h"
 
 
-/****************************************************************************/
-/* DEBUG HELPERS                                                            */
-/****************************************************************************/
-
-#ifdef NDEBUG
-#define ACCESS_DEBUG
-#endif
-
-#ifdef ACCESS_DEBUG
-#define ACCESS_ASSERT(cond)  assert(cond)
-#else
-#define ACCESS_ASSERT(cond)
-#endif
-
 /****************************************************************************/
 /* ACCESS CHECKS                                                            */
 /****************************************************************************/
@@ -83,8 +71,8 @@
 
 bool access_is_accessible_class(classinfo *referer, classinfo *cls)
 {
-	ACCESS_ASSERT(referer);
-	ACCESS_ASSERT(cls);
+	assert(referer);
+	assert(cls);
 
 	/* public classes are always accessible */
 	if (cls->flags & ACC_PUBLIC)
@@ -128,8 +116,8 @@ bool access_is_accessible_class(classinfo *referer, classinfo *cls)
 bool access_is_accessible_member(classinfo *referer, classinfo *declarer,
 								 s4 memberflags)
 {
-	ACCESS_ASSERT(referer);
-	ACCESS_ASSERT(declarer);
+	assert(referer);
+	assert(declarer);
 	
 	/* public members are accessible */
 	if (memberflags & ACC_PUBLIC)
@@ -154,7 +142,7 @@ bool access_is_accessible_member(classinfo *referer, classinfo *declarer,
 	/* {the member is protected and declarer is in another package} */
 
 	/* a necessary condition for access is that referer is a subclass of declarer */
-	ACCESS_ASSERT(referer->linked && declarer->linked);
+	assert((referer->state & CLASS_LINKED) && (declarer->state & CLASS_LINKED));
 	if (builtin_isanysubclass(referer,declarer))
 		return true;
 
@@ -162,6 +150,64 @@ bool access_is_accessible_member(classinfo *referer, classinfo *declarer,
 }
 
 
+/* access_check_caller *********************************************************
+ 
+   Check if the (indirect) caller has access rights to a member.
+  
+   IN:
+       declarer.........the class declaring the member
+       memberflags......the access flags of the member
+	   calldepth........number of callers to ignore
+	                    For example if the stacktrace looks like this:
+
+					   java.lang.reflect.Method.invokeNative (Native Method)
+				   [0] java.lang.reflect.Method.invoke (Method.java:329)
+				   [1] <caller>
+
+				        you must specify 1 so the access rights of <caller> 
+						are checked.
+  
+   RETURN VALUE:
+       true.............access permitted
+       false............access denied, an exception has been thrown
+   
+*******************************************************************************/
+
+bool access_check_caller(classinfo *declarer, s4 memberflags, s4 calldepth)
+{
+	java_objectarray *oa;
+	classinfo        *callerclass;
+
+	/* if everything is public, there is nothing to check */
+
+	if ((declarer->flags & ACC_PUBLIC) && (memberflags & ACC_PUBLIC))
+		return true;
+
+	/* get the caller's class */
+
+	oa = stacktrace_getClassContext();
+	if (!oa)
+		return false;
+
+	assert(calldepth >= 0 && calldepth < oa->header.size);
+
+	callerclass = (classinfo *) oa->data[calldepth];
+
+	/* check access rights */
+
+	if (!access_is_accessible_class(callerclass, declarer)
+		|| !access_is_accessible_member(callerclass, declarer, memberflags))
+	{
+		*exceptionptr =
+			new_exception(string_java_lang_IllegalAccessException);
+		return false;
+	}
+
+	/* access granted */
+
+	return true;
+}
+
 /*
  * These are local overrides for various environment variables in Emacs.
  * Please do not remove this and leave it at the end of the file, where
