@@ -37,7 +37,7 @@
    calls instead of machine instructions, using the C calling
    convention.
 
-   $Id: builtin.c 4960 2006-05-26 12:19:43Z edwin $
+   $Id: builtin.c 5862 2006-10-30 00:45:30Z edwin $
 
 */
 
@@ -45,9 +45,11 @@
 #include "config.h"
 
 #include <assert.h>
+#include <errno.h>
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/time.h>
 
 #include "vm/types.h"
 
@@ -55,6 +57,9 @@
 #include "md-abi.h"
 
 #include "fdlibm/fdlibm.h"
+#if defined(__CYGWIN__) && defined(Bias)
+# undef Bias
+#endif
 
 #include "mm/boehm.h"
 #include "mm/memory.h"
@@ -88,8 +93,6 @@
 #include "vm/builtintable.inc"
 
 
-CYCLES_STATS_DECLARE(builtin_monitorenter,100,2)
-CYCLES_STATS_DECLARE(builtin_monitorexit ,100,2)
 CYCLES_STATS_DECLARE(builtin_new         ,100,5)
 CYCLES_STATS_DECLARE(builtin_overhead    , 80,1)
 
@@ -102,12 +105,9 @@ CYCLES_STATS_DECLARE(builtin_overhead    , 80,1)
 
 static bool builtintable_init(void)
 {
-	descriptor_pool *descpool;
-	s4               dumpsize;
-	utf             *descriptor;
-	s4               entries_internal;
-	s4               entries_automatic;
-	s4               i;
+	descriptor_pool    *descpool;
+	s4                  dumpsize;
+	builtintable_entry *bte;
 
 	/* mark start of dump memory area */
 
@@ -125,23 +125,14 @@ static bool builtintable_init(void)
 	if (!descriptor_pool_add_class(descpool, utf_java_lang_Class))
 		return false;
 
-	/* calculate table entries statically */
-
-	entries_internal =
-		sizeof(builtintable_internal) / sizeof(builtintable_entry);
-
-	entries_automatic =
-		sizeof(builtintable_automatic) / sizeof(builtintable_entry)
-		- 1; /* last filler entry (comment see builtintable.inc) */
-
 	/* first add all descriptors to the pool */
 
-	for (i = 0; i < entries_internal; i++) {
+	for (bte = builtintable_internal; bte->fp != NULL; bte++) {
 		/* create a utf8 string from descriptor */
 
-		descriptor = utf_new_char(builtintable_internal[i].descriptor);
+		bte->descriptor = utf_new_char(bte->cdescriptor);
 
-		if (!descriptor_pool_add(descpool, descriptor, NULL)) {
+		if (!descriptor_pool_add(descpool, bte->descriptor, NULL)) {
 			/* release dump area */
 
 			dump_release(dumpsize);
@@ -150,16 +141,22 @@ static bool builtintable_init(void)
 		}
 	}
 
-	for (i = 0; i < entries_automatic; i++) {
-		/* create a utf8 string from descriptor */
-
-		descriptor = utf_new_char(builtintable_automatic[i].descriptor);
-
-		if (!descriptor_pool_add(descpool, descriptor, NULL)) {
-			/* release dump area */
+	for (bte = builtintable_automatic; bte->fp != NULL; bte++) {
+		bte->descriptor = utf_new_char(bte->cdescriptor);
 
+		if (!descriptor_pool_add(descpool, bte->descriptor, NULL)) {
 			dump_release(dumpsize);
+			return false;
+		}
+	}
 
+	for (bte = builtintable_function; bte->fp != NULL; bte++) {
+		bte->classname  = utf_new_char(bte->cclassname);
+		bte->name       = utf_new_char(bte->cname);
+		bte->descriptor = utf_new_char(bte->cdescriptor);
+
+		if (!descriptor_pool_add(descpool, bte->descriptor, NULL)) {
+			dump_release(dumpsize);
 			return false;
 		}
 	}
@@ -174,28 +171,24 @@ static bool builtintable_init(void)
 
 	/* now parse all descriptors */
 
-	for (i = 0; i < entries_internal; i++) {
-		/* create a utf8 string from descriptor */
-
-		descriptor = utf_new_char(builtintable_internal[i].descriptor);
-
+	for (bte = builtintable_internal; bte->fp != NULL; bte++) {
 		/* parse the descriptor, builtin is always static (no `this' pointer) */
 
-		builtintable_internal[i].md =
-			descriptor_pool_parse_method_descriptor(descpool, descriptor,
-													ACC_STATIC, NULL);
+		bte->md = descriptor_pool_parse_method_descriptor(descpool,
+														  bte->descriptor,
+														  ACC_STATIC, NULL);
 	}
 
-	for (i = 0; i < entries_automatic; i++) {
-		/* create a utf8 string from descriptor */
+	for (bte = builtintable_automatic; bte->fp != NULL; bte++) {
+		bte->md = descriptor_pool_parse_method_descriptor(descpool,
+														  bte->descriptor,
+														  ACC_STATIC, NULL);
+	}
 
-		descriptor = utf_new_char(builtintable_automatic[i].descriptor);
-
-		/* parse the descriptor, builtin is always static (no `this' pointer) */
-
-		builtintable_automatic[i].md =
-			descriptor_pool_parse_method_descriptor(descpool, descriptor,
-													ACC_STATIC, NULL);
+	for (bte = builtintable_function; bte->fp != NULL; bte++) {
+		bte->md = descriptor_pool_parse_method_descriptor(descpool,
+														  bte->descriptor,
+														  ACC_STATIC, NULL);
 	}
 
 	/* release dump area */
@@ -273,11 +266,11 @@ bool builtin_init(void)
 
 builtintable_entry *builtintable_get_internal(functionptr fp)
 {
-	s4 i;
+	builtintable_entry *bte;
 
-	for (i = 0; builtintable_internal[i].fp != NULL; i++) {
-		if (builtintable_internal[i].fp == fp)
-			return &builtintable_internal[i];
+	for (bte = builtintable_internal; bte->fp != NULL; bte++) {
+		if (bte->fp == fp)
+			return bte;
 	}
 
 	return NULL;
@@ -321,6 +314,59 @@ builtintable_entry *builtintable_get_automatic(s4 opcode)
 }
 
 
+/* builtintable_replace_function ***********************************************
+
+   XXX
+
+*******************************************************************************/
+
+bool builtintable_replace_function(instruction *iptr)
+{
+	constant_FMIref    *mr;
+	builtintable_entry *bte;
+
+	/* get name and descriptor of the function */
+
+	switch (iptr->opc) {
+	case ICMD_INVOKESTATIC:
+		/* The instruction MUST be resolved, otherwise we run into
+		   lazy loading troubles.  Anyway, we should/can only replace
+		   very VM-close functions. */
+
+		if (INSTRUCTION_IS_UNRESOLVED(iptr))
+			return false;
+
+		mr = iptr->sx.s23.s3.fmiref;
+		break;	
+
+	default:
+		return false;
+	}
+
+	/* search the function table */
+
+	for (bte = builtintable_function; bte->fp != NULL; bte++) {
+		if ((METHODREF_CLASSNAME(mr) == bte->classname) &&
+			(mr->name             == bte->name) &&
+			(mr->descriptor       == bte->descriptor)) {
+
+			/* set the values in the instruction */
+
+			iptr->opc   = bte->opcode;
+			iptr->sx.s23.s3.bte = bte;
+			if (bte->checkexception)
+				iptr->flags.bits |= INS_FLAG_CHECK;
+			else
+				iptr->flags.bits &= ~INS_FLAG_CHECK;
+
+			return true;
+		}
+	}
+
+	return false;
+}
+
+
 /*****************************************************************************
 								TYPE CHECKS
 *****************************************************************************/
@@ -333,8 +379,9 @@ builtintable_entry *builtintable_get_automatic(s4 opcode)
 	are interpreted as super classes.
 	Return value:  1 ... sub is subclass of super
 				   0 ... otherwise
-					
-*****************************************************************************/					
+
+******************************************************************************/
+
 s4 builtin_isanysubclass(classinfo *sub, classinfo *super)
 {
 	s4 res;
@@ -348,6 +395,10 @@ s4 builtin_isanysubclass(classinfo *sub, classinfo *super)
 			(sub->vftbl->interfacetable[-super->index] != NULL);
 
 	} else {
+		/* java.lang.Object is the only super_class_ of any interface */
+		if (sub->flags & ACC_INTERFACE)
+			return (super == class_java_lang_Object);
+
 		ASM_GETCLASSVALUES_ATOMIC(super->vftbl, sub->vftbl, &classvalues);
 
 		res = (u4) (classvalues.sub_baseval - classvalues.super_baseval) <=
@@ -373,7 +424,12 @@ s4 builtin_isanysubclass_vftbl(vftbl_t *sub, vftbl_t *super)
 		/* super is an interface */
 		res = (sub->interfacetablelength > -base) &&
 			(sub->interfacetable[base] != NULL);
-	} else {
+	} 
+	else {
+		/* java.lang.Object is the only super_class_ of any interface */
+		if (classvalues.sub_baseval <= 0)
+			return classvalues.super_baseval == 1;
+
 	    res = (u4) (classvalues.sub_baseval - classvalues.super_baseval)
 			<= (u4) classvalues.super_diffval;
 	}
@@ -1269,6 +1325,125 @@ java_objectheader *builtin_trace_exception(java_objectheader *xptr,
 #endif /* !defined(NDEBUG) */
 
 
+/* builtin_print_argument ******************************************************
+
+   Prints arguments and return values for the call trace.
+
+*******************************************************************************/
+
+#if !defined(NDEBUG)
+static char *builtin_print_argument(char *logtext, s4 *logtextlen,
+									typedesc *paramtype, s8 value)
+{
+	imm_union          imu;
+	java_objectheader *o;
+	java_lang_String  *s;
+	classinfo         *c;
+	utf               *u;
+	u4                 len;
+
+	switch (paramtype->type) {
+	case TYPE_INT:
+		imu.i = (s4) value;
+		sprintf(logtext + strlen(logtext), "%d (0x%08x)", imu.i, imu.i);
+		break;
+
+	case TYPE_LNG:
+		imu.l = value;
+#if SIZEOF_VOID_P == 4
+		sprintf(logtext + strlen(logtext), "%lld (0x%016llx)", imu.l, imu.l);
+#else
+		sprintf(logtext + strlen(logtext), "%ld (0x%016lx)", imu.l, imu.l);
+#endif
+		break;
+
+	case TYPE_FLT:
+		imu.i = (s4) value;
+		sprintf(logtext + strlen(logtext), "%g (0x%08x)", imu.f, imu.i);
+		break;
+
+	case TYPE_DBL:
+		imu.l = value;
+#if SIZEOF_VOID_P == 4
+		sprintf(logtext + strlen(logtext), "%g (0x%016llx)", imu.d, imu.l);
+#else
+		sprintf(logtext + strlen(logtext), "%g (0x%016lx)", imu.d, imu.l);
+#endif
+		break;
+
+	case TYPE_ADR:
+#if SIZEOF_VOID_P == 4
+		sprintf(logtext + strlen(logtext), "0x%08x", (ptrint) value);
+#else
+		sprintf(logtext + strlen(logtext), "0x%016lx", (ptrint) value);
+#endif
+
+		/* cast to java.lang.Object */
+
+		o = (java_objectheader *) (ptrint) value;
+
+		/* check return argument for java.lang.Class or java.lang.String */
+
+		if (o != NULL) {
+			if (o->vftbl->class == class_java_lang_String) {
+				/* get java.lang.String object and the length of the
+				   string */
+
+				s = (java_lang_String *) o;
+
+				u = javastring_toutf(s, false);
+
+				len = strlen(" (String = \"") + utf_bytes(u) + strlen("\")");
+
+				/* realloc memory for string length */
+
+				logtext = DMREALLOC(logtext, char, *logtextlen, *logtextlen + len);
+				*logtextlen += len;
+
+				/* convert to utf8 string and strcat it to the logtext */
+
+				strcat(logtext, " (String = \"");
+				utf_cat(logtext, u);
+				strcat(logtext, "\")");
+			}
+			else {
+				if (o->vftbl->class == class_java_lang_Class) {
+					/* if the object returned is a java.lang.Class
+					   cast it to classinfo structure and get the name
+					   of the class */
+
+					c = (classinfo *) o;
+
+					u = c->name;
+				}
+				else {
+					/* if the object returned is not a java.lang.String or
+					   a java.lang.Class just print the name of the class */
+
+					u = o->vftbl->class->name;
+				}
+
+				len = strlen(" (Class = \"") + utf_bytes(u) + strlen("\")");
+
+				/* realloc memory for string length */
+
+				logtext = DMREALLOC(logtext, char, *logtextlen, *logtextlen + len);
+				*logtextlen += len;
+
+				/* strcat to the logtext */
+
+				strcat(logtext, " (Class = \"");
+				utf_cat_classname(logtext, u);
+				strcat(logtext, "\")");
+			}
+		}
+	}
+
+	return logtext;
+}
+#endif /* !defined(NDEBUG) */
+
+
 /* builtin_trace_args **********************************************************
 
    Print method call with arguments for -verbose:call.
@@ -1310,11 +1485,29 @@ void builtin_trace_args(s8 a0, s8 a1,
 		strlen(".") +
 		utf_bytes(m->name) +
 		utf_bytes(m->descriptor) +
-		strlen(" SYNCHRONIZED") + strlen("(") + strlen(")");
+		strlen("(") + strlen(")");
+
+	/* Actually it's not possible to have all flags printed, but:
+	   safety first! */
+
+	logtextlen =
+		strlen(" PUBLIC") +
+		strlen(" PRIVATE") +
+		strlen(" PROTECTED") +
+		strlen(" STATIC") +
+		strlen(" FINAL") +
+		strlen(" SYNCHRONIZED") +
+		strlen(" VOLATILE") +
+		strlen(" TRANSIENT") +
+		strlen(" NATIVE") +
+		strlen(" INTERFACE") +
+		strlen(" ABSTRACT");
 
 	/* add maximal argument length */
 
-	logtextlen += strlen("0x123456789abcdef0, 0x123456789abcdef0, 0x123456789abcdef0, 0x123456789abcdef0, 0x123456789abcdef0, 0x123456789abcdef0, 0x123456789abcdef0, 0x123456789abcdef0, ...(255)");
+	logtextlen +=
+		strlen("-9223372036854775808 (0x123456789abcdef0), ") * TRACE_ARGS_NUM +
+		strlen("...(255)");
 
 	/* allocate memory */
 
@@ -1353,172 +1546,69 @@ void builtin_trace_args(s8 a0, s8 a1,
 
 	strcat(logtext, "(");
 
-	/* xxxprintf ?Bug? an PowerPc Linux (rlwinm.inso)                */
-	/* Only Arguments in integer Registers are passed correctly here */
-	/* long longs spilled on Stack have an wrong offset of +4        */
-	/* So preliminary Bugfix: Only pass 3 params at once to sprintf  */
-	/* for SIZEOG_VOID_P == 4 && TRACE_ARGS_NUM == 8                 */
-	switch (md->paramcount) {
-	case 0:
-		break;
+	if (md->paramcount >= 1) {
+		logtext = builtin_print_argument(logtext, &logtextlen,
+										 &md->paramtypes[0], a0);
+	}
 
-#if SIZEOF_VOID_P == 4
-	case 1:
-		sprintf(logtext + strlen(logtext),
-				"0x%llx",
-				a0);
-		break;
+	if (md->paramcount >= 2) {
+		strcat(logtext, ", ");
 
-	case 2:
-		sprintf(logtext + strlen(logtext),
-				"0x%llx, 0x%llx",
-				a0, a1);
-		break;
+		logtext = builtin_print_argument(logtext, &logtextlen,
+										 &md->paramtypes[1], a1);
+	}
 
 #if TRACE_ARGS_NUM >= 4
-	case 3:
-		sprintf(logtext + strlen(logtext),
-				"0x%llx, 0x%llx, 0x%llx",
-				a0, a1, a2);
-		break;
+	if (md->paramcount >= 3) {
+		strcat(logtext, ", ");
 
-	case 4:
-		sprintf(logtext + strlen(logtext), "0x%llx, 0x%llx, 0x%llx"
-				, a0, a1, a2);
-		sprintf(logtext + strlen(logtext), ", 0x%llx", a3);
+		logtext = builtin_print_argument(logtext, &logtextlen,
+										 &md->paramtypes[2], a2);
+	}
 
-		break;
-#endif /* TRACE_ARGS_NUM >= 4 */
+	if (md->paramcount >= 4) {
+		strcat(logtext, ", ");
+
+		logtext = builtin_print_argument(logtext, &logtextlen,
+										 &md->paramtypes[3], a3);
+	}
+#endif
 
 #if TRACE_ARGS_NUM >= 6
-	case 5:
-		sprintf(logtext + strlen(logtext), "0x%llx, 0x%llx, 0x%llx"
-				, a0, a1, a2);
-		sprintf(logtext + strlen(logtext), ", 0x%llx, 0x%llx", a3, a4);
-		break;
+	if (md->paramcount >= 5) {
+		strcat(logtext, ", ");
 
+		logtext = builtin_print_argument(logtext, &logtextlen,
+										 &md->paramtypes[4], a4);
+	}
 
-	case 6:
-		sprintf(logtext + strlen(logtext), "0x%llx, 0x%llx, 0x%llx"
-				, a0, a1, a2);
-		sprintf(logtext + strlen(logtext), ", 0x%llx, 0x%llx, 0x%llx"
-				, a3, a4, a5);
-		break;
-#endif /* TRACE_ARGS_NUM >= 6 */
+	if (md->paramcount >= 6) {
+		strcat(logtext, ", ");
+
+		logtext = builtin_print_argument(logtext, &logtextlen,
+										 &md->paramtypes[5], a5);
+	}
+#endif
 
 #if TRACE_ARGS_NUM == 8
-	case 7:
-		sprintf(logtext + strlen(logtext), "0x%llx, 0x%llx, 0x%llx"
-				, a0, a1, a2);
-		sprintf(logtext + strlen(logtext), ", 0x%llx, 0x%llx, 0x%llx"
-				, a3, a4, a5);
-		sprintf(logtext + strlen(logtext), ", 0x%llx", a6);
-		break;
+	if (md->paramcount >= 7) {
+		strcat(logtext, ", ");
 
-	case 8:
-		sprintf(logtext + strlen(logtext), "0x%llx, 0x%llx, 0x%llx"
-				, a0, a1, a2);
-		sprintf(logtext + strlen(logtext), ", 0x%llx, 0x%llx, 0x%llx"
-				, a3, a4, a5);
-		sprintf(logtext + strlen(logtext), ", 0x%llx, 0x%llx", a6, a7);
-		break;
-#endif /* TRACE_ARGS_NUM == 8 */
+		logtext = builtin_print_argument(logtext, &logtextlen,
+										 &md->paramtypes[6], a6);
+	}
 
-	default:
-#if TRACE_ARGS_NUM == 2
-		sprintf(logtext + strlen(logtext), "0x%llx, 0x%llx, ...(%d)", a0, a1, md->paramcount - 2);
+	if (md->paramcount >= 8) {
+		strcat(logtext, ", ");
 
-#elif TRACE_ARGS_NUM == 4
-		sprintf(logtext + strlen(logtext),
-				"0x%llx, 0x%llx, 0x%llx, 0x%llx, ...(%d)",
-				a0, a1, a2, a3, md->paramcount - 4);
-
-#elif TRACE_ARGS_NUM == 6
-		sprintf(logtext + strlen(logtext),
-				"0x%llx, 0x%llx, 0x%llx, 0x%llx, 0x%llx, 0x%llx, ...(%d)",
-				a0, a1, a2, a3, a4, a5, md->paramcount - 6);
-
-#elif TRACE_ARGS_NUM == 8
-		sprintf(logtext + strlen(logtext),"0x%llx, 0x%llx, 0x%llx,"
-				, a0, a1, a2);
-		sprintf(logtext + strlen(logtext)," 0x%llx, 0x%llx, 0x%llx,"
-				, a3, a4, a5);
-		sprintf(logtext + strlen(logtext)," 0x%llx, 0x%llx, ...(%d)"
-				, a6, a7, md->paramcount - 8);
+		logtext = builtin_print_argument(logtext, &logtextlen,
+										 &md->paramtypes[7], a7);
+	}
 #endif
-		break;
 
-#else /* SIZEOF_VOID_P == 4 */
-
-	case 1:
-		sprintf(logtext + strlen(logtext),
-				"0x%lx",
-				a0);
-		break;
-
-	case 2:
-		sprintf(logtext + strlen(logtext),
-				"0x%lx, 0x%lx",
-				a0, a1);
-		break;
-
-	case 3:
-		sprintf(logtext + strlen(logtext),
-				"0x%lx, 0x%lx, 0x%lx", a0, a1, a2);
-		break;
-
-	case 4:
-		sprintf(logtext + strlen(logtext),
-				"0x%lx, 0x%lx, 0x%lx, 0x%lx",
-				a0, a1, a2, a3);
-		break;
-
-#if TRACE_ARGS_NUM >= 6
-	case 5:
-		sprintf(logtext + strlen(logtext),
-				"0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx",
-				a0, a1, a2, a3, a4);
-		break;
-
-	case 6:
-		sprintf(logtext + strlen(logtext),
-				"0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx",
-				a0, a1, a2, a3, a4, a5);
-		break;
-#endif /* TRACE_ARGS_NUM >= 6 */
-
-#if TRACE_ARGS_NUM == 8
-	case 7:
-		sprintf(logtext + strlen(logtext),
-				"0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx",
-				a0, a1, a2, a3, a4, a5, a6);
-		break;
-
-	case 8:
-		sprintf(logtext + strlen(logtext),
-				"0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx",
-				a0, a1, a2, a3, a4, a5, a6, a7);
-		break;
-#endif /* TRACE_ARGS_NUM == 8 */
-
-	default:
-#if TRACE_ARGS_NUM == 4
-		sprintf(logtext + strlen(logtext),
-				"0x%lx, 0x%lx, 0x%lx, 0x%lx, ...(%d)",
-				a0, a1, a2, a3, md->paramcount - 4);
-
-#elif TRACE_ARGS_NUM == 6
-		sprintf(logtext + strlen(logtext),
-				"0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, ...(%d)",
-				a0, a1, a2, a3, a4, a5, md->paramcount - 6);
-
-#elif TRACE_ARGS_NUM == 8
-		sprintf(logtext + strlen(logtext),
-				"0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, 0x%lx, ...(%d)",
-				a0, a1, a2, a3, a4, a5, a6, a7, md->paramcount - 8);
-#endif
-		break;
-#endif /* SIZEOF_VOID_P == 4 */
+	if (md->paramcount > 8) {
+		sprintf(logtext + strlen(logtext), ", ...(%d)",
+				md->paramcount - TRACE_ARGS_NUM);
 	}
 
 	strcat(logtext, ")");
@@ -1544,18 +1634,13 @@ void builtin_trace_args(s8 a0, s8 a1,
 #if !defined(NDEBUG)
 void builtin_displaymethodstop(methodinfo *m, s8 l, double d, float f)
 {
-	methoddesc  	  *md;
-	char        	  *logtext;
-	s4          	   logtextlen;
-	s4          	   dumpsize;
-	s4          	   i;
-	s4                 pos;
-	java_objectheader *o;
-	java_lang_String  *s;
-	classinfo         *c;
-	s4                 len;
-	utf               *u;
-	imm_union          imu;
+	methoddesc *md;
+	char       *logtext;
+	s4          logtextlen;
+	s4          dumpsize;
+	s4          i;
+	s4          pos;
+	imm_union   val;
 
 	md = m->parseddesc;
 
@@ -1605,92 +1690,27 @@ void builtin_displaymethodstop(methodinfo *m, s8 l, double d, float f)
 	utf_cat(logtext, m->name);
 	utf_cat(logtext, m->descriptor);
 
-	switch (md->returntype.type) {
-	case TYPE_INT:
-		sprintf(logtext + strlen(logtext), "->%d (0x%08x)", (s4) l, (s4) l);
-		break;
+	if (!IS_VOID_TYPE(md->returntype.type)) {
+		strcat(logtext, "->");
 
-	case TYPE_LNG:
-#if SIZEOF_VOID_P == 4
-		sprintf(logtext + strlen(logtext), "->%lld (0x%016llx)", (s8) l, l);
-#else
-		sprintf(logtext + strlen(logtext), "->%ld (0x%016lx)", (s8) l, l);
-#endif
-		break;
+		switch (md->returntype.type) {
+		case TYPE_INT:
+		case TYPE_LNG:
+		case TYPE_ADR:
+			val.l = l;
+			break;
 
-	case TYPE_ADR:
-		sprintf(logtext + strlen(logtext), "->%p", (void *) (ptrint) l);
+		case TYPE_FLT:
+			val.f = f;
+			break;
 
-		/* check return argument for java.lang.Class or java.lang.String */
-
-		o = (java_objectheader *) (ptrint) l;
-
-		if (o != NULL) {
-			if (o->vftbl->class == class_java_lang_String) {
-				/* get java.lang.String object and the length of the
-                   string */
-
-				s = (java_lang_String *) (ptrint) l;
-
-				u = javastring_toutf(s, false);
-
-				len = strlen(", String = \"") + utf_bytes(u) + strlen("\"");
-
-				/* realloc memory for string length */
-
-				DMREALLOC(logtext, char, logtextlen, logtextlen + len);
-
-				/* convert to utf8 string and strcat it to the logtext */
-
-				strcat(logtext, ", String = \"");
-				utf_cat(logtext, u);
-				strcat(logtext, "\"");
-
-			} else {
-				if (o->vftbl->class == class_java_lang_Class) {
-					/* if the object returned is a java.lang.Class
-					   cast it to classinfo structure and get the name
-					   of the class */
-
-					c = (classinfo *) (ptrint) l;
-
-					u = c->name;
-
-				} else {
-					/* if the object returned is not a java.lang.String or
-					   a java.lang.Class just print the name of the class */
-
-					u = o->vftbl->class->name;
-				}
-
-				len = strlen(", Class = \"") + utf_bytes(u) + strlen("\"");
-
-				/* realloc memory for string length */
-
-				DMREALLOC(logtext, char, logtextlen, logtextlen + len);
-
-				/* strcat to the logtext */
-
-				strcat(logtext, ", Class = \"");
-				utf_cat_classname(logtext, u);
-				strcat(logtext, "\"");
-			}
+		case TYPE_DBL:
+			val.d = d;
+			break;
 		}
-		break;
 
-	case TYPE_FLT:
-		imu.f = f;
-		sprintf(logtext + strlen(logtext), "->%.8f (0x%08x)", f, imu.i);
-		break;
-
-	case TYPE_DBL:
-		imu.d = d;
-#if SIZEOF_VOID_P == 4
-		sprintf(logtext + strlen(logtext), "->%.16g (0x%016llx)", d, imu.l);
-#else
-		sprintf(logtext + strlen(logtext), "->%.16g (0x%016lx)", d, imu.l);
-#endif
-		break;
+		logtext =
+			builtin_print_argument(logtext, &logtextlen, &md->returntype, val.l);
 	}
 
 	log_text(logtext);
@@ -1708,60 +1728,12 @@ void builtin_print_cycles_stats(FILE *file)
 	fprintf(file,"builtin cylce count statistics:\n");
 
 	CYCLES_STATS_PRINT_OVERHEAD(builtin_overhead,file);
-	CYCLES_STATS_PRINT(builtin_monitorenter,file);
-	CYCLES_STATS_PRINT(builtin_monitorexit ,file);
 	CYCLES_STATS_PRINT(builtin_new         ,file);
 
 	fprintf(file,"\n");
 }
 #endif /* defined(ENABLE_CYCLES_STATS) */
 
-#if defined(ENABLE_THREADS)
-void builtin_monitorenter(java_objectheader *o)
-{
-#if defined(ENABLE_CYCLES_STATS)
-	u8 cycles_start, cycles_overhead, cycles_end;
-#endif
-
-	CYCLES_STATS_GET(cycles_start);
-	CYCLES_STATS_GET(cycles_overhead);
-
-	lock_monitor_enter((threadobject *) THREADOBJECT, o);
-
-	CYCLES_STATS_GET(cycles_end);
-	CYCLES_STATS_COUNT(builtin_monitorenter, cycles_end - cycles_overhead);
-	CYCLES_STATS_COUNT(builtin_overhead    , cycles_overhead - cycles_start);
-}
-#endif
-
-
-#if defined(ENABLE_THREADS)
-/*
- * Locks the class object - needed for static synchronized methods.
- */
-void builtin_staticmonitorenter(classinfo *c)
-{
-	builtin_monitorenter(&c->object.header);
-}
-#endif
-
-
-#if defined(ENABLE_THREADS)
-void builtin_monitorexit(java_objectheader *o)
-{
-#if defined(ENABLE_CYCLES_STATS)
-	u8 cycles_start, cycles_end;
-#endif
-
-	CYCLES_STATS_GET(cycles_start);
-
-	lock_monitor_exit((threadobject *) THREADOBJECT, o);
-
-	CYCLES_STATS_GET(cycles_end);
-	CYCLES_STATS_COUNT(builtin_monitorexit, cycles_end - cycles_start);
-}
-#endif
-
 
 /*****************************************************************************
 			  MISCELLANEOUS HELPER FUNCTIONS
@@ -2553,6 +2525,118 @@ float builtin_d2f(double a)
 #endif /* !(SUPPORT_FLOAT && SUPPORT_DOUBLE) */
 
 
+/* builtin_arraycopy ***********************************************************
+
+   Builtin for java.lang.System.arraycopy.
+
+   ATTENTION: This builtin function returns a boolean value to signal
+   the ICMD_BUILTIN if there was an exception.
+
+*******************************************************************************/
+
+bool builtin_arraycopy(java_arrayheader *src, s4 srcStart,
+					   java_arrayheader *dest, s4 destStart, s4 len)
+{
+	arraydescriptor *sdesc;
+	arraydescriptor *ddesc;
+	s4               i;
+
+	if ((src == NULL) || (dest == NULL)) { 
+		exceptions_throw_nullpointerexception();
+		return false;
+	}
+
+	sdesc = src->objheader.vftbl->arraydesc;
+	ddesc = dest->objheader.vftbl->arraydesc;
+
+	if (!sdesc || !ddesc || (sdesc->arraytype != ddesc->arraytype)) {
+		exceptions_throw_arraystoreexception();
+		return false;
+	}
+
+	/* we try to throw exception with the same message as SUN does */
+
+	if ((len < 0) || (srcStart < 0) || (destStart < 0) ||
+		(srcStart  + len < 0) || (srcStart  + len > src->size) ||
+		(destStart + len < 0) || (destStart + len > dest->size)) {
+		exceptions_throw_arrayindexoutofboundsexception();
+		return false;
+	}
+
+	if (sdesc->componentvftbl == ddesc->componentvftbl) {
+		/* We copy primitive values or references of exactly the same type */
+
+		s4 dataoffset = sdesc->dataoffset;
+		s4 componentsize = sdesc->componentsize;
+
+		memmove(((u1 *) dest) + dataoffset + componentsize * destStart,
+				((u1 *) src)  + dataoffset + componentsize * srcStart,
+				(size_t) len * componentsize);
+	}
+	else {
+		/* We copy references of different type */
+
+		java_objectarray *oas = (java_objectarray *) src;
+		java_objectarray *oad = (java_objectarray *) dest;
+                
+		if (destStart <= srcStart) {
+			for (i = 0; i < len; i++) {
+				java_objectheader *o = oas->data[srcStart + i];
+
+				if (!builtin_canstore(oad, o)) {
+					exceptions_throw_arraystoreexception();
+					return false;
+				}
+
+				oad->data[destStart + i] = o;
+			}
+		}
+		else {
+			/* XXX this does not completely obey the specification!
+			   If an exception is thrown only the elements above the
+			   current index have been copied. The specification
+			   requires that only the elements *below* the current
+			   index have been copied before the throw. */
+
+			for (i = len - 1; i >= 0; i--) {
+				java_objectheader *o = oas->data[srcStart + i];
+
+				if (!builtin_canstore(oad, o)) {
+					exceptions_throw_arraystoreexception();
+					return false;
+				}
+
+				oad->data[destStart + i] = o;
+			}
+		}
+	}
+
+	return true;
+}
+
+
+/* builtin_currenttimemillis ***************************************************
+
+   Return the current time in milliseconds.
+
+*******************************************************************************/
+
+s8 builtin_currenttimemillis(void)
+{
+	struct timeval tv;
+	s8             result;
+
+	if (gettimeofday(&tv, NULL) == -1)
+		vm_abort("gettimeofday failed: %s", strerror(errno));
+
+	result = (s8) tv.tv_sec;
+	result *= 1000;
+	result += (tv.tv_usec / 1000);
+
+	return result;
+}
+
+
 /* builtin_clone_array *********************************************************
 
    Wrapper function for cloning arrays.
@@ -2572,20 +2656,6 @@ java_arrayheader *builtin_clone_array(void *env, java_arrayheader *o)
 }
 
 
-/* builtin_asm_get_exceptionptrptr *********************************************
-
-   this is a wrapper for calls from asmpart
-
-*******************************************************************************/
-
-#if defined(ENABLE_THREADS)
-java_objectheader **builtin_asm_get_exceptionptrptr(void)
-{
-	return builtin_get_exceptionptrptr();
-}
-#endif
-
-
 /*
  * These are local overrides for various environment variables in Emacs.
  * Please do not remove this and leave it at the end of the file, where
