@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: class.h 4582 2006-03-11 20:30:48Z edwin $
+   $Id: class.h 5492 2006-09-14 18:20:28Z edwin $
 
 */
 
@@ -164,6 +164,7 @@ extern classinfo *class_java_lang_String;
 extern classinfo *class_java_lang_System;
 extern classinfo *class_java_lang_Thread;
 extern classinfo *class_java_lang_ThreadGroup;
+extern classinfo *class_java_lang_VMSystem;
 extern classinfo *class_java_lang_VMThread;
 extern classinfo *class_java_io_Serializable;
 
@@ -173,12 +174,14 @@ extern classinfo *class_java_io_Serializable;
 extern classinfo *class_java_lang_Throwable;
 extern classinfo *class_java_lang_VMThrowable;
 extern classinfo *class_java_lang_Error;
-extern classinfo *class_java_lang_NoClassDefFoundError;
+extern classinfo *class_java_lang_AbstractMethodError;
 extern classinfo *class_java_lang_LinkageError;
+extern classinfo *class_java_lang_NoClassDefFoundError;
 extern classinfo *class_java_lang_NoSuchMethodError;
 extern classinfo *class_java_lang_OutOfMemoryError;
 
 extern classinfo *class_java_lang_Exception;
+extern classinfo *class_java_lang_ClassCastException;
 extern classinfo *class_java_lang_ClassNotFoundException;
 extern classinfo *class_java_lang_IllegalArgumentException;
 extern classinfo *class_java_lang_IllegalMonitorStateException;
@@ -309,6 +312,8 @@ void class_print(classinfo *c);
 void class_println(classinfo *c);
 void class_classref_print(constant_classref *cr);
 void class_classref_println(constant_classref *cr);
+void class_classref_or_classinfo_print(classref_or_classinfo c);
+void class_classref_or_classinfo_println(classref_or_classinfo c);
 #endif
 
 /* debug purposes */
