@@ -28,7 +28,7 @@
 
    Changes: Christian Thalinger
 
-   $Id: classcache.c 4945 2006-05-23 19:52:47Z motse $
+   $Id: classcache.c 5192 2006-07-31 14:21:15Z twisti $
 
 */
 
@@ -39,6 +39,11 @@
 #include <assert.h>
 
 #include "mm/memory.h"
+
+#if defined(ENABLE_THREADS)
+# include "threads/native/lock.h"
+#endif
+
 #include "vm/classcache.h"
 #include "vm/exceptions.h"
 #include "vm/hashtable.h"
@@ -213,8 +218,8 @@ void classcache_print_statistics(FILE *file) {
 	/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
 
 #if defined(ENABLE_THREADS)
-# define CLASSCACHE_LOCK()      builtin_monitorenter(lock_hashtable_classcache)
-# define CLASSCACHE_UNLOCK()    builtin_monitorexit(lock_hashtable_classcache)
+# define CLASSCACHE_LOCK()      LOCK_MONITOR_ENTER(lock_hashtable_classcache)
+# define CLASSCACHE_UNLOCK()    LOCK_MONITOR_EXIT(lock_hashtable_classcache)
 #else
 # define CLASSCACHE_LOCK()
 # define CLASSCACHE_UNLOCK()
@@ -227,12 +232,8 @@ void classcache_print_statistics(FILE *file) {
 hashtable hashtable_classcache;
 
 #if defined(ENABLE_THREADS)
-#if defined(ENABLE_JVMTI)
-java_objectheader *lock_hashtable_classcache;
-#else
 static java_objectheader *lock_hashtable_classcache;
 #endif
-#endif
 
 
 /*============================================================================*/
@@ -1329,6 +1330,131 @@ bool classcache_add_constraints_for_params(classloader * a,
 }
 #endif /* defined(ENABLE_VERIFIER) */
 
+
+/* classcache_number_of_loaded_classes *****************************************
+
+   Counts the number of loaded classes and returns it.
+
+   Note: This function assumes that the CLASSCACHE_LOCK is held by the
+   caller!
+
+*******************************************************************************/
+
+static s4 classcache_number_of_loaded_classes(void)
+{
+	classcache_name_entry  *en;
+	classcache_class_entry *clsen;
+	s4                      number;
+	s4                      i;
+
+	/* initialize class counter */
+
+	number = 0;
+
+	for (i = 0; i < hashtable_classcache.size; i++) {
+		/* iterate over hashlink */
+
+		for (en = hashtable_classcache.ptr[i]; en != NULL; en = en->hashlink) {
+			/* filter pseudo classes $NEW$, $NULL$, $ARRAYSTUB$ out */
+
+			if (en->name->text[0] == '$')
+				continue;
+
+			/* iterate over classes with same name */
+
+			for (clsen = en->classes; clsen != NULL; clsen = clsen->next) {
+				/* get only loaded classes */
+
+				if (clsen->classobj != NULL)
+					number++;
+			}
+		}
+	}
+
+	return number;
+}
+
+
+/* classcache_get_loaded_class_count *******************************************
+
+   Counts the number of loaded classes and returns it.
+
+*******************************************************************************/
+
+s4 classcache_get_loaded_class_count(void)
+{
+	s4 count;
+
+	CLASSCACHE_LOCK();
+
+	count = classcache_number_of_loaded_classes();
+	
+	CLASSCACHE_UNLOCK();
+
+	return count;
+}
+
+
+/* classcache_get_loaded_classes ***********************************************
+
+   Returns an array of all loaded classes as array.  The array is
+   allocaed on the Java heap.
+
+*******************************************************************************/
+
+#if defined(ENABLE_JVMTI)
+void classcache_get_loaded_classes(s4 *class_count_ptr,
+								   classinfo ***classes_ptr)
+{
+	classinfo              **classes;
+	s4                       class_count;
+	classcache_name_entry   *en;
+	classcache_class_entry  *clsen;
+	s4                       i;
+	s4                       j;
+
+	CLASSCACHE_LOCK();
+
+	/* get the number of loaded classes and allocate the array */
+
+	class_count = classcache_number_of_loaded_classes();
+
+	classes = GCMNEW(classinfo*, class_count);
+
+	/* look in every slot of the hashtable */
+
+	for (i = 0, j = 0; i < hashtable_classcache.size; i++) {
+		/* iterate over hashlink */
+
+		for (en = hashtable_classcache.ptr[i]; en != NULL; en = en->hashlink) {
+			/* filter pseudo classes $NEW$, $NULL$, $ARRAYSTUB$ out */
+
+			if (en->name->text[0] == '$')
+				continue;
+
+			/* iterate over classes with same name */
+
+			for (clsen = en->classes; clsen != NULL; clsen = clsen->next) {
+				/* get only loaded classes */
+
+				if (clsen->classobj != NULL) {
+					classes[j] = clsen->classobj;
+					j++;
+				}
+			}
+		}
+	}
+
+	/* pass the return values */
+
+	*class_count_ptr = class_count;
+	*classes_ptr     = classes;
+
+	CLASSCACHE_UNLOCK();
+}
+#endif /* defined(ENABLE_JVMTI) */
+
+
 /*============================================================================*/
 /* DEBUG DUMPS                                                                */
 /*============================================================================*/
