@@ -26,9 +26,9 @@
 
    Authors: Edwin Steiner
 
-   Changes:
+   Changes: Christian Thalinger
 
-   $Id: classcache.h 4957 2006-05-26 11:48:10Z edwin $
+   $Id: classcache.h 5192 2006-07-31 14:21:15Z twisti $
 
 */
 
@@ -41,6 +41,10 @@
 
 #include <stdio.h>  /* for FILE */
 
+#if defined(ENABLE_JVMTI)
+# include "native/jni.h"
+#endif
+
 #include "vm/hashtable.h"
 #include "vm/references.h"
 
@@ -57,17 +61,6 @@ typedef java_objectheader classloader;
 
 extern hashtable hashtable_classcache;
 
-#if defined(ENABLE_JVMTI)
-#if defined(ENABLE_THREADS)
-# define CLASSCACHE_LOCK()      builtin_monitorenter(lock_hashtable_classcache)
-# define CLASSCACHE_UNLOCK()    builtin_monitorexit(lock_hashtable_classcache)
-#else
-# define CLASSCACHE_LOCK()
-# define CLASSCACHE_UNLOCK()
-#endif
-
-extern java_objectheader *lock_hashtable_classcache;
-#endif 
 
 /* structs ********************************************************************/
 
@@ -150,6 +143,13 @@ bool classcache_add_constraints_for_params(classloader *a,classloader *b,
 										   methodinfo *m);
 #endif
 
+s4 classcache_get_loaded_class_count(void);
+
+#if defined(ENABLE_JVMTI)
+void classcache_get_loaded_classes(s4 *class_count_ptr,
+								   classinfo ***classes_ptr);
+#endif
+
 #ifndef NDEBUG
 void classcache_debug_dump(FILE *file,utf *only);
 #endif
