@@ -28,7 +28,7 @@
 
    Changes: Edwin Steiner
 
-   $Id: exceptions.h 4921 2006-05-15 14:24:36Z twisti $
+   $Id: exceptions.h 5785 2006-10-15 22:25:54Z edwin $
 
 */
 
@@ -38,8 +38,6 @@
 
 /* forward typedefs ***********************************************************/
 
-typedef struct exceptionentry exceptionentry;
-
 #include "config.h"
 #include "vm/types.h"
 
@@ -52,7 +50,7 @@ typedef struct exceptionentry exceptionentry;
 
 
 #if defined(ENABLE_THREADS)
-#define exceptionptr    builtin_get_exceptionptrptr()
+#define exceptionptr    &(THREADOBJECT->_exceptionptr)
 #else
 #define exceptionptr    &_no_threads_exceptionptr
 #endif
@@ -62,21 +60,6 @@ extern java_objectheader *_no_threads_exceptionptr;
 #endif
 
 
-/* exceptionentry **************************************************************
-
-   Datastructure which represents an exception entry in the exception
-   table residing in the data segment.
-
-*******************************************************************************/
-
-struct exceptionentry {
-	classref_or_classinfo  catchtype;
-	u1                    *handlerpc;
-	u1                    *endpc;
-	u1                    *startpc;
-};
-
-
 /* function prototypes ********************************************************/
 
 /* load and link exceptions used in the system */
@@ -118,6 +101,10 @@ java_objectheader *new_exception_int(const char *classname, s4 i);
 
 /* functions to generate compiler exceptions */
 
+java_objectheader *exceptions_new_abstractmethoderror(void);
+java_objectheader *exceptions_asm_new_abstractmethoderror(u1 *sp, u1 *ra);
+void exceptions_throw_abstractmethoderror(void);
+
 java_objectheader *new_classformaterror(classinfo *c, const char *message, ...);
 void exceptions_throw_classformaterror(classinfo *c, const char *message, ...);
 
@@ -130,7 +117,9 @@ void exceptions_throw_nosuchmethoderror(classinfo *c, utf *name, utf *desc);
 
 java_objectheader *new_internalerror(const char *message, ...);
 
-java_objectheader *new_verifyerror(methodinfo *m, const char *message, ...);
+java_objectheader *exceptions_new_verifyerror(methodinfo *m,
+											  const char *message, ...);
+void exceptions_throw_verifyerror(methodinfo *m, const char *message, ...);
 void exceptions_throw_verifyerror_for_stack(methodinfo *m, int type);
 
 java_objectheader *new_unsupportedclassversionerror(classinfo *c,
@@ -141,13 +130,16 @@ java_objectheader *new_arithmeticexception(void);
 java_objectheader *new_arrayindexoutofboundsexception(s4 index);
 void exceptions_throw_arrayindexoutofboundsexception(void);
 
-java_objectheader *new_arraystoreexception(void);
-java_objectheader *new_classcastexception(void);
+java_objectheader *exceptions_new_arraystoreexception(void);
+void exceptions_throw_arraystoreexception(void);
+
+java_objectheader *exceptions_new_classcastexception(java_objectheader *o);
 
 java_objectheader *new_illegalargumentexception(void);
 void exceptions_throw_illegalargumentexception(void);
 
-java_objectheader *new_illegalmonitorstateexception(void);
+java_objectheader *exceptions_new_illegalmonitorstateexception(void);
+void               exceptions_throw_illegalmonitorstateexception(void);
 
 java_objectheader *new_negativearraysizeexception(void);
 void exceptions_throw_negativearraysizeexception(void);
@@ -160,6 +152,8 @@ void exceptions_throw_stringindexoutofboundsexception(void);
 
 void classnotfoundexception_to_noclassdeffounderror(void);
 
+java_objectheader *exceptions_get_and_clear_exception(void);
+
 void exceptions_print_exception(java_objectheader *xptr);
 
 #endif /* _EXCEPTIONS_H */
