@@ -28,7 +28,7 @@
 
    Changes: Christian Ullrich
 
-   $Id: md-abi.c 4960 2006-05-26 12:19:43Z edwin $
+   $Id: md-abi.c 5635 2006-10-02 16:36:54Z christian $
 
 */
 
@@ -40,15 +40,24 @@
 
 #include "vm/descriptor.h"
 #include "vm/global.h"
+#include "vm/jit/abi.h"
 
 
 /* register descripton array **************************************************/
 
 s4 nregdescint[] = {
+	/*   v0,      t0,      t1,      t2,      t3,      t4,      t5,      t6,   */
 	REG_RET, REG_TMP, REG_TMP, REG_TMP, REG_TMP, REG_TMP, REG_TMP, REG_TMP, 
+
+	/*   t7,      s0,      s1,      s2,      s3,      s4,      s5,      s6,   */
 	REG_TMP, REG_SAV, REG_SAV, REG_SAV, REG_SAV, REG_SAV, REG_SAV, REG_SAV, 
+
+	/*   a0,      a1,      a2,      a3,      a4,      a5,      t8,      t9,   */
 	REG_ARG, REG_ARG, REG_ARG, REG_ARG, REG_ARG, REG_ARG, REG_TMP, REG_TMP,
+
+	/*  t10,   itmp1,      ra,      pv,      at,   itmp3,      sp,    zero,   */
 	REG_TMP, REG_RES, REG_RES, REG_RES, REG_RES, REG_RES, REG_RES, REG_RES,
+
 	REG_END
 };
 
@@ -94,13 +103,14 @@ void md_param_alloc(methoddesc *md)
 				pd->regoff = reguse;
 				reguse++;
 				md->argintreguse = reguse;
-
-			} else {
+			}
+			else {
 				pd->inmemory = true;
 				pd->regoff = stacksize;
 				stacksize++;
 			}
 			break;
+
 		case TYPE_FLT:
 		case TYPE_DBL:
 			if (i < FLT_ARG_CNT) {
@@ -108,7 +118,8 @@ void md_param_alloc(methoddesc *md)
 				pd->regoff = reguse;
 				reguse++;
 				md->argfltreguse = reguse;
-			} else {
+			}
+			else {
 				pd->inmemory = true;
 				pd->regoff = stacksize;
 				stacksize++;
@@ -132,36 +143,40 @@ void md_param_alloc(methoddesc *md)
    (SAVEDVAR)
 
    --- in
-   m:                       Methodinfo of current method
-   return_type:             Return Type of the Method (TYPE_INT.. TYPE_ADR)
-   							TYPE_VOID is not allowed!
+   jd:                      jitdata of the current method
    stackslot:               Java Stackslot to contain the Return Value
    
    --- out
    if precoloring was possible:
-   stackslot->varkind       =ARGVAR
-   			->varnum        =-1
-   			->flags         =0
-   			->regoff        =[REG_RESULT, REG_FRESULT]
+   VAR(stackslot->varnum)->flags       = PREALLOC
+   			             ->vv.regoff   = [REG_RESULT|REG_FRESULT]
+   rd->arg[flt|int]reguse   set to a value according the register usage
 
+   NOTE: Do not pass a LOCALVAR in stackslot->varnum.
 *******************************************************************************/
 
-void md_return_alloc(methodinfo *m, registerdata *rd, s4 return_type,
-					 stackptr stackslot)
+void md_return_alloc(jitdata *jd, stackptr stackslot)
 {
-	/* Only precolor the stackslot, if it is not a SAVEDVAR <-> has not   */
-	/* to survive method invokations */
+	methodinfo *m;
+	methoddesc *md;
+
+	/* get required compiler data */
+
+	m = jd->m;
+
+	md = m->parseddesc;
+
+	/* Only precolor the stackslot, if it is not a SAVEDVAR <-> has
+	   not to survive method invokations. */
 
 	if (!(stackslot->flags & SAVEDVAR)) {
-		stackslot->varkind = ARGVAR;
-		stackslot->varnum = -1;
-		stackslot->flags = 0;
 
-		if (IS_INT_LNG_TYPE(return_type)) {
-			stackslot->regoff = REG_RESULT;
-		} else { /* float/double */
-			stackslot->regoff = REG_FRESULT;
-		}
+		VAR(stackslot->varnum)->flags = PREALLOC;
+
+		if (IS_INT_LNG_TYPE(md->returntype.type))
+			VAR(stackslot->varnum)->vv.regoff = REG_RESULT;
+		else
+			VAR(stackslot->varnum)->vv.regoff = REG_FRESULT;
 	}
 }
 
