@@ -28,18 +28,22 @@
 
    Changes:
 
-   $Id: patcher.c 4634 2006-03-16 15:16:16Z twisti $
+   $Id: patcher.c 5290 2006-09-04 17:12:48Z christian $
 
 */
 
 
 #include "config.h"
+
+#include <assert.h>
+
 #include "vm/types.h"
 
 #include "mm/memory.h"
 #include "native/native.h"
 #include "vm/builtin.h"
 #include "vm/class.h"
+#include "vm/exceptions.h"
 #include "vm/field.h"
 #include "vm/initialize.h"
 #include "vm/options.h"
@@ -47,6 +51,301 @@
 #include "vm/resolve.h"
 #include "vm/jit/asmpart.h"
 #include "vm/jit/patcher.h"
+#include "vm/jit/methodheader.h"
+
+
+/* patcher_wrapper *************************************************************
+
+   Wrapper for all patchers.  It also creates the stackframe info
+   structure.
+
+   If the return value of the patcher function is false, it gets the
+   exception object, clears the exception pointer and returns the
+   exception.
+
+*******************************************************************************/
+
+java_objectheader *patcher_wrapper(u1 *sp, u1 *pv, u1 *ra)
+{
+	stackframeinfo     sfi;
+	u1                *xpc;
+	java_objectheader *o;
+	u4                 mcode;
+	functionptr        f;
+	bool               result;
+	java_objectheader *e;
+
+	/* define the patcher function */
+
+	bool (*patcher_function)(u1 *);
+
+	assert(pv != NULL);
+
+	/* get stuff from the stack */
+
+	xpc   = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
+	f     = (functionptr)         *((ptrint *) (sp + 0 * 8));
+
+	/* calculate and set the new return address */
+
+	xpc = xpc - 1 * 4;
+
+	*((ptrint *) (sp + 5 * 8)) = (ptrint) xpc;
+
+	/* store PV into the patcher function position */
+
+	*((ptrint *) (sp + 0 * 8)) = (ptrint) pv;
+
+	/* cast the passed function to a patcher function */
+
+	patcher_function = (bool (*)(u1 *)) (ptrint) f;
+
+	/* enter a monitor on the patching position */
+
+	PATCHER_MONITORENTER;
+
+	/* create the stackframeinfo */
+
+	stacktrace_create_extern_stackframeinfo(&sfi, pv, sp + 6 * 8, ra, xpc);
+
+	/* call the proper patcher function */
+
+	result = (patcher_function)(sp);
+
+	/* remove the stackframeinfo */
+
+	stacktrace_remove_stackframeinfo(&sfi);
+
+	/* check for an error, get the exception and return it */
+
+	if (result == false) {
+		e = exceptions_get_and_clear_exception();
+
+		PATCHER_MONITOREXIT;
+
+		return e;
+	}
+
+	/* patch back original code */
+
+	mcode = *((u4 *) (sp + 3 * 8));
+
+	*((u4 *) xpc) = mcode;
+
+	/* synchronize instruction cache */
+
+	md_icacheflush(NULL, 0);
+
+	PATCHER_MARK_PATCHED_MONITOREXIT;
+
+	return NULL;
+}
+
+/* patcher_initialize_class ****************************************************
+
+   Initalizes a given classinfo pointer.  This function does not patch
+   any data.
+
+*******************************************************************************/
+
+bool patcher_initialize_class(u1 *sp)
+{
+	classinfo *c;
+
+	/* get stuff from the stack */
+
+	c = (classinfo *) *((ptrint *) (sp + 2 * 8));
+
+	/* check if the class is initialized */
+
+	if (!(c->state & CLASS_INITIALIZED))
+		if (!initialize_class(c))
+			return false;
+
+	return true;
+}
+
+/* patcher_resolve_class *****************************************************
+
+   Initalizes a given classinfo pointer.  This function does not patch
+   any data.
+
+*******************************************************************************/
+
+#ifdef ENABLE_VERIFIER
+bool patcher_resolve_class(u1 *sp)
+{
+	unresolved_class *uc;
+	classinfo        *c;
+
+	/* get stuff from the stack */
+
+	uc = (unresolved_class *)  *((ptrint *) (sp + 2 * 8));
+
+	/* resolve the class */
+
+	if (!resolve_class(uc, resolveEager, false, &c))
+		return false;
+
+	return true;
+}
+#endif /* ENABLE_VERIFIER */
+
+
+/* patcher_resolve_classref_to_classinfo ***************************************
+
+   ACONST:
+
+   <patched call postition>
+   a61bff80    ldq     a0,-128(pv)
+
+   MULTIANEWARRAY:
+
+   <patched call position>
+   a63bff80    ldq     a1,-128(pv)
+   47de0412    mov     sp,a2
+   a77bff78    ldq     pv,-136(pv)
+   6b5b4000    jsr     (pv)
+
+   ARRAYCHECKCAST:
+
+   <patched call position>
+   a63bfe60    ldq     a1,-416(pv)
+   a77bfe58    ldq     pv,-424(pv)
+   6b5b4000    jsr     (pv)
+
+*******************************************************************************/
+
+bool patcher_resolve_classref_to_classinfo(u1 *sp)
+{
+	constant_classref *cr;
+	s4                 disp;
+	u1                *pv;
+	classinfo         *c;
+
+	/* get stuff from the stack */
+
+	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
+	disp  =                       *((s4 *)     (sp + 1 * 8));
+	pv    = (u1 *)                *((ptrint *) (sp + 0 * 8));
+
+	/* get the classinfo */
+
+	if (!(c = resolve_classref_eager(cr)))
+		return false;
+
+	/* patch the classinfo pointer */
+
+	*((ptrint *) (pv + disp)) = (ptrint) c;
+
+	return true;
+}
+
+
+/* patcher_resolve_classref_to_vftbl *******************************************
+
+   CHECKCAST (class):
+   INSTANCEOF (class):
+
+   <patched call position>
+   a7940000    ldq     at,0(a4)
+   a7bbff28    ldq     gp,-216(pv)
+
+*******************************************************************************/
+
+bool patcher_resolve_classref_to_vftbl(u1 *sp)
+{
+	constant_classref *cr;
+	s4                 disp;
+	u1                *pv;
+	classinfo         *c;
+
+	/* get stuff from the stack */
+
+	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
+	disp  =                       *((s4 *)     (sp + 1 * 8));
+	pv    = (u1 *)                *((ptrint *) (sp + 0 * 8));
+
+	/* get the fieldinfo */
+
+	if (!(c = resolve_classref_eager(cr)))
+		return false;
+
+	/* patch super class' vftbl */
+
+	*((ptrint *) (pv + disp)) = (ptrint) c->vftbl;
+
+	return true;
+}
+
+
+/* patcher_resolve_classref_to_flags *******************************************
+
+   CHECKCAST/INSTANCEOF:
+
+   <patched call position>
+
+*******************************************************************************/
+
+bool patcher_resolve_classref_to_flags(u1 *sp)
+{
+	constant_classref *cr;
+	s4                 disp;
+	u1                *pv;
+	classinfo         *c;
+
+	/* get stuff from the stack */
+
+	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
+	disp  =                       *((s4 *)     (sp + 1 * 8));
+	pv    = (u1 *)                *((ptrint *) (sp + 0 * 8));
+
+	/* get the fieldinfo */
+
+	if (!(c = resolve_classref_eager(cr)))
+		return false;
+
+	/* patch class flags */
+
+	*((s4 *) (pv + disp)) = (s4) c->flags;
+
+	return true;
+}
+
+
+/* patcher_resolve_native_function *********************************************
+
+   XXX
+
+*******************************************************************************/
+
+#if !defined(WITH_STATIC_CLASSPATH)
+bool patcher_resolve_native_function(u1 *sp)
+{
+	methodinfo  *m;
+	s4           disp;
+	u1          *pv;
+	functionptr  f;
+
+	/* get stuff from the stack */
+
+	m     = (methodinfo *) *((ptrint *) (sp + 2 * 8));
+	disp  =                *((s4 *)     (sp + 1 * 8));
+	pv    = (u1 *)         *((ptrint *) (sp + 0 * 8));
+
+	/* resolve native function */
+
+	if (!(f = native_resolve_function(m)))
+		return false;
+
+	/* patch native function pointer */
+
+	*((ptrint *) (pv + disp)) = (ptrint) f;
+
+	return true;
+}
+#endif /* !defined(WITH_STATIC_CLASSPATH) */
 
 
 /* patcher_get_putstatic *******************************************************
@@ -61,62 +360,32 @@
 
 bool patcher_get_putstatic(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
-	unresolved_field  *uf;
-	s4                 disp;
-	u1                *pv;
-	fieldinfo         *fi;
+	unresolved_field *uf;
+	s4                disp;
+	u1               *pv;
+	fieldinfo        *fi;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
-	uf    = (unresolved_field *)  *((ptrint *) (sp + 2 * 8));
-	disp  =                       *((s4 *)     (sp + 1 * 8));
-	pv    = (u1 *)                *((ptrint *) (sp + 0 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	uf    = (unresolved_field *) *((ptrint *) (sp + 2 * 8));
+	disp  =                      *((s4 *)     (sp + 1 * 8));
+	pv    = (u1 *)               *((ptrint *) (sp + 0 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(fi = resolve_field_eager(uf))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(fi = resolve_field_eager(uf)))
 		return false;
-	}
 
 	/* check if the field's class is initialized */
 
-	if (!(fi->class->state & CLASS_INITIALIZED)) {
-		if (!initialize_class(fi->class)) {
-			PATCHER_MONITOREXIT;
-
+	if (!(fi->class->state & CLASS_INITIALIZED))
+		if (!initialize_class(fi->class))
 			return false;
-		}
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
-
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
 
 	/* patch the field value's address */
 
 	*((ptrint *) (pv + disp)) = (ptrint) &(fi->value);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -132,239 +401,31 @@ bool patcher_get_putstatic(u1 *sp)
 
 bool patcher_get_putfield(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
-	unresolved_field  *uf;
-	fieldinfo         *fi;
+	u1               *ra;
+	unresolved_field *uf;
+	fieldinfo        *fi;
 
-	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
-	uf    = (unresolved_field *)  *((ptrint *) (sp + 2 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)               *((ptrint *) (sp + 5 * 8));
+	uf    = (unresolved_field *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(fi = resolve_field_eager(uf))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(fi = resolve_field_eager(uf)))
 		return false;
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
 
 	/* if we show disassembly, we have to skip the nop */
 
-	if (opt_showdisassemble)
-		ra = ra + 4;
+	if (opt_showdisassemble) {
+		/* patch the field's offset into the instruction */
 
-	/* patch the field's offset */
-
-	*((u4 *) ra) |= (s2) (fi->offset & 0x0000ffff);
-
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
-	return true;
-}
-
-
-/* patcher_aconst **************************************************************
-
-   Machine code:
-
-   <patched call postition>
-   a61bff80    ldq     a0,-128(pv)
-
-*******************************************************************************/
-
-bool patcher_aconst(u1 *sp)
-{
-	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
-	constant_classref *cr;
-	s4                 disp;
-	u1                *pv;
-	classinfo         *c;
-
-	/* get stuff from the stack */
-
-	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
-	disp  =                       *((s4 *)     (sp + 1 * 8));
-	pv    = (u1 *)                *((ptrint *) (sp + 0 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
-	/* get the classinfo */
-
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
-		return false;
+		*((u4 *) (ra + 4)) |= (s2) (fi->offset & 0x0000ffff);
 	}
+	else {
+		/* otherwise store the patched instruction on the stack */
 
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
-
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	/* patch the classinfo pointer */
-
-	*((ptrint *) (pv + disp)) = (ptrint) c;
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
-	return true;
-}
-
-
-/* patcher_builtin_multianewarray **********************************************
-
-   Machine code:
-
-   <patched call position>
-   a63bff80    ldq     a1,-128(pv)
-   47de0412    mov     sp,a2
-   a77bff78    ldq     pv,-136(pv)
-   6b5b4000    jsr     (pv)
-
-*******************************************************************************/
-
-bool patcher_builtin_multianewarray(u1 *sp)
-{
-	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
-	constant_classref *cr;
-	s4                 disp;
-	u1                *pv;
-	classinfo         *c;
-
-	/* get stuff from the stack */
-
-	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
-	disp  =                       *((s4 *)     (sp + 1 * 8));
-	pv    = (u1 *)                *((ptrint *) (sp + 0 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
-	/* get the classinfo */
-
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
-		return false;
+		*((u4 *) (sp + 3 * 8)) |= (s2) (fi->offset & 0x0000ffff);
 	}
 
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
-
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	/* patch the classinfo pointer */
-
-	*((ptrint *) (pv + disp)) = (ptrint) c;
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
-	return true;
-}
-
-
-/* patcher_builtin_arraycheckcast **********************************************
-
-   Machine code:
-
-   <patched call position>
-   a63bfe60    ldq     a1,-416(pv)
-   a77bfe58    ldq     pv,-424(pv)
-   6b5b4000    jsr     (pv)
-
-*******************************************************************************/
-
-bool patcher_builtin_arraycheckcast(u1 *sp)
-{
-	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
-	constant_classref *cr;
-	s4                 disp;
-	u1                *pv;
-	classinfo         *c;
-
-	/* get stuff from the stack */
-
-	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
-	disp  =                       *((s4 *)     (sp + 1 * 8));
-	pv    = (u1 *)                *((ptrint *) (sp + 0 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
-	/* get the classinfo */
-
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
-		return false;
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
-
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	/* patch the classinfo pointer */
-
-	*((ptrint *) (pv + disp)) = (ptrint) c;
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -381,9 +442,6 @@ bool patcher_builtin_arraycheckcast(u1 *sp)
 
 bool patcher_invokestatic_special(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
 	unresolved_method *um;
 	s4                 disp;
 	u1                *pv;
@@ -391,42 +449,19 @@ bool patcher_invokestatic_special(u1 *sp)
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
 	um    = (unresolved_method *) *((ptrint *) (sp + 2 * 8));
 	disp  =                       *((s4 *)     (sp + 1 * 8));
 	pv    = (u1 *)                *((ptrint *) (sp + 0 * 8));
 
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
 	/* get the fieldinfo */
 
-	if (!(m = resolve_method_eager(um))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(m = resolve_method_eager(um)))
 		return false;
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
-
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
 
 	/* patch stubroutine */
 
 	*((ptrint *) (pv + disp)) = (ptrint) m->stubroutine;
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -445,36 +480,18 @@ bool patcher_invokestatic_special(u1 *sp)
 bool patcher_invokevirtual(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
 	unresolved_method *um;
 	methodinfo        *m;
 
 	/* get stuff from the stack */
 
 	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
 	um    = (unresolved_method *) *((ptrint *) (sp + 2 * 8));
 
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
 	/* get the fieldinfo */
 
-	if (!(m = resolve_method_eager(um))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(m = resolve_method_eager(um)))
 		return false;
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
 
 	/* if we show disassembly, we have to skip the nop */
 
@@ -486,12 +503,6 @@ bool patcher_invokevirtual(u1 *sp)
 	*((s4 *) (ra + 4)) |= (s4) ((OFFSET(vftbl_t, table[0]) +
 								 sizeof(methodptr) * m->vftblindex) & 0x0000ffff);
 
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -511,36 +522,18 @@ bool patcher_invokevirtual(u1 *sp)
 bool patcher_invokeinterface(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
 	unresolved_method *um;
 	methodinfo        *m;
 
 	/* get stuff from the stack */
 
 	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
 	um    = (unresolved_method *) *((ptrint *) (sp + 2 * 8));
 
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
 	/* get the fieldinfo */
 
-	if (!(m = resolve_method_eager(um))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(m = resolve_method_eager(um)))
 		return false;
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
 
 	/* if we show disassembly, we have to skip the nop */
 
@@ -557,72 +550,6 @@ bool patcher_invokeinterface(u1 *sp)
 	*((s4 *) (ra + 4 + 4)) |=
 		(s4) ((sizeof(methodptr) * (m - m->class->methods)) & 0x0000ffff);
 
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
-	return true;
-}
-
-
-/* patcher_checkcast_instanceof_flags ******************************************
-
-   Machine code:
-
-   <patched call position>
-
-*******************************************************************************/
-
-bool patcher_checkcast_instanceof_flags(u1 *sp)
-{
-	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
-	constant_classref *cr;
-	s4                 disp;
-	u1                *pv;
-	classinfo         *c;
-
-	/* get stuff from the stack */
-
-	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
-	disp  =                       *((s4 *)     (sp + 1 * 8));
-	pv    = (u1 *)                *((ptrint *) (sp + 0 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
-	/* get the fieldinfo */
-
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
-		return false;
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
-
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	/* patch class flags */
-
-	*((s4 *) (pv + disp)) = (s4) c->flags;
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -643,36 +570,18 @@ bool patcher_checkcast_instanceof_flags(u1 *sp)
 bool patcher_checkcast_instanceof_interface(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
 	constant_classref *cr;
 	classinfo         *c;
 
 	/* get stuff from the stack */
 
 	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
 	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
 	/* get the fieldinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
 
 	/* if we show disassembly, we have to skip the nop */
 
@@ -686,243 +595,10 @@ bool patcher_checkcast_instanceof_interface(u1 *sp)
 	*((s4 *) (ra + 4 * 4)) |= (s4) ((OFFSET(vftbl_t, interfacetable[0]) -
 									 c->index * sizeof(methodptr*)) & 0x0000ffff);
 
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
 
-/* patcher_checkcast_instanceof_class ******************************************
-
-   Machine code:
-
-   <patched call position>
-   a7940000    ldq     at,0(a4)
-   a7bbff28    ldq     gp,-216(pv)
-
-*******************************************************************************/
-
-bool patcher_checkcast_instanceof_class(u1 *sp)
-{
-	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
-	constant_classref *cr;
-	s4                 disp;
-	u1                *pv;
-	classinfo         *c;
-
-	/* get stuff from the stack */
-
-	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
-	disp  =                       *((s4 *)     (sp + 1 * 8));
-	pv    = (u1 *)                *((ptrint *) (sp + 0 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
-	/* get the fieldinfo */
-
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
-		return false;
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
-
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	/* patch super class' vftbl */
-
-	*((ptrint *) (pv + disp)) = (ptrint) c->vftbl;
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
-	return true;
-}
-
-
-/* patcher_clinit **************************************************************
-
-   XXX
-
-*******************************************************************************/
-
-bool patcher_clinit(u1 *sp)
-{
-	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
-	classinfo         *c;
-
-	/* get stuff from the stack */
-
-	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
-	c     = (classinfo *)         *((ptrint *) (sp + 2 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
-	/* check if the class is initialized */
-
-	if (!(c->state & CLASS_INITIALIZED)) {
-		if (!initialize_class(c)) {
-			PATCHER_MONITOREXIT;
-
-			return false;
-		}
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
-
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
-	return true;
-}
-
-
-/* patcher_athrow_areturn ******************************************************
-
-   Machine code:
-
-   <patched call position>
-
-*******************************************************************************/
-
-#ifdef ENABLE_VERIFIER
-bool patcher_athrow_areturn(u1 *sp)
-{
-	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
-	unresolved_class  *uc;
-	classinfo         *c;
-
-	/* get stuff from the stack */
-
-	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
-	uc    = (unresolved_class *)  *((ptrint *) (sp + 2 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
-	/* resolve the class */
-
-	if (!resolve_class(uc, resolveEager, false, &c)) {
-		PATCHER_MONITOREXIT;
-
-		return false;
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
-
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
-	return true;
-}
-#endif /* ENABLE_VERIFIER */
-
-
-/* patcher_resolve_native ******************************************************
-
-   XXX
-
-*******************************************************************************/
-
-#if !defined(WITH_STATIC_CLASSPATH)
-bool patcher_resolve_native(u1 *sp)
-{
-	u1                *ra;
-	java_objectheader *o;
-	u4                 mcode;
-	methodinfo        *m;
-	s4                 disp;
-	u1                *pv;
-	functionptr        f;
-
-	/* get stuff from the stack */
-
-	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-	mcode =                       *((u4 *)     (sp + 3 * 8));
-	m     = (methodinfo *)        *((ptrint *) (sp + 2 * 8));
-	disp  =                       *((s4 *)     (sp + 1 * 8));
-	pv    = (u1 *)                *((ptrint *) (sp + 0 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 1 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
-
-	/* resolve native function */
-
-	if (!(f = native_resolve_function(m))) {
-		PATCHER_MONITOREXIT;
-
-		return false;
-	}
-
-	/* patch back original code */
-
-	*((u4 *) ra) = mcode;
-
-	/* synchronize instruction cache */
-
-	md_icacheflush(NULL, 0);
-
-	/* patch native function pointer */
-
-	*((ptrint *) (pv + disp)) = (ptrint) f;
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
-	return true;
-}
-#endif /* !defined(WITH_STATIC_CLASSPATH) */
-
-
 /*
  * These are local overrides for various environment variables in Emacs.
  * Please do not remove this and leave it at the end of the file, where
