@@ -30,7 +30,7 @@
    Changes: Christian Thalinger
             Edwin Steiner
 
-   $Id: asmpart.h 4921 2006-05-15 14:24:36Z twisti $
+   $Id: asmpart.h 5812 2006-10-20 14:22:23Z twisti $
 
 */
 
@@ -137,20 +137,26 @@ void asm_handle_exception(void);
 void asm_handle_nat_exception(void);
 #endif
 
+/* stub for throwing AbstractMethodError's */
+#if defined(ENABLE_JIT)
+void asm_abstractmethoderror(void);
+#endif
+
+#if defined(ENABLE_INTRP)
+void intrp_asm_abstractmethoderror(void);
+#endif
+
 /* wrapper for code patching functions */
-void asm_wrapper_patcher(void);
+void asm_patcher_wrapper(void);
 
 /* functions for on-stack replacement */
 void asm_replacement_out(void);
 void asm_replacement_in(executionstate *es);
 
-void *asm_switchstackandcall(void *stack, void *func, void **stacktopsave, void * p);
-
 #if defined(ENABLE_THREADS)
 extern critical_section_node_t asm_criticalsections;
 #endif
 
-
 #if defined(ENABLE_JIT)
 void asm_getclassvalues_atomic(vftbl_t *super, vftbl_t *sub, castinfo *out);
 #endif
