@@ -48,7 +48,7 @@
    memory. All functions writing values into the data area return the offset
    relative the begin of the code area (start of procedure).	
 
-   $Id: codegen-common.c 4960 2006-05-26 12:19:43Z edwin $
+   $Id: codegen-common.c 5868 2006-10-30 11:21:36Z edwin $
 
 */
 
@@ -97,13 +97,15 @@
 #include "vm/jit/stacktrace.h"
 #include "vm/jit/replace.h"
 
+#if defined(ENABLE_INTRP)
+#include "vm/jit/intrp/intrp.h"
+#endif
+
 
 /* in this tree we store all method addresses *********************************/
 
-#if defined(__I386__) || defined(__X86_64__) || defined(ENABLE_INTRP) || defined(DISABLE_GC)
 static avl_tree *methodtree = NULL;
 static s4 methodtree_comparator(const void *pc, const void *element);
-#endif
 
 
 /* codegen_init ****************************************************************
@@ -114,7 +116,6 @@ static s4 methodtree_comparator(const void *pc, const void *element);
 
 void codegen_init(void)
 {
-#if defined(__I386__) || defined(__X86_64__) || defined(ENABLE_INTRP) || defined(DISABLE_GC)
 	/* this tree is global, not method specific */
 
 	if (!methodtree) {
@@ -135,7 +136,6 @@ void codegen_init(void)
 		avl_insert(methodtree, mte);
 #endif /* defined(ENABLE_JIT) */
 	}
-#endif /* defined(__I386__) || defined(__X86_64__) || defined(ENABLE_INTRP) || defined(DISABLE_GC) */
 }
 
 
@@ -179,11 +179,9 @@ void codegen_setup(jitdata *jd)
 		cd->superstarts = NULL;
 	}
 #endif
-	
-	cd->dsegtop = DMNEW(u1, DSEGINITSIZE);
-	cd->dsegsize = DSEGINITSIZE;
-	cd->dsegtop += cd->dsegsize;
-	cd->dseglen = 0;
+
+	cd->dseg           = NULL;
+	cd->dseglen        = 0;
 
 	cd->jumpreferences = NULL;
 
@@ -200,13 +198,6 @@ void codegen_setup(jitdata *jd)
 	cd->linenumbertab = 0;
 	
 	cd->method = m;
-	cd->exceptiontable = 0;
-	cd->exceptiontablelength = 0;
-
-	if (m->exceptiontablelength > 0) {
-		cd->exceptiontablelength = m->exceptiontablelength;
-		cd->exceptiontable = DMNEW(exceptiontable, m->exceptiontablelength + 1);
-	}
 
 	cd->maxstack = m->maxstack;
 	cd->maxlocals = m->maxlocals;
@@ -400,9 +391,9 @@ void codegen_add_arraystoreexception_ref(codegendata *cd)
 
 *******************************************************************************/
 
-void codegen_add_classcastexception_ref(codegendata *cd)
+void codegen_add_classcastexception_ref(codegendata *cd, s4 reg)
 {
-	codegen_add_exception_ref(cd, -1, STACKTRACE_inline_classcastexception);
+	codegen_add_exception_ref(cd, reg, STACKTRACE_inline_classcastexception);
 }
 
 
@@ -442,7 +433,7 @@ void codegen_addpatchref(codegendata *cd, functionptr patcher, voidptr ref,
 	patchref *pr;
 	s4        branchpos;
 
-	branchpos = (u1 *) cd->mcodeptr - cd->mcodebase;
+	branchpos = cd->mcodeptr - cd->mcodebase;
 
 	pr = DNEW(patchref);
 
@@ -461,12 +452,11 @@ void codegen_addpatchref(codegendata *cd, functionptr patcher, voidptr ref,
 	   the basic block code generation is completed, we check the
 	   range and maybe generate some nop's. */
 
-	cd->lastmcodeptr = ((u1 *) cd->mcodeptr) + PATCHER_CALL_SIZE;
+	cd->lastmcodeptr = cd->mcodeptr + PATCHER_CALL_SIZE;
 #endif
 }
 
 
-#if defined(__I386__) || defined(__X86_64__) || defined(ENABLE_INTRP) || defined(DISABLE_GC)
 /* methodtree_comparator *******************************************************
 
    Comparator function used for the AVL tree of methods.
@@ -524,13 +514,14 @@ void codegen_insertmethod(u1 *startpc, u1 *endpc)
 }
 
 
-/* codegen_findmethod **********************************************************
+/* codegen_get_pv_from_pc ******************************************************
 
-   Find the PV for the given PC by searching in the AVL tree of methods.
+   Find the PV for the given PC by searching in the AVL tree of
+   methods.
 
 *******************************************************************************/
 
-u1 *codegen_findmethod(u1 *pc)
+u1 *codegen_get_pv_from_pc(u1 *pc)
 {
 	methodtree_element  mtepc;
 	methodtree_element *mte;
@@ -542,17 +533,54 @@ u1 *codegen_findmethod(u1 *pc)
 
 	mte = avl_find(methodtree, &mtepc);
 
-	if (!mte) {
-		printf("Cannot find Java function at %p\n", (void *) (ptrint) pc);
-		assert(0);
+	if (mte == NULL) {
+		/* No method was found.  Let's dump a stacktrace. */
 
-		throw_cacao_exception_exit(string_java_lang_InternalError,
-								   "Cannot find Java function at %p", pc);
+		log_println("We received a SIGSEGV and tried to handle it, but we were");
+		log_println("unable to find a Java method at:");
+		log_println("");
+#if SIZEOF_VOID_P == 8
+		log_println("PC=0x%016lx", pc);
+#else
+		log_println("PC=0x%08x", pc);
+#endif
+		log_println("");
+		log_println("Dumping the current stacktrace:");
+
+		stacktrace_dump_trace(THREADOBJECT);
+
+		vm_abort("Exiting...");
 	}
 
 	return mte->startpc;
 }
-#endif /* defined(__I386__) || defined(__X86_64__) || defined(ENABLE_INTRP) || defined(DISABLE_GC) */
+
+
+/* codegen_get_pv_from_pc_nocheck **********************************************
+
+   Find the PV for the given PC by searching in the AVL tree of
+   methods.  This method does not check the return value and is used
+   by the profiler.
+
+*******************************************************************************/
+
+u1 *codegen_get_pv_from_pc_nocheck(u1 *pc)
+{
+	methodtree_element  mtepc;
+	methodtree_element *mte;
+
+	/* allocation of the search structure on the stack is much faster */
+
+	mtepc.startpc = pc;
+	mtepc.endpc   = pc;
+
+	mte = avl_find(methodtree, &mtepc);
+
+	if (mte == NULL)
+		return NULL;
+	else
+		return mte->startpc;
+}
 
 
 /* codegen_finish **************************************************************
@@ -590,7 +618,7 @@ void codegen_finish(jitdata *jd)
 
 	/* calculate the code length */
 
-	mcodelen = (s4) ((u1 *) cd->mcodeptr - cd->mcodebase);
+	mcodelen = (s4) (cd->mcodeptr - cd->mcodebase);
 
 #if defined(ENABLE_THREADS)
 	extralen = sizeof(critical_section_node_t) * cd->threadcritcount;
@@ -605,7 +633,7 @@ void codegen_finish(jitdata *jd)
 	}
 #endif
 
-	alignedmcodelen = ALIGN(mcodelen, MAX_ALIGN);
+	alignedmcodelen = MEMORY_ALIGN(mcodelen, MAX_ALIGN);
 
 #if defined(ENABLE_INTRP)
 	if (opt_intrp)
@@ -615,7 +643,7 @@ void codegen_finish(jitdata *jd)
 	}
 #endif
 
-	cd->dseglen = ALIGN(cd->dseglen, MAX_ALIGN);
+	cd->dseglen = MEMORY_ALIGN(cd->dseglen, MAX_ALIGN);
 	alignedlen = alignedmcodelen + cd->dseglen;
 
 #if defined(ENABLE_INTRP)
@@ -627,18 +655,21 @@ void codegen_finish(jitdata *jd)
 	/* allocate new memory */
 
 	code->mcodelength = mcodelen + cd->dseglen;
-	code->mcode = CNEW(u1, alignedlen + extralen);
-
-	/* copy data and code to their new location */
-
-	MCOPY((void *) code->mcode, cd->dsegtop - cd->dseglen, u1, cd->dseglen);
-	MCOPY((void *) (code->mcode + cd->dseglen), cd->mcodebase, u1, mcodelen);
+	code->mcode       = CNEW(u1, alignedlen + extralen);
 
 	/* set the entrypoint of the method */
 	
 	assert(code->entrypoint == NULL);
 	code->entrypoint = epoint = (code->mcode + cd->dseglen);
 
+	/* fill the data segment (code->entrypoint must already be set!) */
+
+	dseg_finish(jd);
+
+	/* copy code to the new location */
+
+	MCOPY((void *) code->entrypoint, cd->mcodebase, u1, mcodelen);
+
 #if defined(ENABLE_INTRP)
 	/* relocate native dynamic superinstruction code (if any) */
 
@@ -708,12 +739,9 @@ void codegen_finish(jitdata *jd)
 		}
 	}
 
-#if defined(__I386__) || defined(__X86_64__) || defined(ENABLE_INTRP) || defined(DISABLE_GC)
 	/* add method into methodtree to find the entrypoint */
 
 	codegen_insertmethod(code->entrypoint, code->entrypoint + mcodelen);
-#endif
-
 
 #if defined(__I386__) || defined(__X86_64__) || defined(__XDSPCORE__) || defined(ENABLE_INTRP)
 	/* resolve data segment references */
@@ -721,7 +749,6 @@ void codegen_finish(jitdata *jd)
 	dseg_resolve_datareferences(jd);
 #endif
 
-
 #if defined(ENABLE_THREADS)
 	{
 		critical_section_node_t *n = (critical_section_node_t *) ((ptrint) code->mcode + alignedlen);
@@ -772,6 +799,7 @@ codeinfo *codegen_createnativestub(functionptr f, methodinfo *m)
 	jd->m     = m;
 	jd->cd    = DNEW(codegendata);
 	jd->rd    = DNEW(registerdata);
+	jd->flags = 0;
 
 	/* Allocate codeinfo memory from the heap as we need to keep them. */
 
@@ -781,6 +809,16 @@ codeinfo *codegen_createnativestub(functionptr f, methodinfo *m)
 
 	code = jd->code;
 
+	/* set the flags for the current JIT run */
+
+#if defined(ENABLE_PROFILING)
+	if (opt_prof)
+		jd->flags |= JITDATA_FLAG_INSTRUMENT;
+#endif
+
+	if (opt_verbosecall)
+		jd->flags |= JITDATA_FLAG_VERBOSECALL;
+
 	/* setup code generation stuff */
 
 #if defined(ENABLE_JIT)
@@ -946,16 +984,19 @@ void codegen_start_native_call(u1 *datasp, u1 *pv, u1 *sp, u1 *ra)
 /* codegen_finish_native_call **************************************************
 
    Removes the stuff required for a native (JNI) function call.
+   Additionally it checks for an exceptions and in case, get the
+   exception object and clear the pointer.
 
 *******************************************************************************/
 
-void codegen_finish_native_call(u1 *datasp)
+java_objectheader *codegen_finish_native_call(u1 *datasp)
 {
-	stackframeinfo  *sfi;
-	stackframeinfo **psfi;
-	localref_table  *lrt;
-	localref_table  *plrt;
-	s4               localframes;
+	stackframeinfo     *sfi;
+	stackframeinfo    **psfi;
+	localref_table     *lrt;
+	localref_table     *plrt;
+	s4                  localframes;
+	java_objectheader  *e;
 
 	/* get data structures from stack */
 
@@ -996,6 +1037,12 @@ void codegen_finish_native_call(u1 *datasp)
 	/* now store the previous local frames in the thread structure */
 
 	LOCALREFTABLE = lrt;
+
+	/* get the exception and return it */
+
+	e = exceptions_get_and_clear_exception();
+
+	return e;
 }
 
 
@@ -1043,59 +1090,30 @@ void removenativestub(u1 *stub)
 
 *******************************************************************************/
 
-s4 codegen_reg_of_var(registerdata *rd, u2 opcode, stackptr v, s4 tempregnum)
+s4 codegen_reg_of_var(u2 opcode, varinfo *v, s4 tempregnum)
 {
-	varinfo *var;
 
+#if 0
 	/* Do we have to generate a conditional move?  Yes, then always
 	   return the temporary register.  The real register is identified
 	   during the store. */
 
 	if (opcode & ICMD_CONDITION_MASK)
 		return tempregnum;
+#endif
 
-	switch (v->varkind) {
-	case TEMPVAR:
-		if (!(v->flags & INMEMORY))
-			return(v->regoff);
-		break;
-
-	case STACKVAR:
-		var = &(rd->interfaces[v->varnum][v->type]);
-		v->regoff = var->regoff;
-		if (!(var->flags & INMEMORY))
-			return(var->regoff);
-		break;
-
-	case LOCALVAR:
-		var = &(rd->locals[v->varnum][v->type]);
-		v->regoff = var->regoff;
-		if (!(var->flags & INMEMORY)) {
+	if (!(v->flags & INMEMORY)) {
 #if defined(__ARM__) && defined(__ARMEL__)
-			if (IS_2_WORD_TYPE(v->type) && (GET_HIGH_REG(var->regoff) == REG_SPLIT))
-				return(PACK_REGS(GET_LOW_REG(var->regoff), GET_HIGH_REG(tempregnum)));
+		if (IS_2_WORD_TYPE(v->type) && (GET_HIGH_REG(v->vv.regoff) == REG_SPLIT))
+			return PACK_REGS(GET_LOW_REG(v->vv.regoff),
+							 GET_HIGH_REG(tempregnum));
 #endif
 #if defined(__ARM__) && defined(__ARMEB__)
-			if (IS_2_WORD_TYPE(v->type) && (GET_LOW_REG(var->regoff) == REG_SPLIT))
-				return(PACK_REGS(GET_LOW_REG(tempregnum), GET_HIGH_REG(var->regoff)));
+		if (IS_2_WORD_TYPE(v->type) && (GET_LOW_REG(v->vv.regoff) == REG_SPLIT))
+			return PACK_REGS(GET_LOW_REG(tempregnum),
+							 GET_HIGH_REG(v->vv.regoff));
 #endif
-			return(var->regoff);
-		}
-		break;
-
-	case ARGVAR:
-		if (!(v->flags & INMEMORY)) {
-#if defined(__ARM__) && defined(__ARMEL__)
-			if (IS_2_WORD_TYPE(v->type) && (GET_HIGH_REG(v->regoff) == REG_SPLIT))
-				return(PACK_REGS(GET_LOW_REG(v->regoff), GET_HIGH_REG(tempregnum)));
-#endif
-#if defined(__ARM__) && defined(__ARMEB__)
-			if (IS_2_WORD_TYPE(v->type) && (GET_LOW_REG(v->regoff) == REG_SPLIT))
-				return(PACK_REGS(GET_LOW_REG(tempregnum), GET_HIGH_REG(v->regoff)));
-#endif
-			return(v->regoff);
-		}
-		break;
+		return v->vv.regoff;
 	}
 
 #if defined(ENABLE_STATISTICS)
@@ -1103,11 +1121,30 @@ s4 codegen_reg_of_var(registerdata *rd, u2 opcode, stackptr v, s4 tempregnum)
 		count_spills_read++;
 #endif
 
-	v->flags |= INMEMORY;
-
 	return tempregnum;
 }
 
+/* codegen_reg_of_dst **********************************************************
+
+   This function determines a register, to which the result of an
+   operation should go, when it is ultimatively intended to store the
+   result in iptr->dst.var.  If dst.var is assigned to an actual
+   register, this register will be returned.  Otherwise (when it is
+   spilled) this function returns tempregnum.  If not already done,
+   regoff and flags are set in the stack location.
+       
+   On ARM we have to check if a long/double variable is splitted
+   across reg/stack (HIGH_REG == REG_SPLIT). We return the actual
+   register of dst.var for LOW_REG and the tempregnum for HIGH_REG in such
+   cases.  (michi 2005/07/24)
+
+*******************************************************************************/
+
+s4 codegen_reg_of_dst(jitdata *jd, instruction *iptr, s4 tempregnum)
+{
+	return codegen_reg_of_var(iptr->opc, VAROP(iptr->dst), tempregnum);
+}
+
 
 #if defined(ENABLE_THREADS)
 void codegen_threadcritrestart(codegendata *cd, int offset)
