@@ -43,7 +43,7 @@
 #include "vm/jit/codegen-common.h"
 #include "vm/jit/jit.h"
 #include "vm/jit/reg.h"
-#include "vm/jit/stack.h"
+#include "vm/jit/show.h"
 
 
 /* patterns for a total number of 3 instructions ******************************/
@@ -147,8 +147,8 @@ bool ifconv_static(jitdata *jd)
 
 	/* iterate over all basic blocks */
 
-	bptr   = m->basicblocks;
-	bcount = m->basicblockcount;
+	bptr   = jd->basicblocks;
+	bcount = jd->basicblockcount;
 
 	for (; bcount >= 0; bcount--, bptr++) {
 		/* Deleted basic blocks are just skipped. */
@@ -205,7 +205,8 @@ bool ifconv_static(jitdata *jd)
 		case ICMD_IF_ACMPNE:
 			/* basic blocks can only have 1 predecessor */
 
-			if ((bptr[1].pre_count != 1) || (bptr[2].pre_count != 1))
+			if ((bptr[1].predecessorcount != 1) ||
+				(bptr[2].predecessorcount != 1))
 				break;
 
 			check(jd, bptr);
@@ -273,10 +274,12 @@ bool ifconv_static(jitdata *jd)
 
 #if !defined(NDEBUG)
 				method_println(m);
-				stack_show_basicblock(jd, &bptr[0]);
-				stack_show_basicblock(jd, &bptr[1]);
-				stack_show_basicblock(jd, &bptr[2]);
-				stack_show_basicblock(jd, &bptr[3]);
+#if 0
+				show_basicblock(jd, &bptr[0]);
+				show_basicblock(jd, &bptr[1]);
+				show_basicblock(jd, &bptr[2]);
+				show_basicblock(jd, &bptr[3]);
+#endif
 #endif
 
 				/* check the condition */
@@ -398,7 +401,9 @@ bool ifconv_static(jitdata *jd)
 
 #if !defined(NDEBUG)
 				method_println(m);
-				stack_show_basicblock(jd, &bptr[0]);
+#if 0
+				show_basicblock(jd, &bptr[0]);
+#endif
 #endif
 
 				/* delete the 2 following basic blocks */
@@ -426,13 +431,11 @@ bool ifconv_static(jitdata *jd)
 static void check(jitdata *jd, basicblock *bptr)
 {
 	methodinfo *m;
-	codegendata *cd;
 	int pattern = 0;
 
 	/* get required compiler data */
 
 	m  = jd->m;
-	cd = jd->cd;
 
 	/*
 	  generated by jikes.
@@ -967,12 +970,12 @@ static void check(jitdata *jd, basicblock *bptr)
 
 #if !defined(NDEBUG)
 	if (pattern != 0) {
-		printf("PATTERN %02d: (BB: %3d) ", pattern, m->basicblockcount - bptr->debug_nr);
+		printf("PATTERN %02d: (BB: %3d) ", pattern, jd->basicblockcount - bptr->nr);
 		method_println(m);
 
 		/* 								if (pattern == 27) { */
-		/* 									stack_show_basicblock(m, cd, &bptr[1]); */
-		/* 									stack_show_basicblock(m, cd, &bptr[2]); */
+		/* 									show_basicblock(m, cd, &bptr[1]); */
+		/* 									show_basicblock(m, cd, &bptr[2]); */
 		/* 								} */
 
 		fflush(stdout);
@@ -983,10 +986,12 @@ static void check(jitdata *jd, basicblock *bptr)
 
 			(bptr[1].iinstr[1].opc == ICMD_GOTO))
 			{
-				printf("CHECK 1   : (BB: %3d) ", m->basicblockcount - bptr->debug_nr);
+				printf("CHECK 1   : (BB: %3d) ", jd->basicblockcount - bptr->nr);
 				method_println(m);
-				stack_show_basicblock(jd, &bptr[1]);
-				stack_show_basicblock(jd, &bptr[2]);
+#if 0
+				show_basicblock(jd, &bptr[1]);
+				show_basicblock(jd, &bptr[2]);
+#endif
 				fflush(stdout);
 			}
 
@@ -995,10 +1000,12 @@ static void check(jitdata *jd, basicblock *bptr)
 
 			(bptr[1].iinstr[2].opc == ICMD_GOTO))
 			{
-				printf("CHECK 2   : (BB: %3d) ", m->basicblockcount - bptr->debug_nr);
+				printf("CHECK 2   : (BB: %3d) ", jd->basicblockcount - bptr->nr);
 				method_println(m);
-				stack_show_basicblock(jd, &bptr[1]);
-				stack_show_basicblock(jd, &bptr[2]);
+#if 0
+				show_basicblock(jd, &bptr[1]);
+				show_basicblock(jd, &bptr[2]);
+#endif
 				fflush(stdout);
 			}
 
@@ -1007,10 +1014,12 @@ static void check(jitdata *jd, basicblock *bptr)
 
 			(bptr[1].iinstr[3].opc == ICMD_GOTO))
 			{
-				printf("CHECK 3   : (BB: %3d) ", m->basicblockcount - bptr->debug_nr);
+				printf("CHECK 3   : (BB: %3d) ", jd->basicblockcount - bptr->nr);
 				method_println(m);
-				stack_show_basicblock(jd, &bptr[1]);
-				stack_show_basicblock(jd, &bptr[2]);
+#if 0
+				show_basicblock(jd, &bptr[1]);
+				show_basicblock(jd, &bptr[2]);
+#endif
 				fflush(stdout);
 			}
 
@@ -1019,10 +1028,12 @@ static void check(jitdata *jd, basicblock *bptr)
 
 			(bptr[1].iinstr[2].opc == ICMD_GOTO))
 			{
-				printf("CHECK 4   : (BB: %3d) ", m->basicblockcount - bptr->debug_nr);
+				printf("CHECK 4   : (BB: %3d) ", jd->basicblockcount - bptr->nr);
 				method_println(m);
-				stack_show_basicblock(jd, &bptr[1]);
-				stack_show_basicblock(jd, &bptr[2]);
+#if 0
+				show_basicblock(jd, &bptr[1]);
+				show_basicblock(jd, &bptr[2]);
+#endif
 				fflush(stdout);
 			}
 	}
