@@ -27,13 +27,13 @@
    Authors: Christian Thalinger
             Anton Ertl
 
-   Changes:
+   Changes: Edwin Steiner
 
-   $Id: engine.c 4921 2006-05-15 14:24:36Z twisti $
+   $Id: engine.c 5835 2006-10-26 11:29:42Z edwin $
 */
 
 
-/* #define VM_DEBUG */
+#define VM_DEBUG
 
 #include "config.h"
 
@@ -147,6 +147,12 @@
 	    } \
 	}
 
+#define THROW_CLASSCASTEXCEPTION(o) \
+    { \
+		classcastexception_object = o; \
+        THROW(classcastexception); \
+	}
+
 #define CHECK_OUT_OF_BOUNDS(_array, _idx)              \
         {                                            \
           if (length_array(_array) <= (u4) (_idx)) { \
@@ -190,7 +196,8 @@
 #define access_array_float(array, index)               \
         ((((java_floatarray*)(array))->data)[index])
 
-#define MAXLOCALS(stub) (((Cell *)stub)[1])
+/* (see createcompilerstub in codegen.c) */
+#define FRAMESIZE(stub) (((Cell *)stub)[1])
 
 #if 0
 #define CLEARSTACK(_start, _end) \
@@ -215,24 +222,24 @@ engine(Inst *ip0, Cell * sp0, Cell * fp)
 {
   Inst *ip;
   register Cell *sp SPREG = sp0;
-  Inst ca1; /* code address; this is the next dispatched instruction */
+  /* Inst ca1; XXX unused? */ /* code address; this is the next dispatched instruction */
   IF_spTOS(register Cell spTOS TOSREG;)
   static Inst   labels[] = {
 #define INST_ADDR(_inst) (&&I_##_inst)
-#include "java-labels.i"
+#include <java-labels.i>
 #undef INST_ADDR
 	  NULL,
 #define INST_ADDR(_inst) (&&J_##_inst)
-#include "java-labels.i"
+#include <java-labels.i>
 #undef INST_ADDR
 #define INST_ADDR(_inst) (&&K_##_inst)
-#include "java-labels.i"
+#include <java-labels.i>
 #undef INST_ADDR
     (Label)&&after_last,
     (Label)&&before_goto,
     (Label)&&after_goto,
 #define INST_ADDR(_inst) (&&H_##_inst)
-#include "java-labels.i"
+#include <java-labels.i>
 #undef INST_ADDR
   };
   /* local variables for the various throw codes; this helps make
@@ -243,7 +250,8 @@ engine(Inst *ip0, Cell * sp0, Cell * fp)
   Label throw_classcastexception 			 = &&throw_classcastexception1;  
   Label throw_nullpointerexception 		     = &&throw_nullpointerexception1;
   Label throw_arraystoreexception            = &&throw_arraystoreexception1;
-  s4 arrayindexoutofbounds_index; /* pass the index to the throw code */
+  java_objectheader *classcastexception_object = NULL;
+  s4 arrayindexoutofbounds_index = 0; /* pass the index to the throw code */
 
   if (vm_debug)
       fprintf(vm_out,"entering engine(%p,%p,%p)\n",ip0,sp,fp);
@@ -265,10 +273,15 @@ engine(Inst *ip0, Cell * sp0, Cell * fp)
 
       /* the actual codes jumped to through the ...exception variables */
 	  THROWCODE(arithmeticexception);
-	  THROWCODE(classcastexception);
 	  THROWCODE(nullpointerexception);
 	  THROWCODE(arraystoreexception);
 
+  throw_classcastexception1:
+	  global_sp = sp;
+	  *exceptionptr = stacktrace_inline_classcastexception(NULL, (u1 *) fp, (u1 *) IP, (u1 *) IP, classcastexception_object);
+	  CLEAR_global_sp;
+	  THROW0;
+
   throw_arrayindexoutofboundsexception1:
 	  global_sp = sp;
 	  *exceptionptr = stacktrace_inline_arrayindexoutofboundsexception(NULL, (u1 *) fp, (u1 *) IP, (u1 *) IP, arrayindexoutofbounds_index);
@@ -284,7 +297,7 @@ engine(Inst *ip0, Cell * sp0, Cell * fp)
   NEXT;
 
 #define INST_ADDR(_inst) (&&I_##_inst)
-#include "java-vm.i"
+#include <java-vm.i>
 #undef NAME
  after_last: return NULL;
 }
@@ -301,4 +314,5 @@ engine(Inst *ip0, Cell * sp0, Cell * fp)
  * c-basic-offset: 4
  * tab-width: 4
  * End:
+ * vim:noexpandtab:sw=4:ts=4:
  */
