@@ -29,7 +29,7 @@
 
    Changes: Edwin Steiner
 
-   $Id: intrp.h 4953 2006-05-25 12:28:51Z twisti $
+   $Id: intrp.h 5869 2006-10-30 11:52:09Z edwin $
 
 */
 
@@ -46,6 +46,14 @@
 #include "vm/types.h"
 #include "arch.h"
 
+/* we need Cell in some headers below */
+
+#if SIZEOF_VOID_P == 8
+typedef s8 Cell;
+#else
+typedef s4 Cell;
+#endif
+
 #include "vm/class.h"
 #include "vm/global.h"
 #include "vm/method.h"
@@ -56,12 +64,6 @@
 typedef void *Label;
 typedef void *Inst;
 
-#if SIZEOF_VOID_P == 8
-typedef s8 Cell;
-#else
-typedef s4 Cell;
-#endif
-
 #if 1
 #define MAYBE_UNUSED __attribute__((unused))
 #else
@@ -110,7 +112,7 @@ typedef union {
 
 #if defined(ENABLE_THREADS)
 
-#define global_sp    (*(Cell **) &(THREADOBJECT->_global_sp))
+#define global_sp    (THREADOBJECT->_global_sp)
 
 #else /* defined(ENABLE_THREADS) */
 
@@ -207,15 +209,16 @@ extern Inst *vm_prim;
 extern Cell peeptable;
 extern FILE *vm_out;
 
+/* defined in peephole.c: */
 void init_peeptable(void);
 ptrint peephole_opt(ptrint inst1, ptrint inst2, Cell peeptable);
-void gen_inst(codegendata *cd, ptrint instr);
-
-void vm_disassemble(Inst *ip, Inst *endp, Inst vm_prim[]);
-Inst *vm_disassemble_inst(Inst *ip, Inst vm_prim[]);
 
+/* defined in engine.c: */
 java_objectheader *engine(Inst *ip0, Cell * sp, Cell * fp);
 
+/* defined in disass.c: */
+void vm_disassemble(Inst *ip, Inst *endp, Inst vm_prim[]);
+Inst *vm_disassemble_inst(Inst *ip, Inst vm_prim[]);
 
 /* print types for disassembler and tracer */
 void printarg_ui      (u4                 ui      );
@@ -242,14 +245,25 @@ void printarg_aum     (unresolved_method *aum     );
 void printarg_avftbl  (vftbl_t *          avftbl  );
 void printarg_Cell    (Cell               x       );
 
+/* defined in profile.c: */
 void vm_uncount_block(Inst *ip);
 block_count *vm_block_insert(Inst *ip);
 
+/* defined in codegen.c: */
 Cell *nativecall(functionptr f, methodinfo *m, Cell *sp, Inst *ra, Cell *fp, u1 *addrcif);
 u1 *createcalljavafunction(methodinfo *m);
 
+/* defined in asmpart.c: */
 Inst *intrp_asm_handle_exception(Inst *ip, java_objectheader *o, Cell *fp, Cell **new_spp, Cell **new_fpp);
 
+/* defined in dynamic-super.c: */
+void gen_inst(codegendata *cd, ptrint instr);
+void append_dispatch(codegendata *cd);
+void finish_ss(codegendata *cd);
+void patchersuper_rewrite(Inst *p);
+void dynamic_super_init(void);
+void dynamic_super_rewrite(codegendata *cd);
+
 #endif /* _INTRP_H */
 
 
