@@ -1,9 +1,9 @@
 /* vm/jit/loop/tracing.c - trace functions
 
-   Copyright (C) 1996-2005 R. Grafl, A. Krall, C. Kruegel, C. Oates,
-   R. Obermaisser, M. Platter, M. Probst, S. Ring, E. Steiner,
-   C. Thalinger, D. Thuernbeck, P. Tomsich, C. Ullrich, J. Wenninger,
-   Institut f. Computersprachen - TU Wien
+   Copyright (C) 1996-2005, 2006 R. Grafl, A. Krall, C. Kruegel,
+   C. Oates, R. Obermaisser, M. Platter, M. Probst, S. Ring,
+   E. Steiner, C. Thalinger, D. Thuernbeck, P. Tomsich, C. Ullrich,
+   J. Wenninger, Institut f. Computersprachen - TU Wien
 
    This file is part of CACAO.
 
@@ -19,10 +19,10 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+   02110-1301, USA.
 
-   Contact: cacao@complang.tuwien.ac.at
+   Contact: cacao@cacaojvm.org
 
    Authors: Christopher Kruegel
 
@@ -31,7 +31,7 @@
    instruction. For more details see function tracing(basicblock, int,
    int) below.
 
-   $Id: tracing.c,v 1.13 2005/11/07 18:38:33 twisti Exp $
+   $Id: tracing.c 4862 2006-04-30 15:58:53Z edwin $
 
 */
 
@@ -216,7 +216,6 @@ Trace* tracing(basicblock *block, int index, int temp)
 {
 	instruction        *iptr;
 	methodinfo         *m;
-	unresolved_method  *um;
 	builtintable_entry *bte;
 	methoddesc         *md;
 	s4                  args;
@@ -582,14 +581,7 @@ Trace* tracing(basicblock *block, int index, int temp)
 		case ICMD_INVOKESPECIAL:    /* ..., objectref, [arg1, [arg2 ...]] ==> . */
 		case ICMD_INVOKEVIRTUAL:
 		case ICMD_INVOKEINTERFACE:
-			m = iptr->val.a;        /* get method pointer and                 */
-
-			if (m)
-				md = m->parseddesc;
-			else {
-				um = iptr->target;
-				md = um->methodref->parseddesc.md;
-			}
+			INSTRUCTION_GET_METHODDESC(iptr,md);
 
 			args = md->paramcount;  /* number of arguments                    */
 
