@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: arch.h 4357 2006-01-22 23:33:38Z twisti $
+   $Id: arch.h 5173 2006-07-25 15:57:11Z twisti $
 
 */
 
@@ -70,6 +70,9 @@
 #define SUPPORT_LONG_MUL                 1
 #define SUPPORT_LONG_DIV                 1
 
+#define SUPPORT_LONG_DIV_POW2            1
+#define SUPPORT_LONG_REM_POW2            1
+
 #define SUPPORT_CONST_LOGICAL            1  /* AND, OR, XOR with immediates   */
 #define SUPPORT_CONST_MUL                1  /* mutiply with immediate         */
 
@@ -77,8 +80,6 @@
 #define SUPPORT_CONST_STORE_ZERO_ONLY    1  /* on some risc machines we can   */
                                             /* only store REG_ZERO            */
 
-/* #define CONDITIONAL_LOADCONST           1 */
-
 #else /* SIZEOF_VOID_P == 8 */
 
 #define SUPPORT_DIVISION                 1
@@ -104,6 +105,9 @@
 #define SUPPORT_LONG_MUL                 0
 #define SUPPORT_LONG_DIV                 0
 
+#define SUPPORT_LONG_DIV_POW2            0
+#define SUPPORT_LONG_REM_POW2            0
+
 #define SUPPORT_CONST_LOGICAL            1  /* AND, OR, XOR with immediates   */
 #define SUPPORT_CONST_MUL                1  /* mutiply with immediate         */
 
@@ -111,8 +115,6 @@
 #define SUPPORT_CONST_STORE_ZERO_ONLY    1  /* on some risc machines we can   */
                                             /* only store REG_ZERO            */
 
-/* #define CONDITIONAL_LOADCONST           1 */
-
 #define SUPPORT_COMBINE_INTEGER_REGISTERS
 #define SUPPORT_PASS_FLOATARGS_IN_INTREGS
 
