@@ -28,7 +28,7 @@
 
    Changes: Edwin Steiner
 
-   $Id: md.c 4821 2006-04-24 10:08:07Z twisti $
+   $Id: md.c 5233 2006-08-14 10:59:39Z twisti $
 
 */
 
@@ -172,7 +172,7 @@ u1 *md_get_method_patch_address(u1 *ra, stackframeinfo *sfi, u1 *mptr)
 }
 
 
-/* md_codegen_findmethod *******************************************************
+/* md_codegen_get_pv_from_pc ***************************************************
 
    Machine code:
 
@@ -182,7 +182,7 @@ u1 *md_get_method_patch_address(u1 *ra, stackframeinfo *sfi, u1 *mptr)
 
 *******************************************************************************/
 
-u1 *md_codegen_findmethod(u1 *ra)
+u1 *md_codegen_get_pv_from_pc(u1 *ra)
 {
 	u1 *pv;
 	u4  mcode;
@@ -212,8 +212,8 @@ u1 *md_codegen_findmethod(u1 *ra)
 #endif
 
 		offset += (s2) (mcode & 0x0000ffff);
-
-	} else {
+	}
+	else {
 		/* get offset of first instruction (daddiu) */
 
 		mcode = *((u4 *) ra);
