@@ -28,18 +28,22 @@
 
    Changes:
 
-   $Id: patcher.c 4640 2006-03-16 17:24:18Z twisti $
+   $Id: patcher.c 5326 2006-09-05 17:21:21Z edwin $
 
 */
 
 
 #include "config.h"
+
+#include <assert.h>
+
 #include "vm/types.h"
 
 #include "mm/memory.h"
 #include "native/native.h"
 #include "vm/builtin.h"
 #include "vm/class.h"
+#include "vm/exceptions.h"
 #include "vm/field.h"
 #include "vm/initialize.h"
 #include "vm/options.h"
@@ -49,6 +53,78 @@
 #include "vm/jit/patcher.h"
 
 
+/* patcher_wrapper *************************************************************
+
+   Wrapper for all patchers.  It also creates the stackframe info
+   structure.
+
+   If the return value of the patcher function is false, it gets the
+   exception object, clears the exception pointer and returns the
+   exception.
+
+*******************************************************************************/
+
+java_objectheader *patcher_wrapper(u1 *sp, u1 *pv, u1 *ra)
+{
+	stackframeinfo     sfi;
+	u1                *xpc;
+	java_objectheader *o;
+	functionptr        f;
+	bool               result;
+	java_objectheader *e;
+
+	/* define the patcher function */
+
+	bool (*patcher_function)(u1 *);
+
+	assert(pv != NULL);
+
+	/* get stuff from the stack */
+
+	xpc = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	o   = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
+	f   = (functionptr)         *((ptrint *) (sp + 0 * 8));
+
+	/* store PV into the patcher function position */
+
+	*((ptrint *) (sp + 0 * 8)) = (ptrint) pv;
+
+	/* cast the passed function to a patcher function */
+
+	patcher_function = (bool (*)(u1 *)) (ptrint) f;
+
+	/* enter a monitor on the patching position */
+
+	PATCHER_MONITORENTER;
+
+	/* create the stackframeinfo */
+
+	stacktrace_create_extern_stackframeinfo(&sfi, pv, sp + 6 * 8, ra, xpc);
+
+	/* call the proper patcher function */
+
+	result = (patcher_function)(sp);
+
+	/* remove the stackframeinfo */
+
+	stacktrace_remove_stackframeinfo(&sfi);
+
+	/* check for return value and exit accordingly */
+
+	if (result == false) {
+		e = exceptions_get_and_clear_exception();
+
+		PATCHER_MONITOREXIT;
+
+		return e;
+	}
+
+	PATCHER_MARK_PATCHED_MONITOREXIT;
+
+	return NULL;
+}
+
+
 /* patcher_get_putstatic *******************************************************
 
    Machine code:
@@ -61,61 +137,37 @@
 
 bool patcher_get_putstatic(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
-	u4                 mcode[2];
-#endif
-	unresolved_field  *uf;
-	s4                 disp;
-	u1                *pv;
-	fieldinfo         *fi;
+	u1               *ra;
+	u4                mcode[2];
+	unresolved_field *uf;
+	s4                disp;
+	u1               *pv;
+	fieldinfo        *fi;
 
 	/* get stuff from the stack */
 
-	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
-	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
-	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
-	uf       = (unresolved_field *)  *((ptrint *) (sp + 2 * 8));
-	disp     =                       *((s4 *)     (sp + 1 * 8));
-	pv       = (u1 *)                *((ptrint *) (sp + 0 * 8));
-
-	PATCHER_MONITORENTER;
+	ra       = (u1 *)               *((ptrint *) (sp + 5 * 8));
+	mcode[0] =                      *((u4 *)     (sp + 3 * 8));
+	mcode[1] =                      *((u4 *)     (sp + 3 * 8 + 4));
+	uf       = (unresolved_field *) *((ptrint *) (sp + 2 * 8));
+	disp     =                      *((s4 *)     (sp + 1 * 8));
+	pv       = (u1 *)               *((ptrint *) (sp + 0 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(fi = resolve_field_eager(uf))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(fi = resolve_field_eager(uf)))
 		return false;
-	}
 
 	/* check if the field's class is initialized */
 
-	if (!(fi->class->state & CLASS_INITIALIZED)) {
-		if (!initialize_class(fi->class)) {
-			PATCHER_MONITOREXIT;
-
+	if (!(fi->class->state & CLASS_INITIALIZED))
+		if (!initialize_class(fi->class))
 			return false;
-		}
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* synchronize instruction cache */
 
@@ -129,8 +181,6 @@ bool patcher_get_putstatic(u1 *sp)
 
 	md_dcacheflush(pv + disp, SIZEOF_VOID_P);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -146,45 +196,25 @@ bool patcher_get_putstatic(u1 *sp)
 
 bool patcher_get_putfield(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
-	u4                 mcode[2];
-#endif
-	unresolved_field  *uf;
-	fieldinfo         *fi;
+	u1               *ra;
+	u4                mcode[2];
+	unresolved_field *uf;
+	fieldinfo        *fi;
 
-	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
-	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
-	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
-	uf       = (unresolved_field *)  *((ptrint *) (sp + 2 * 8));
-
-	PATCHER_MONITORENTER;
+	ra       = (u1 *)               *((ptrint *) (sp + 5 * 8));
+	mcode[0] =                      *((u4 *)     (sp + 3 * 8));
+	mcode[1] =                      *((u4 *)     (sp + 3 * 8 + 4));
+	uf       = (unresolved_field *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(fi = resolve_field_eager(uf))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(fi = resolve_field_eager(uf)))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* if we show disassembly, we have to skip the nop's */
 
@@ -212,17 +242,16 @@ bool patcher_get_putfield(u1 *sp)
 
 	if (opt_showdisassemble) {
 #if SIZEOF_VOID_P == 4
-		if (fi->type == TYPE_LNG) {
+		if (fi->type == TYPE_LNG)
 			md_icacheflush(ra - 2 * 4, 4 * 4);
-		} else
+		else
 #endif
 			md_icacheflush(ra - 2 * 4, 3 * 4);
-	} else {
+	}
+	else {
 		md_icacheflush(ra, 2 * 4);
 	}
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -239,12 +268,7 @@ bool patcher_get_putfield(u1 *sp)
 bool patcher_aconst(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
 	u4                 mcode[2];
-#endif
 	constant_classref *cr;
 	s4                 disp;
 	u1                *pv;
@@ -253,36 +277,21 @@ bool patcher_aconst(u1 *sp)
 	/* get stuff from the stack */
 
 	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
 	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
 	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
 	cr       = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 	disp     =                       *((s4 *)     (sp + 1 * 8));
 	pv       = (u1 *)                *((ptrint *) (sp + 0 * 8));
 
-	PATCHER_MONITORENTER;
-
 	/* get the classinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* synchronize instruction cache */
 
@@ -296,8 +305,6 @@ bool patcher_aconst(u1 *sp)
 
 	md_dcacheflush(pv + disp, SIZEOF_VOID_P);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -318,12 +325,7 @@ bool patcher_aconst(u1 *sp)
 bool patcher_builtin_multianewarray(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else	
 	u4                 mcode[2];
-#endif
 	constant_classref *cr;
 	s4                 disp;
 	u1                *pv;
@@ -332,36 +334,21 @@ bool patcher_builtin_multianewarray(u1 *sp)
 	/* get stuff from the stack */
 
 	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
 	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
 	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
 	cr       = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 	disp     =                       *((s4 *)     (sp + 1 * 8));
 	pv       = (u1 *)                *((ptrint *) (sp + 0 * 8));
 
-	PATCHER_MONITORENTER;
-
 	/* get the classinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* synchronize instruction cache */
 
@@ -375,8 +362,6 @@ bool patcher_builtin_multianewarray(u1 *sp)
 
 	md_dcacheflush(pv + disp, SIZEOF_VOID_P);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -396,12 +381,7 @@ bool patcher_builtin_multianewarray(u1 *sp)
 bool patcher_builtin_arraycheckcast(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
 	u4                 mcode[2];
-#endif
 	constant_classref *cr;
 	s4                 disp;
 	u1                *pv;
@@ -410,36 +390,21 @@ bool patcher_builtin_arraycheckcast(u1 *sp)
 	/* get stuff from the stack */
 
 	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
 	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
 	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
 	cr       = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 	disp     =                       *((s4 *)     (sp + 1 * 8));
 	pv       = (u1 *)                *((ptrint *) (sp + 0 * 8));
 
-	PATCHER_MONITORENTER;
-
 	/* get the classinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* synchronize instruction cache */
 
@@ -453,8 +418,6 @@ bool patcher_builtin_arraycheckcast(u1 *sp)
 
 	md_dcacheflush(pv + disp, SIZEOF_VOID_P);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -473,12 +436,7 @@ bool patcher_builtin_arraycheckcast(u1 *sp)
 bool patcher_invokestatic_special(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
 	u4                 mcode[2];
-#endif
 	unresolved_method *um;
 	s4                 disp;
 	u1                *pv;
@@ -487,36 +445,21 @@ bool patcher_invokestatic_special(u1 *sp)
 	/* get stuff from the stack */
 
 	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
 	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
 	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
 	um       = (unresolved_method *) *((ptrint *) (sp + 2 * 8));
 	disp     =                       *((s4 *)     (sp + 1 * 8));
 	pv       = (u1 *)                *((ptrint *) (sp + 0 * 8));
 
-	PATCHER_MONITORENTER;
-
 	/* get the fieldinfo */
 
-	if (!(m = resolve_method_eager(um))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(m = resolve_method_eager(um)))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* synchronize instruction cache */
 
@@ -530,8 +473,6 @@ bool patcher_invokestatic_special(u1 *sp)
 
 	md_dcacheflush(pv + disp, SIZEOF_VOID_P);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -551,46 +492,26 @@ bool patcher_invokestatic_special(u1 *sp)
 bool patcher_invokevirtual(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
 	u4                 mcode[2];
-#endif
 	unresolved_method *um;
 	methodinfo        *m;
 
 	/* get stuff from the stack */
 
 	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
 	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
 	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
 	um       = (unresolved_method *) *((ptrint *) (sp + 2 * 8));
 
-	PATCHER_MONITORENTER;
-
 	/* get the fieldinfo */
 
-	if (!(m = resolve_method_eager(um))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(m = resolve_method_eager(um)))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* if we show disassembly, we have to skip the nop's */
 
@@ -609,8 +530,6 @@ bool patcher_invokevirtual(u1 *sp)
 	else
 		md_icacheflush(ra, 2 * 4);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -631,46 +550,26 @@ bool patcher_invokevirtual(u1 *sp)
 bool patcher_invokeinterface(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
 	u4                 mcode[2];
-#endif
 	unresolved_method *um;
 	methodinfo        *m;
 
 	/* get stuff from the stack */
 
 	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
 	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
 	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
 	um       = (unresolved_method *) *((ptrint *) (sp + 2 * 8));
 
-	PATCHER_MONITORENTER;
-
 	/* get the fieldinfo */
 
-	if (!(m = resolve_method_eager(um))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(m = resolve_method_eager(um)))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* if we show disassembly, we have to skip the nop's */
 
@@ -694,8 +593,6 @@ bool patcher_invokeinterface(u1 *sp)
 	else
 		md_icacheflush(ra, 3 * 4);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -715,12 +612,7 @@ bool patcher_invokeinterface(u1 *sp)
 bool patcher_checkcast_instanceof_flags(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
 	u4                 mcode[2];
-#endif
 	constant_classref *cr;
 	s4                 disp;
 	u1                *pv;
@@ -729,36 +621,21 @@ bool patcher_checkcast_instanceof_flags(u1 *sp)
 	/* get stuff from the stack */
 
 	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
 	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
 	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
 	cr       = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 	disp     =                       *((s4 *)     (sp + 1 * 8));
 	pv       = (u1 *)                *((ptrint *) (sp + 0 * 8));
 
-	PATCHER_MONITORENTER;
-
 	/* get the fieldinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* synchronize instruction cache */
 
@@ -772,8 +649,6 @@ bool patcher_checkcast_instanceof_flags(u1 *sp)
 
 	md_dcacheflush(pv + disp, sizeof(s4));
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -795,46 +670,26 @@ bool patcher_checkcast_instanceof_flags(u1 *sp)
 bool patcher_checkcast_instanceof_interface(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
 	u4                 mcode[2];
-#endif
 	constant_classref *cr;
 	classinfo         *c;
 
 	/* get stuff from the stack */
 
 	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
 	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
 	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
 	cr       = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 
-	PATCHER_MONITORENTER;
-
 	/* get the fieldinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* if we show disassembly, we have to skip the nop's */
 
@@ -855,8 +710,6 @@ bool patcher_checkcast_instanceof_interface(u1 *sp)
 	else
 		md_icacheflush(ra, 6 * 4);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -874,12 +727,7 @@ bool patcher_checkcast_instanceof_interface(u1 *sp)
 bool patcher_checkcast_instanceof_class(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
 	u4                 mcode[2];
-#endif
 	constant_classref *cr;
 	s4                 disp;
 	u1                *pv;
@@ -888,36 +736,21 @@ bool patcher_checkcast_instanceof_class(u1 *sp)
 	/* get stuff from the stack */
 
 	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
 	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
 	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
 	cr       = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 	disp     =                       *((s4 *)     (sp + 1 * 8));
 	pv       = (u1 *)                *((ptrint *) (sp + 0 * 8));
 
-	PATCHER_MONITORENTER;
-
 	/* get the fieldinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* synchronize instruction cache */
 
@@ -931,8 +764,6 @@ bool patcher_checkcast_instanceof_class(u1 *sp)
 
 	md_dcacheflush(pv + disp, SIZEOF_VOID_P);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -945,55 +776,32 @@ bool patcher_checkcast_instanceof_class(u1 *sp)
 
 bool patcher_clinit(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
-	u4                 mcode[2];
-#endif
-	classinfo         *c;
+	u1        *ra;
+	u4         mcode[2];
+	classinfo *c;
 
 	/* get stuff from the stack */
 
-	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
-	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
-	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
-	c        = (classinfo *)         *((ptrint *) (sp + 2 * 8));
-
-	PATCHER_MONITORENTER;
+	ra       = (u1 *)        *((ptrint *) (sp + 5 * 8));
+	mcode[0] =               *((u4 *)     (sp + 3 * 8));
+	mcode[1] =               *((u4 *)     (sp + 3 * 8 + 4));
+	c        = (classinfo *) *((ptrint *) (sp + 2 * 8));
 
 	/* check if the class is initialized */
 
-	if (!(c->state & CLASS_INITIALIZED)) {
-		if (!initialize_class(c)) {
-			PATCHER_MONITOREXIT;
-
+	if (!(c->state & CLASS_INITIALIZED))
+		if (!initialize_class(c))
 			return false;
-		}
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* synchronize instruction cache */
 
 	md_icacheflush(ra, 2 * 4);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -1009,54 +817,32 @@ bool patcher_clinit(u1 *sp)
 #ifdef ENABLE_VERIFIER
 bool patcher_athrow_areturn(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
-	u4                 mcode[2];
-#endif
-	unresolved_class  *uc;
-	classinfo         *c;
+	u1               *ra;
+	u4                mcode[2];
+	unresolved_class *uc;
+	classinfo        *c;
 
 	/* get stuff from the stack */
 
-	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
-	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
-	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
-	uc       = (unresolved_class *)  *((ptrint *) (sp + 2 * 8));
-
-	PATCHER_MONITORENTER;
+	ra       = (u1 *)               *((ptrint *) (sp + 5 * 8));
+	mcode[0] =                      *((u4 *)     (sp + 3 * 8));
+	mcode[1] =                      *((u4 *)     (sp + 3 * 8 + 4));
+	uc       = (unresolved_class *) *((ptrint *) (sp + 2 * 8));
 
 	/* resolve the class */
 
-	if (!resolve_class(uc, resolveEager, false, &c)) {
-		PATCHER_MONITOREXIT;
-
+	if (!resolve_class(uc, resolveEager, false, &c))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* synchronize instruction cache */
 
 	md_icacheflush(ra, 2 * 4);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 #endif /* ENABLE_VERIFIER */
@@ -1071,56 +857,31 @@ bool patcher_athrow_areturn(u1 *sp)
 #if !defined(WITH_STATIC_CLASSPATH)
 bool patcher_resolve_native(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-#if SIZEOF_VOID_P == 8
-	u8                 mcode;
-#else
-	u4                 mcode[2];
-#endif
-	methodinfo        *m;
-	s4                 disp;
-	u1                *pv;
-	functionptr        f;
+	u1          *ra;
+	u4           mcode[2];
+	methodinfo  *m;
+	s4           disp;
+	u1          *pv;
+	functionptr  f;
 
 	/* get stuff from the stack */
 
-	ra       = (u1 *)                *((ptrint *) (sp + 5 * 8));
-	o        = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
-#if SIZEOF_VOID_P == 8
-	mcode    =                       *((u8 *)     (sp + 3 * 8));
-#else
-	mcode[0] =                       *((u4 *)     (sp + 3 * 8));
-	mcode[1] =                       *((u4 *)     (sp + 3 * 8 + 4));
-#endif
-	m        = (methodinfo *)        *((ptrint *) (sp + 2 * 8));
-	disp     =                       *((s4 *)     (sp + 1 * 8));
-	pv       = (u1 *)                *((ptrint *) (sp + 0 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 2 * 4;
-	*((ptrint *) (sp + 5 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra       = (u1 *)         *((ptrint *) (sp + 5 * 8));
+	mcode[0] =                *((u4 *)     (sp + 3 * 8));
+	mcode[1] =                *((u4 *)     (sp + 3 * 8 + 4));
+	m        = (methodinfo *) *((ptrint *) (sp + 2 * 8));
+	disp     =                *((s4 *)     (sp + 1 * 8));
+	pv       = (u1 *)         *((ptrint *) (sp + 0 * 8));
 
 	/* resolve native function */
 
-	if (!(f = native_resolve_function(m))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(f = native_resolve_function(m)))
 		return false;
-	}
 
 	/* patch back original code */
 
-#if SIZEOF_VOID_P == 8
-	*((u4 *) (ra + 0 * 4)) = mcode;
-	*((u4 *) (ra + 1 * 4)) = mcode >> 32;
-#else
 	*((u4 *) (ra + 0 * 4)) = mcode[0];
 	*((u4 *) (ra + 1 * 4)) = mcode[1];
-#endif
 
 	/* synchronize instruction cache */
 
@@ -1134,8 +895,6 @@ bool patcher_resolve_native(u1 *sp)
 
 	md_dcacheflush(pv + disp, SIZEOF_VOID_P);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 #endif /* !defined(WITH_STATIC_CLASSPATH) */
