@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: arch.h 4386 2006-01-30 11:26:34Z christian $
+   $Id: arch.h 5173 2006-07-25 15:57:11Z twisti $
 
 */
 
@@ -65,6 +65,9 @@
 #define SUPPORT_LONG_MUL                 0
 #define SUPPORT_LONG_DIV                 0
 
+#define SUPPORT_LONG_DIV_POW2            0
+#define SUPPORT_LONG_REM_POW2            0
+
 #define SUPPORT_CONST_LOGICAL            1  /* AND, OR, XOR with immediates   */
 #define SUPPORT_CONST_MUL                1  /* mutiply with immediate         */
 
@@ -72,8 +75,6 @@
 #define SUPPORT_CONST_STORE_ZERO_ONLY    0  /* on some risc machines we can   */
                                             /* only store REG_ZERO            */
 
-/* #define CONDITIONAL_LOADCONST            1 */
-
 #define SPECIALMEMUSE
 #define HAS_4BYTE_STACKSLOT
 #define ALIGN_LONGS_IN_MEMORY       /* Align Longs and/or Doubles at         */
