@@ -28,7 +28,7 @@
 
    Changes: Christian Ullrich
 
-   $Id: md-abi.c 4710 2006-03-30 10:23:11Z twisti $
+   $Id: md-abi.c 5634 2006-10-02 14:18:04Z edwin $
 
 */
 
@@ -40,6 +40,7 @@
 
 #include "vm/descriptor.h"
 #include "vm/global.h"
+#include "vm/jit/abi.h"
 
 
 /* register descripton arrays *************************************************/
@@ -114,7 +115,7 @@ void md_param_alloc(methoddesc *md)
 
 	iarg = 0;
 	farg = 0;
-	stacksize = LA_WORD_SIZE;
+	stacksize = LA_SIZE_IN_POINTERS;
 
 	/* get params field of methoddesc */
 
@@ -128,44 +129,51 @@ void md_param_alloc(methoddesc *md)
 				pd->inmemory = false;
 				pd->regoff = iarg;           /* rd->arg[int|flt]regs index !! */
 				iarg++;
-			} else {
+			}
+			else {
 				pd->inmemory = true;
 				pd->regoff = stacksize;
 			}
 			stacksize++;
 			break;
+
 		case TYPE_LNG:
 			if (iarg < INT_ARG_CNT - 1) {
 				pd->inmemory = false;
 				                             /* rd->arg[int|flt]regs index !! */
 				pd->regoff = PACK_REGS(iarg + 1, iarg); 
 				iarg += 2;
-			} else {
+			}
+			else {
 				pd->inmemory = true;
 				pd->regoff = stacksize;
 				iarg = INT_ARG_CNT;
 			}
 			stacksize += 2;
 			break;
+
 		case TYPE_FLT:
 			if (farg < FLT_ARG_CNT) {
 				pd->inmemory = false;
 				pd->regoff = farg;           /* rd->arg[int|flt]regs index !! */
-				iarg++;     /* skip 1 integer argument register */
+				iarg++;                   /* skip 1 integer argument register */
 				farg++;
-			} else {
+			}
+			else {
 				pd->inmemory = true;
 				pd->regoff = stacksize;
 			}
 			stacksize++;
 			break;
+
 		case TYPE_DBL:
 			if (farg < FLT_ARG_CNT) {
 				pd->inmemory = false;
 				pd->regoff = farg;           /* rd->arg[int|flt]regs index !! */
-				iarg += 2;  /* skip 2 integer argument registers */
+				iarg += 2;               /* skip 2 integer argument registers */
 				farg++;
-			} else {
+			}
+			else {
 				pd->inmemory = true;
 				pd->regoff = stacksize;
 			}
@@ -177,19 +185,22 @@ void md_param_alloc(methoddesc *md)
 
 	/* Since R3/R4, F1 (==A0/A1, A0) are used for passing return values, this */
 	/* argument register usage has to be regarded, too                        */
+
 	if (IS_INT_LNG_TYPE(md->returntype.type)) {
 		if (iarg < (IS_2_WORD_TYPE(md->returntype.type) ? 2 : 1))
 			iarg = IS_2_WORD_TYPE(md->returntype.type) ? 2 : 1;
-	} else {
+	}
+	else {
 		if (IS_FLT_DBL_TYPE(md->returntype.type))
 			if (farg < 1)
 				farg = 1;
 	}
 
 	/* fill register and stack usage */
+
 	md->argintreguse = iarg;
 	md->argfltreguse = farg;
-	md->memuse = stacksize;
+	md->memuse       = stacksize;
 }
 
 
@@ -200,56 +211,63 @@ void md_param_alloc(methoddesc *md)
    for float/double)
 
    --- in
-   m:                       Methodinfo of current method
-   return_type:             Return Type of the Method (TYPE_INT.. TYPE_ADR)
-                            TYPE_VOID is not allowed!
+   jd:                      jitdata of the current method
    stackslot:               Java Stackslot to contain the Return Value
    
    --- out
    if precoloring was possible:
-   stackslot->varkind       =ARGVAR
-            ->varnum        =-1
-            ->flags         =0
-            ->regoff        =[REG_RESULT, (REG_RESULT2/REG_RESULT), REG_FRESULT]
+   VAR(stackslot->varnum)->flags  = PREALLOC
+   			             ->regoff = [REG_RESULT|REG_FRESULT]
    rd->arg[flt|int]reguse   set to a value according the register usage
 
+   NOTE: Do not pass a LOCALVAR in stackslot->varnum.
+
 *******************************************************************************/
 
-void md_return_alloc(methodinfo *m, registerdata *rd, s4 return_type,
-					 stackptr stackslot)
+void md_return_alloc(jitdata *jd, stackptr stackslot)
 {
+	methodinfo   *m;
+	registerdata *rd;
+	methoddesc   *md;
+
+	/* get required compiler data */
+
+	m  = jd->m;
+	rd = jd->rd;
+
+	md = m->parseddesc;
+
 	/* In Leafmethods Local Vars holding parameters are precolored to
 	   their argument register -> so leafmethods with paramcount > 0
 	   could already use R3 == a00! */
 
-	if (!m->isleafmethod || (m->parseddesc->paramcount == 0)) {
+	if (!jd->isleafmethod || (md->paramcount == 0)) {
 		/* Only precolor the stackslot, if it is not a SAVEDVAR <->
 		   has not to survive method invokations. */
 
 		if (!(stackslot->flags & SAVEDVAR)) {
-			stackslot->varkind = ARGVAR;
-			stackslot->varnum = -1;
-			stackslot->flags = 0;
 
-			if (IS_INT_LNG_TYPE(return_type)) {
-				if (!IS_2_WORD_TYPE(return_type)) {
+			VAR(stackslot->varnum)->flags = PREALLOC;
+
+			if (IS_INT_LNG_TYPE(md->returntype.type)) {
+				if (!IS_2_WORD_TYPE(md->returntype.type)) {
 					if (rd->argintreguse < 1)
 						rd->argintreguse = 1;
 
-					stackslot->regoff = REG_RESULT;
-
-				} else {
+					VAR(stackslot->varnum)->vv.regoff = REG_RESULT;
+				}
+				else {
 					if (rd->argintreguse < 2)
 						rd->argintreguse = 2;
 
-					stackslot->regoff = PACK_REGS(REG_RESULT2, REG_RESULT);
+					VAR(stackslot->varnum)->vv.regoff = REG_RESULT_PACKED;
 				}
-
-			} else { /* float/double */
+			}
+			else {
 				if (rd->argfltreguse < 1)
 					rd->argfltreguse = 1;
 
-				stackslot->regoff = REG_FRESULT;
+				VAR(stackslot->varnum)->vv.regoff = REG_FRESULT;
 			}
 		}
 	}
