@@ -28,7 +28,7 @@
 
    Changes: Christian Ullrich
 
-   $Id: md-abi.h 4357 2006-01-22 23:33:38Z twisti $
+   $Id: md-abi.h 5382 2006-09-06 21:17:00Z twisti $
 
 */
 
@@ -56,6 +56,11 @@
 #define REG_SP           1   /* stack pointer                                 */
 #define REG_ZERO         0   /* almost always zero: only in address calc.     */
 
+#define REG_A0           3   /* define some argument registers                */
+#define REG_A1           4
+#define REG_A2           5
+#define REG_A3           6
+
 /* floating point registers */
 
 #define REG_FRESULT      1   /* to deliver floating point method results      */
@@ -81,13 +86,20 @@
 #define TRACE_ARGS_NUM   8
 
 
+/* packed register defines ****************************************************/
+
+#define REG_ITMP12_PACKED    PACK_REGS(REG_ITMP2, REG_ITMP1)
+#define REG_ITMP23_PACKED    PACK_REGS(REG_ITMP3, REG_ITMP2)
+#define REG_RESULT_PACKED    PACK_REGS(REG_RESULT2, REG_RESULT)
+
+
 /* ABI defines ****************************************************************/
 
-#define LA_SIZE         24   /* linkage area size                             */
-#define LA_SIZE_ALIGNED 32   /* linkage area size aligned to 16-byte          */
-#define LA_WORD_SIZE     6   /* linkage area size in words: 6 * 4 = 24        */
+#define LA_SIZE             24  /* linkage area size                          */
+#define LA_SIZE_ALIGNED     32  /* linkage area size aligned to 16-byte       */
+#define LA_SIZE_IN_POINTERS LA_SIZE / SIZEOF_VOID_P
 
-#define LA_LR_OFFSET     8   /* link register offset in linkage area          */
+#define LA_LR_OFFSET         8  /* link register offset in linkage area       */
 
 /* #define ALIGN_FRAME_SIZE(sp)       (sp) */
 
