@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: md-asm.h 4921 2006-05-15 14:24:36Z twisti $
+   $Id: md-asm.h 5580 2006-09-29 12:46:15Z edwin $
 
 */
 
@@ -36,9 +36,6 @@
 #ifndef _MD_ASM_H
 #define _MD_ASM_H
 
-#include <mach/ppc/asm.h>
-
-
 /* define register names compatible with Linux names **************************/
 
 #define fr0     f0
@@ -126,44 +123,44 @@
 #define mptrn 12
 
 
-#define ftmp3 fr0
+#define ftmp3 f0
 
-#define fa0   fr1
-#define fa1   fr2
-#define fa2   fr3
-#define fa3   fr4
-#define fa4   fr5
-#define fa5   fr6
-#define fa6   fr7
+#define fa0   f1
+#define fa1   f2
+#define fa2   f3
+#define fa3   f4
+#define fa4   f5
+#define fa5   f6
+#define fa6   f7
 
-#define fa7   fr8
-#define fa8   fr9
-#define fa9   fr10
-#define fa10  fr11
-#define fa11  fr12
-#define fa12  fr13
+#define fa7   f8
+#define fa8   f9
+#define fa9   f10
+#define fa10  f11
+#define fa11  f12
+#define fa12  f13
 
-#define fs0   fr14
-#define fs1   fr15
+#define fs0   f14
+#define fs1   f15
 
-#define ftmp1 fr16
-#define ftmp2 fr17
+#define ftmp1 f16
+#define ftmp2 f17
 
-#define ft0   fr18
-#define ft1   fr19
-#define ft2   fr20
-#define ft3   fr21
-#define ft4   fr22
-#define ft5   fr23
+#define ft0   f18
+#define ft1   f19
+#define ft2   f20
+#define ft3   f21
+#define ft4   f22
+#define ft5   f23
 
-#define fs2   fr24
-#define fs3   fr25
-#define fs4   fr26
-#define fs5   fr27
-#define fs6   fr28
-#define fs7   fr29
-#define fs8   fr30
-#define fs9   fr31
+#define fs2   f24
+#define fs3   f25
+#define fs4   f26
+#define fs5   f27
+#define fs6   f28
+#define fs7   f29
+#define fs8   f30
+#define fs9   f31
 
 #define fv0   fa0
 
@@ -238,27 +235,37 @@
 
 #define asm_handle_nat_exception              _asm_handle_nat_exception
 #define asm_handle_exception                  _asm_handle_exception
-#define asm_handle_nullptr_exception          _asm_handle_nullptr_exception
-#define asm_wrapper_patcher                   _asm_wrapper_patcher
+
+#define asm_abstractmethoderror               _asm_abstractmethoderror
+
+#define asm_patcher_wrapper                   _asm_patcher_wrapper
+
 #define asm_replacement_out                   _asm_replacement_out
 #define asm_replacement_in                    _asm_replacement_in
 
 #define asm_cacheflush                        _asm_cacheflush
-#define asm_initialize_thread_stack           _asm_initialize_thread_stack
-#define asm_perform_threadswitch              _asm_perform_threadswitch
-#define asm_switchstackandcall                _asm_switchstackandcall
+
 #define asm_criticalsections                  _asm_criticalsections
 #define asm_getclassvalues_atomic             _asm_getclassvalues_atomic
 
 
 /* external defines ***********************************************************/
 
-#define builtin_asm_get_exceptionptrptr \
-    L_builtin_asm_get_exceptionptrptr$stub
+#define exceptions_get_and_clear_exception \
+    L_exceptions_get_and_clear_exception$stub
 
 #define builtin_throw_exception               L_builtin_throw_exception$stub
+
+#define md_codegen_get_pv_from_pc             L_md_codegen_get_pv_from_pc$stub
 #define exceptions_handle_exception           L_exceptions_handle_exception$stub
+
 #define jit_asm_compile                       L_jit_asm_compile$stub
+
+#define exceptions_asm_new_abstractmethoderror \
+    L_exceptions_asm_new_abstractmethoderror$stub
+
+#define patcher_wrapper                       L_patcher_wrapper$stub
+
 #define replace_me                            L_replace_me$stub
 
 #define stacktrace_create_extern_stackframeinfo \
@@ -267,10 +274,6 @@
 #define stacktrace_remove_stackframeinfo \
     L_stacktrace_remove_stackframeinfo$stub
 
-#if !defined(ENABLE_THREADS)
-#define _no_threads_exceptionptr              __no_threads_exceptionptr
-#endif
-
 #endif /* _MD_ASM_H */
 
 
