@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: md-abi.h 4357 2006-01-22 23:33:38Z twisti $
+   $Id: md-abi.h 5382 2006-09-06 21:17:00Z twisti $
 
 */
 
@@ -56,6 +56,11 @@
 #define REG_SP           1   /* stack pointer                                 */
 #define REG_ZERO         0   /* almost always zero: only in address calc.     */
 
+#define REG_A0           3   /* define some argument registers                */
+#define REG_A1           4
+#define REG_A2           5
+#define REG_A3           6
+
 /* floating point registers */
 
 #define REG_FRESULT      1   /* to deliver floating point method results      */
@@ -63,7 +68,7 @@
 #define REG_FTMP2       17   /* temporary floating point register             */
 #define REG_FTMP3        0   /* temporary floating point register             */
 
-#define REG_IFTMP        0   /* temporary integer and floating point register */
+#define REG_IFTMP       16   /* temporary integer and floating point register */
 
 
 #define INT_REG_CNT     32   /* number of integer registers                   */
@@ -81,13 +86,20 @@
 #define TRACE_ARGS_NUM   8
 
 
+/* packed register defines ****************************************************/
+
+#define REG_ITMP12_PACKED    PACK_REGS(REG_ITMP2, REG_ITMP1)
+#define REG_ITMP23_PACKED    PACK_REGS(REG_ITMP3, REG_ITMP2)
+#define REG_RESULT_PACKED    PACK_REGS(REG_RESULT2, REG_RESULT)
+
+
 /* ABI defines ****************************************************************/
 
-#define LA_SIZE          8   /* linkage area size                             */
-#define LA_SIZE_ALIGNED 16   /* linkage area size aligned to 16-byte          */
-#define LA_WORD_SIZE     2   /* linkage area size in words: 2 * 4 = 8         */
+#define LA_SIZE              8  /* linkage area size                          */
+#define LA_SIZE_ALIGNED     16  /* linkage area size aligned to 16-byte       */
+#define LA_SIZE_IN_POINTERS LA_SIZE / SIZEOF_VOID_P
 
-#define LA_LR_OFFSET     4   /* link register offset in linkage area          */
+#define LA_LR_OFFSET         4  /* link register offset in linkage area       */
 
 /* #define ALIGN_FRAME_SIZE(sp)       (sp) */
 
