@@ -29,7 +29,7 @@
    Changes: Christian Ullrich
             Edwin Steiner
 
-   $Id: reg.h 4710 2006-03-30 10:23:11Z twisti $
+   $Id: reg.h 5708 2006-10-06 22:37:13Z edwin $
 
 */
 
@@ -49,24 +49,31 @@ typedef struct registerdata registerdata;
 #include "arch.h"
 
 #include "vm/jit/jit.h"
+#include "vm/jit/verify/typeinfo.h"
 
 
 /************************* pseudo variable structure **************************/
 
 struct varinfo {
-	int type;                   /* basic type of variable                     */
-	int flags;                  /* flags (SAVED, INMEMORY)                    */
-	int regoff;                 /* register number or memory offset           */
+	s4 type;                   /* basic type of variable                     */
+	s4 flags;                  /* flags (SAVED, INMEMORY)                    */
+	union {
+		s4 regoff;             /* register number or memory offset           */
+		s4 i;
+		s8 l;
+		float f;
+		double d;
+		basicblock *retaddr;
+	} vv;
+#if defined(ENABLE_VERIFIER)
+	typeinfo typeinfo;         /* type info for reference types              */
+#endif
 };
 
 typedef struct varinfo varinfo5[5];
 
 
 struct registerdata {
-	varinfo5 *locals;
-	varinfo5 *interfaces;
-
-
 	int intreg_ret;                 /* register to return integer values      */
 	int fltreg_ret;                 /* register for return float values       */
 
@@ -93,6 +100,8 @@ struct registerdata {
 	int *freetmpadrregs;            /* free scratch address registers         */
 	int *freesavadrregs;            /* free saved address registers           */
 
+	int *adrusedinout;              /* is this adr register uses as INOUT?    */
+
 	int argadrreguse;               /* used argument address register count   */
 	int tmpadrreguse;               /* used scratch address register count    */
 	int savadrreguse;               /* used saved address register count      */
@@ -124,6 +133,13 @@ struct registerdata {
 	int freesavinttop;              /* free saved integer register count      */
 	int freetmpflttop;              /* free scratch float register count      */
 	int freesavflttop;              /* free saved float register count        */
+
+	int *intusedinout;              /* is this int register uses as INOUT?    */
+	int *fltusedinout;              /* is this flt register uses as INOUT?    */
+	int *regisoutvar;               /* true if reg. is outvar of this block   */
+	int *regcopycount;              /* counts copies of each register regoff  */
+	int *memcopycount;              /* counts copies of each INMEMORY regoff  */
+	int memcopycountsize;           /* size of memcopycount buffer            */
 };
 
 
