@@ -82,6 +82,7 @@
 
 bool replace_create_replacement_points(jitdata *jd)
 {
+#if 0
 	codeinfo     *code;
 	registerdata *rd;
 	basicblock *bptr;
@@ -119,7 +120,8 @@ bool replace_create_replacement_points(jitdata *jd)
 
 	count = 0;
 	alloccount = 0;
-	for (bptr = m->basicblocks; bptr; bptr = bptr->next) {
+
+	for (bptr = jd->basicblocks; bptr; bptr = bptr->next) {
 		if (!(bptr->bitflags & BBFLAG_REPLACEMENT))
 			continue;
 
@@ -197,7 +199,8 @@ bool replace_create_replacement_points(jitdata *jd)
 	/* initialize replacement point structs */
 
 	rp = rplpoints;
-	for (bptr = m->basicblocks; bptr; bptr = bptr->next) {
+
+	for (bptr = jd->basicblocks; bptr; bptr = bptr->next) {
 		if (!(bptr->bitflags & BBFLAG_REPLACEMENT))
 			continue;
 
@@ -228,17 +231,17 @@ bool replace_create_replacement_points(jitdata *jd)
 
 	/* store the data in the codeinfo */
 
-	code->rplpoints = rplpoints;
+	code->rplpoints     = rplpoints;
 	code->rplpointcount = count;
-	code->regalloc = regalloc;
+	code->regalloc      = regalloc;
 	code->regalloccount = alloccount;
-	code->globalcount = globalcount;
+	code->globalcount   = globalcount;
 	code->savedintcount = INT_SAV_CNT - rd->savintreguse;
 	code->savedfltcount = FLT_SAV_CNT - rd->savfltreguse;
-	code->memuse = rd->memuse;
-	code->isleafmethod = m->isleafmethod; /* XXX will be moved to codeinfo */
+	code->memuse        = rd->memuse;
 
 	/* everything alright */
+#endif
 
 	return true;
 }
