@@ -29,7 +29,7 @@
    Changes: Christian Thalinger
             Edwin Steiner
 
-   $Id: stacktrace.c 4962 2006-05-26 12:26:41Z edwin $
+   $Id: stacktrace.c 5805 2006-10-19 09:32:29Z twisti $
 
 */
 
@@ -109,6 +109,7 @@ void stacktrace_create_stackframeinfo(stackframeinfo *sfi, u1 *pv, u1 *sp,
 {
 	stackframeinfo **psfi;
 	methodinfo      *m;
+	codeinfo        *code;
 
 	/* get current stackframe info pointer */
 
@@ -119,19 +120,23 @@ void stacktrace_create_stackframeinfo(stackframeinfo *sfi, u1 *pv, u1 *sp,
 	if (pv == NULL) {
 #if defined(ENABLE_INTRP)
 		if (opt_intrp)
-			pv = codegen_findmethod(ra);
+			pv = codegen_get_pv_from_pc(ra);
 		else
 #endif
 			{
 #if defined(ENABLE_JIT)
-				pv = md_codegen_findmethod(ra);
+				pv = md_codegen_get_pv_from_pc(ra);
 #endif
 			}
 	}
 
-	/* get methodinfo pointer from data segment */
+	/* get codeinfo pointer from data segment */
 
-	m = *((methodinfo **) (pv + MethodPointer));
+	code = *((codeinfo **) (pv + CodeinfoPointer));
+
+	/* For asm_vm_call_method the codeinfo pointer is NULL. */
+
+	m = (code == NULL) ? NULL : code->m;
 
 	/* fill new stackframe info structure */
 
@@ -172,7 +177,7 @@ void stacktrace_create_inline_stackframeinfo(stackframeinfo *sfi, u1 *pv,
 		/* if we don't have pv handy */
 
 		if (pv == NULL)
-			pv = codegen_findmethod(ra);
+			pv = codegen_get_pv_from_pc(ra);
 
 	}
 #endif
@@ -220,12 +225,12 @@ void stacktrace_create_extern_stackframeinfo(stackframeinfo *sfi, u1 *pv,
 	if (pv == NULL) {
 #if defined(ENABLE_INTRP)
 		if (opt_intrp)
-			pv = codegen_findmethod(ra);
+			pv = codegen_get_pv_from_pc(ra);
 		else
 #endif
 			{
 #if defined(ENABLE_JIT)
-				pv = md_codegen_findmethod(ra);
+				pv = md_codegen_get_pv_from_pc(ra);
 #endif
 			}
 	}
@@ -288,10 +293,15 @@ void stacktrace_create_native_stackframeinfo(stackframeinfo *sfi, u1 *pv,
 {
 	stackframeinfo **psfi;
 	methodinfo      *m;
+	codeinfo        *code;
 
-	/* get methodinfo pointer from data segment */
+	/* get codeinfo pointer from data segment */
 
-	m = *((methodinfo **) (pv + MethodPointer));
+	code = *((codeinfo **) (pv + CodeinfoPointer));
+
+	/* For asm_vm_call_method the codeinfo pointer is NULL. */
+
+	m = (code == NULL) ? NULL : code->m;
 
 	/* get current stackframe info pointer */
 
@@ -409,7 +419,7 @@ java_objectheader *stacktrace_inline_arraystoreexception(u1 *pv, u1 *sp, u1 *ra,
 
 	/* create exception */
 
-	o = new_arraystoreexception();
+	o = exceptions_new_arraystoreexception();
 
 	/* remove stackframeinfo */
 
@@ -426,10 +436,11 @@ java_objectheader *stacktrace_inline_arraystoreexception(u1 *pv, u1 *sp, u1 *ra,
 *******************************************************************************/
 
 java_objectheader *stacktrace_inline_classcastexception(u1 *pv, u1 *sp, u1 *ra,
-														u1 *xpc)
+														u1 *xpc,
+														java_objectheader *o)
 {
 	stackframeinfo     sfi;
-	java_objectheader *o;
+	java_objectheader *e;
 
 	/* create stackframeinfo */
 
@@ -437,13 +448,13 @@ java_objectheader *stacktrace_inline_classcastexception(u1 *pv, u1 *sp, u1 *ra,
 
 	/* create exception */
 
-	o = new_classcastexception();
+	e = exceptions_new_classcastexception(o);
 
 	/* remove stackframeinfo */
 
 	stacktrace_remove_stackframeinfo(&sfi);
 
-	return o;
+	return e;
 }
 
 
@@ -736,7 +747,13 @@ static bool stacktrace_add_method(stacktracebuffer *stb, methodinfo *m, u1 *pv,
 
 	lntentry = (linenumbertable_entry *) (lntstart - SIZEOF_VOID_P);
 
-	/* find the realization of the method the pc is in    */
+	/* find the realization of the method the pc is in */
+
+#if 0
+	code = *((codeinfo **) (pv + CodeinfoPointer));
+#endif
+
+#if 1
 	/* XXX Note: This is preliminary. It would be cleaner */
 	/* to get the codeinfo * from the PV                  */
 
@@ -760,6 +777,7 @@ static bool stacktrace_add_method(stacktracebuffer *stb, methodinfo *m, u1 *pv,
 
 		code = code->prev;
 	}
+#endif
 
 	/* search the line number table */
 
@@ -791,6 +809,7 @@ stacktracebuffer *stacktrace_create(threadobject* thread)
 	stacktracebuffer *stb;
 	stackframeinfo   *sfi;
 	methodinfo       *m;
+	codeinfo         *code;
 	u1               *pv;
 	u1               *sp;
 	u4                framesize;
@@ -836,7 +855,7 @@ stacktracebuffer *stacktrace_create(threadobject* thread)
 
 	m = NULL;
 
-	while (m || sfi) {
+	while ((m != NULL) || (sfi != NULL)) {
 		/* m == NULL should only happen for the first time and inline
 		   stackframe infos, like from the exception stubs or the
 		   patcher wrapper. */
@@ -866,18 +885,23 @@ stacktracebuffer *stacktrace_create(threadobject* thread)
 
 #if defined(ENABLE_INTRP)
 				if (opt_intrp)
-					pv = codegen_findmethod(ra);
+					pv = codegen_get_pv_from_pc(ra);
 				else
 #endif
 					{
 #if defined(ENABLE_JIT)
-						pv = md_codegen_findmethod(ra);
+						pv = md_codegen_get_pv_from_pc(ra);
 #endif
 					}
 
 				/* get methodinfo pointer from parent data segment */
 
-				m = *((methodinfo **) (pv + MethodPointer));
+				code = *((codeinfo **) (pv + CodeinfoPointer));
+
+				/* For asm_vm_call_method the codeinfo pointer is
+				   NULL. */
+
+				m = (code == NULL) ? NULL : code->m;
 
 			} else {
 				/* Inline stackframe infos are special: they have a
@@ -903,7 +927,12 @@ stacktracebuffer *stacktrace_create(threadobject* thread)
 
 				/* get methodinfo from current Java method */
 
-				m = *((methodinfo **) (pv + MethodPointer));
+				code = *((codeinfo **) (pv + CodeinfoPointer));
+
+				/* For asm_vm_call_method the codeinfo pointer is
+				   NULL. */
+
+				m = (code == NULL) ? NULL : code->m;
 
 				/* if m == NULL, this is a asm_calljavafunction call */
 
@@ -932,8 +961,8 @@ stacktracebuffer *stacktrace_create(threadobject* thread)
 					fflush(stdout);
 #endif
 
-					/* set stack pointer to stackframe of parent Java */
-					/* function of the current Java function */
+					/* Set stack pointer to stackframe of parent Java
+					   function of the current Java function. */
 
 #if defined(__I386__) || defined (__X86_64__)
 					sp += framesize + SIZEOF_VOID_P;
@@ -941,14 +970,19 @@ stacktracebuffer *stacktrace_create(threadobject* thread)
 					sp += framesize;
 #endif
 
-					/* get data segment and methodinfo pointer from parent */
-					/* method */
+					/* get data segment and methodinfo pointer from
+					   parent method */
 
 #if defined(ENABLE_JIT)
-					pv = md_codegen_findmethod(ra);
+					pv = md_codegen_get_pv_from_pc(ra);
 #endif
 
-					m = *((methodinfo **) (pv + MethodPointer));
+					code = *((codeinfo **) (pv + CodeinfoPointer));
+
+					/* For asm_vm_call_method the codeinfo pointer is
+					   NULL. */
+
+					m = (code == NULL) ? NULL : code->m;
 
 #if defined(ENABLE_INTRP)
 					}
@@ -1007,16 +1041,20 @@ stacktracebuffer *stacktrace_create(threadobject* thread)
 
 #if defined(ENABLE_INTRP)
 			if (opt_intrp)
-				pv = codegen_findmethod(ra);
+				pv = codegen_get_pv_from_pc(ra);
 			else
 #endif
 				{
 #if defined(ENABLE_JIT)
-					pv = md_codegen_findmethod(ra);
+					pv = md_codegen_get_pv_from_pc(ra);
 #endif
 				}
 
-			m = *((methodinfo **) (pv + MethodPointer));
+			code = *((codeinfo **) (pv + CodeinfoPointer));
+
+			/* For asm_vm_call_method the codeinfo pointer is NULL. */
+
+			m = (code == NULL) ? NULL : code->m;
 
 			/* walk the stack */
 
@@ -1382,7 +1420,7 @@ static void stacktrace_print_trace_from_buffer(stacktracebuffer *stb)
 
 *******************************************************************************/
 
-void stacktrace_dump_trace(void)
+void stacktrace_dump_trace(threadobject *thread)
 {
 	stacktracebuffer *stb;
 	s4                dumpsize;
@@ -1415,14 +1453,13 @@ void stacktrace_dump_trace(void)
 
 	/* create a stacktrace for the current thread */
 
-	stb = stacktrace_create(THREADOBJECT);
+	stb = stacktrace_create(thread);
 
 	/* print stacktrace */
 
-	if (stb) {
+	if (stb)
 		stacktrace_print_trace_from_buffer(stb);
-
-	} else {
+	else {
 		puts("\t<<No stacktrace available>>");
 		fflush(stdout);
 	}
