@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: stacktrace.h 4921 2006-05-15 14:24:36Z twisti $
+   $Id: stacktrace.h 5805 2006-10-19 09:32:29Z twisti $
 
 */
 
@@ -45,6 +45,12 @@ typedef struct stacktrace_entry stacktrace_entry;
 #include "config.h"
 #include "vm/types.h"
 
+#if defined(ENABLE_THREADS)
+# include "threads/native/threads.h"
+#else
+# include "threads/none/threads.h"
+#endif
+
 #include "vm/method.h"
 
 
@@ -136,7 +142,8 @@ java_objectheader *stacktrace_inline_arraystoreexception(u1 *pv, u1 *sp, u1 *ra,
     (functionptr) stacktrace_inline_arraystoreexception
 
 java_objectheader *stacktrace_inline_classcastexception(u1 *pv, u1 *sp, u1 *ra,
-														u1 *xpc);
+														u1 *xpc,
+														java_objectheader *o);
 #define STACKTRACE_inline_classcastexception \
     (functionptr) stacktrace_inline_classcastexception
 
@@ -165,7 +172,7 @@ java_objectarray  *stacktrace_getClassContext(void);
 classinfo         *stacktrace_getCurrentClass(void);
 java_objectarray  *stacktrace_getStack(void);
 
-void stacktrace_dump_trace(void);
+void stacktrace_dump_trace(threadobject *thread);
 void stacktrace_print_trace(java_objectheader *xptr);
 
 
