@@ -29,7 +29,7 @@
 
    Changes:
 
-   $Id: codegen.h 4943 2006-05-23 08:51:33Z twisti $
+   $Id: codegen.h 5632 2006-10-02 13:43:15Z edwin $
 
 */
 
@@ -124,26 +124,24 @@
     } while (0)
 
 
-#define M_COPY(s,d)                     emit_copy(jd, iptr, (s), (d))
-
 #define ICONST(r,c) \
     do { \
-        if (iptr->val.i == 0) \
-            M_CLR(d); \
+        if ((c) == 0) \
+            M_CLR((d)); \
         else \
-            M_IMOV_IMM(iptr->val.i, d); \
+            M_IMOV_IMM((c), (d)); \
     } while (0)
 /*     do { \ */
-/*        M_IMOV_IMM(iptr->val.i, d); \ */
+/*        M_IMOV_IMM((c), (d)); \ */
 /*     } while (0) */
 
 
 #define LCONST(r,c) \
     do { \
-        if (iptr->val.l == 0) \
-            M_CLR(d); \
+        if ((c) == 0) \
+            M_CLR((d)); \
         else \
-            M_MOV_IMM(iptr->val.l, d); \
+            M_MOV_IMM((c), (d)); \
     } while (0)
 
 
@@ -348,14 +346,14 @@
 
 #define PROFILE_CYCLE_START \
     do { \
-        if (opt_prof) { \
+        if (JITDATA_HAS_FLAG_INSTRUMENT(jd)) { \
             M_PUSH(RAX); \
             M_PUSH(RDX); \
             \
-            M_MOV_IMM((ptrint) m, REG_ITMP3); \
+            M_MOV_IMM(code, REG_ITMP3); \
             M_RDTSC; \
-            M_ISUB_MEMBASE(RAX, REG_ITMP3, OFFSET(methodinfo, cycles)); \
-            M_ISBB_MEMBASE(RDX, REG_ITMP3, OFFSET(methodinfo, cycles) + 4); \
+            M_ISUB_MEMBASE(RAX, REG_ITMP3, OFFSET(codeinfo, cycles)); \
+            M_ISBB_MEMBASE(RDX, REG_ITMP3, OFFSET(codeinfo, cycles) + 4); \
             \
             M_POP(RDX); \
             M_POP(RAX); \
@@ -364,14 +362,14 @@
 
 #define PROFILE_CYCLE_STOP \
     do { \
-        if (opt_prof) { \
+        if (JITDATA_HAS_FLAG_INSTRUMENT(jd)) { \
             M_PUSH(RAX); \
             M_PUSH(RDX); \
             \
-            M_MOV_IMM((ptrint) m, REG_ITMP3); \
+            M_MOV_IMM(code, REG_ITMP3); \
             M_RDTSC; \
-            M_IADD_MEMBASE(RAX, REG_ITMP3, OFFSET(methodinfo, cycles)); \
-            M_IADC_MEMBASE(RDX, REG_ITMP3, OFFSET(methodinfo, cycles) + 4); \
+            M_IADD_MEMBASE(RAX, REG_ITMP3, OFFSET(codeinfo, cycles)); \
+            M_IADC_MEMBASE(RDX, REG_ITMP3, OFFSET(codeinfo, cycles) + 4); \
             \
             M_POP(RDX); \
             M_POP(RAX); \
