@@ -28,31 +28,39 @@
 
    Changes:
 
-   $Id: emit.c 4943 2006-05-23 08:51:33Z twisti $
+   $Id: emit.c 5595 2006-09-30 23:06:36Z edwin $
 
 */
 
+#include "config.h"
 
 #include "vm/types.h"
 
 #include "md-abi.h"
 
-#include "vm/jit/codegen-common.h"
-#include "vm/jit/emit.h"
-#include "vm/jit/jit.h"
 #include "vm/jit/x86_64/codegen.h"
-#include "vm/jit/x86_64/md-emit.h"
+#include "vm/jit/x86_64/emit.h"
+
+#if defined(ENABLE_THREADS)
+# include "threads/native/lock.h"
+#endif
+
+#include "vm/builtin.h"
+#include "vm/jit/abi-asm.h"
+#include "vm/jit/asmpart.h"
+#include "vm/jit/codegen-common.h"
+#include "vm/jit/emit-common.h"
+#include "vm/jit/jit.h"
+#include "vm/jit/replace.h"
 
 
-/* code generation functions **************************************************/
+/* emit_load *******************************************************************
 
-/* emit_load_s1 ****************************************************************
-
-   Emits a possible load of the first source operand.
+   Emits a possible load of an operand.
 
 *******************************************************************************/
 
-s4 emit_load_s1(jitdata *jd, instruction *iptr, stackptr src, s4 tempreg)
+s4 emit_load(jitdata *jd, instruction *iptr, varinfo *src, s4 tempreg)
 {
 	codegendata  *cd;
 	s4            disp;
@@ -62,18 +70,18 @@ s4 emit_load_s1(jitdata *jd, instruction *iptr, stackptr src, s4 tempreg)
 
 	cd = jd->cd;
 
-	if (src->flags & INMEMORY) {
+	if (IS_INMEMORY(src->flags)) {
 		COUNT_SPILLS;
 
-		disp = src->regoff * 8;
+		disp = src->vv.regoff * 8;
 
 		if (IS_FLT_DBL_TYPE(src->type)) {
 			if (IS_2_WORD_TYPE(src->type))
 				M_DLD(tempreg, REG_SP, disp);
 			else
 				M_FLD(tempreg, REG_SP, disp);
-
-		} else {
+		}
+		else {
 			if (IS_INT_TYPE(src->type))
 				M_ILD(tempreg, REG_SP, disp);
 			else
@@ -81,92 +89,9 @@ s4 emit_load_s1(jitdata *jd, instruction *iptr, stackptr src, s4 tempreg)
 		}
 
 		reg = tempreg;
-	} else
-		reg = src->regoff;
-
-	return reg;
-}
-
-
-/* emit_load_s2 ****************************************************************
-
-   Emits a possible load of the second source operand.
-
-*******************************************************************************/
-
-s4 emit_load_s2(jitdata *jd, instruction *iptr, stackptr src, s4 tempreg)
-{
-	codegendata  *cd;
-	s4            disp;
-	s4            reg;
-
-	/* get required compiler data */
-
-	cd = jd->cd;
-
-	if (src->flags & INMEMORY) {
-		COUNT_SPILLS;
-
-		disp = src->regoff * 8;
-
-		if (IS_FLT_DBL_TYPE(src->type)) {
-			if (IS_2_WORD_TYPE(src->type))
-				M_DLD(tempreg, REG_SP, disp);
-			else
-				M_FLD(tempreg, REG_SP, disp);
-
-		} else {
-			if (IS_INT_TYPE(src->type))
-				M_ILD(tempreg, REG_SP, disp);
-			else
-				M_LLD(tempreg, REG_SP, disp);
-		}
-
-		reg = tempreg;
-	} else
-		reg = src->regoff;
-
-	return reg;
-}
-
-
-/* emit_load_s3 ****************************************************************
-
-   Emits a possible load of the third source operand.
-
-*******************************************************************************/
-
-s4 emit_load_s3(jitdata *jd, instruction *iptr, stackptr src, s4 tempreg)
-{
-	codegendata  *cd;
-	s4            disp;
-	s4            reg;
-
-	/* get required compiler data */
-
-	cd = jd->cd;
-
-	if (src->flags & INMEMORY) {
-		COUNT_SPILLS;
-
-		disp = src->regoff * 8;
-
-		if (IS_FLT_DBL_TYPE(src->type)) {
-			if (IS_2_WORD_TYPE(src->type))
-				M_DLD(tempreg, REG_SP, disp);
-			else
-				M_FLD(tempreg, REG_SP, disp);
-
-		} else {
-			if (IS_INT_TYPE(src->type))
-				M_ILD(tempreg, REG_SP, disp);
-			else
-				M_LLD(tempreg, REG_SP, disp);
-		}
-
-		reg = tempreg;
-	} else
-		reg = src->regoff;
+	}
+	else
+		reg = src->vv.regoff;
 
 	return reg;
 }
@@ -181,19 +106,20 @@ s4 emit_load_s3(jitdata *jd, instruction *iptr, stackptr src, s4 tempreg)
     
 *******************************************************************************/
 
-void emit_store(jitdata *jd, instruction *iptr, stackptr dst, s4 d)
+inline void emit_store(jitdata *jd, instruction *iptr, varinfo *dst, s4 d)
 {
 	codegendata  *cd;
-	registerdata *rd;
 	s4            disp;
+#if 0
 	s4            s;
 	u2            opcode;
+#endif
 
 	/* get required compiler data */
 
 	cd = jd->cd;
-	rd = jd->rd;
 
+#if 0
 	/* do we have to generate a conditional move? */
 
 	if ((iptr != NULL) && (iptr->opc & ICMD_CONDITION_MASK)) {
@@ -214,19 +140,20 @@ void emit_store(jitdata *jd, instruction *iptr, stackptr dst, s4 d)
 
 		emit_cmovxx(cd, iptr, s, d);
 	}
+#endif
 
-	if (dst->flags & INMEMORY) {
+	if (IS_INMEMORY(dst->flags)) {
 		COUNT_SPILLS;
 
-		disp = dst->regoff * 8;
+		disp = dst->vv.regoff * 8;
 
 		if (IS_FLT_DBL_TYPE(dst->type)) {
 			if (IS_2_WORD_TYPE(dst->type))
 				M_DST(d, REG_SP, disp);
 			else
 				M_FST(d, REG_SP, disp);
-
-		} else
+		}
+		else
 			M_LST(d, REG_SP, disp);
 	}
 }
@@ -234,25 +161,34 @@ void emit_store(jitdata *jd, instruction *iptr, stackptr dst, s4 d)
 
 /* emit_copy *******************************************************************
 
-   XXX
+   Generates a register/memory to register/memory copy.
 
 *******************************************************************************/
 
-void emit_copy(jitdata *jd, instruction *iptr, stackptr src, stackptr dst)
+void emit_copy(jitdata *jd, instruction *iptr, varinfo *src, varinfo *dst)
 {
 	codegendata  *cd;
-	registerdata *rd;
 	s4            s1, d;
 
 	/* get required compiler data */
 
 	cd = jd->cd;
-	rd = jd->rd;
 
-	if ((src->regoff != dst->regoff) ||
+	if ((src->vv.regoff != dst->vv.regoff) ||
 		((src->flags ^ dst->flags) & INMEMORY)) {
-		d = codegen_reg_of_var(rd, iptr->opc, dst, REG_IFTMP);
-		s1 = emit_load_s1(jd, iptr, src, d);
+
+		/* If one of the variables resides in memory, we can eliminate
+		   the register move from/to the temporary register with the
+		   order of getting the destination register and the load. */
+
+		if (IS_INMEMORY(src->flags)) {
+			d = codegen_reg_of_var(iptr->opc, dst, REG_IFTMP);
+			s1 = emit_load(jd, iptr, src, d);
+		}
+		else {
+			s1 = emit_load(jd, iptr, src, REG_IFTMP);
+			d = codegen_reg_of_var(iptr->opc, dst, s1);
+		}
 
 		if (s1 != d) {
 			if (IS_FLT_DBL_TYPE(src->type))
@@ -268,7 +204,8 @@ void emit_copy(jitdata *jd, instruction *iptr, stackptr src, stackptr dst)
 
 void emit_cmovxx(codegendata *cd, instruction *iptr, s4 s, s4 d)
 {
-	switch ((iptr->opc & ICMD_CONDITION_MASK) >> 8) {
+#if 0
+	switch (iptr->flags.fields.condition) {
 	case ICMD_IFEQ:
 		M_CMOVEQ(s, d);
 		break;
@@ -288,9 +225,362 @@ void emit_cmovxx(codegendata *cd, instruction *iptr, s4 s, s4 d)
 		M_CMOVLE(s, d);
 		break;
 	}
+#endif
 }
 
 
+/* emit_exception_stubs ********************************************************
+
+   Generates the code for the exception stubs.
+
+*******************************************************************************/
+
+void emit_exception_stubs(jitdata *jd)
+{
+	codegendata  *cd;
+	registerdata *rd;
+	exceptionref *eref;
+	s4           targetdisp;
+
+	/* get required compiler data */
+
+	cd = jd->cd;
+	rd = jd->rd;
+
+	/* generate exception stubs */
+
+	targetdisp = 0;
+
+	for (eref = cd->exceptionrefs; eref != NULL; eref = eref->next) {
+		gen_resolvebranch(cd->mcodebase + eref->branchpos, 
+						  eref->branchpos,
+						  cd->mcodeptr - cd->mcodebase);
+
+		MCODECHECK(512);
+
+		/* Check if the exception is an
+		   ArrayIndexOutOfBoundsException.  If so, move index register
+		   into a4. */
+
+		if (eref->reg != -1)
+			M_MOV(eref->reg, rd->argintregs[4]);
+
+		/* calcuate exception address */
+
+		M_MOV_IMM(0, rd->argintregs[3]);
+		dseg_adddata(cd);
+		M_AADD_IMM32(eref->branchpos - 6, rd->argintregs[3]);
+
+		/* move function to call into REG_ITMP3 */
+
+		M_MOV_IMM(eref->function, REG_ITMP3);
+
+		if (targetdisp == 0) {
+			targetdisp = cd->mcodeptr - cd->mcodebase;
+
+			emit_lea_membase_reg(cd, RIP, -((cd->mcodeptr + 7) - cd->mcodebase), rd->argintregs[0]);
+			M_MOV(REG_SP, rd->argintregs[1]);
+			M_ALD(rd->argintregs[2], REG_SP, cd->stackframesize * 8);
+
+			M_ASUB_IMM(2 * 8, REG_SP);
+			M_AST(rd->argintregs[3], REG_SP, 0 * 8);             /* store XPC */
+
+			M_CALL(REG_ITMP3);
+
+			M_ALD(REG_ITMP2_XPC, REG_SP, 0 * 8);
+			M_AADD_IMM(2 * 8, REG_SP);
+
+			M_MOV_IMM(asm_handle_exception, REG_ITMP3);
+			M_JMP(REG_ITMP3);
+		}
+		else {
+			M_JMP_IMM((cd->mcodebase + targetdisp) -
+					  (cd->mcodeptr + PATCHER_CALL_SIZE));
+		}
+	}
+}
+
+
+/* emit_patcher_stubs **********************************************************
+
+   Generates the code for the patcher stubs.
+
+*******************************************************************************/
+
+void emit_patcher_stubs(jitdata *jd)
+{
+	codegendata *cd;
+	patchref    *pref;
+	u8           mcode;
+	u1          *savedmcodeptr;
+	u1          *tmpmcodeptr;
+	s4           targetdisp;
+	s4           disp;
+
+	/* get required compiler data */
+
+	cd = jd->cd;
+
+	/* generate code patching stub call code */
+
+	targetdisp = 0;
+
+	for (pref = cd->patchrefs; pref != NULL; pref = pref->next) {
+		/* check size of code segment */
+
+		MCODECHECK(512);
+
+		/* Get machine code which is patched back in later. A
+		   `call rel32' is 5 bytes long (but read 8 bytes). */
+
+		savedmcodeptr = cd->mcodebase + pref->branchpos;
+		mcode = *((u8 *) savedmcodeptr);
+
+		/* patch in `call rel32' to call the following code */
+
+		tmpmcodeptr  = cd->mcodeptr;    /* save current mcodeptr              */
+		cd->mcodeptr = savedmcodeptr;   /* set mcodeptr to patch position     */
+
+		M_CALL_IMM(tmpmcodeptr - (savedmcodeptr + PATCHER_CALL_SIZE));
+
+		cd->mcodeptr = tmpmcodeptr;     /* restore the current mcodeptr       */
+
+		/* move pointer to java_objectheader onto stack */
+
+#if defined(ENABLE_THREADS)
+		/* create a virtual java_objectheader */
+
+		(void) dseg_addaddress(cd, NULL);                          /* flcword */
+		(void) dseg_addaddress(cd, lock_get_initial_lock_word());
+		disp = dseg_addaddress(cd, NULL);                          /* vftbl   */
+
+		emit_lea_membase_reg(cd, RIP, -((cd->mcodeptr + 7) - cd->mcodebase) + disp, REG_ITMP3);
+		M_PUSH(REG_ITMP3);
+#else
+		M_PUSH_IMM(0);
+#endif
+
+		/* move machine code bytes and classinfo pointer into registers */
+
+		M_MOV_IMM(mcode, REG_ITMP3);
+		M_PUSH(REG_ITMP3);
+
+		M_MOV_IMM(pref->ref, REG_ITMP3);
+		M_PUSH(REG_ITMP3);
+
+		M_MOV_IMM(pref->disp, REG_ITMP3);
+		M_PUSH(REG_ITMP3);
+
+		M_MOV_IMM(pref->patcher, REG_ITMP3);
+		M_PUSH(REG_ITMP3);
+
+		if (targetdisp == 0) {
+			targetdisp = cd->mcodeptr - cd->mcodebase;
+
+			M_MOV_IMM(asm_patcher_wrapper, REG_ITMP3);
+			M_JMP(REG_ITMP3);
+		}
+		else {
+			M_JMP_IMM((cd->mcodebase + targetdisp) -
+					  (cd->mcodeptr + PATCHER_CALL_SIZE));
+		}
+	}
+}
+
+
+/* emit_replacement_stubs ******************************************************
+
+   Generates the code for the replacement stubs.
+
+*******************************************************************************/
+
+void emit_replacement_stubs(jitdata *jd)
+{
+	codegendata *cd;
+	codeinfo    *code;
+	rplpoint    *rplp;
+	s4           disp;
+	s4           i;
+
+	/* get required compiler data */
+
+	cd   = jd->cd;
+	code = jd->code;
+
+	rplp = code->rplpoints;
+
+	for (i = 0; i < code->rplpointcount; ++i, ++rplp) {
+		/* check code segment size */
+
+		MCODECHECK(512);
+
+		/* note start of stub code */
+
+		rplp->outcode = (u1 *) (ptrint) (cd->mcodeptr - cd->mcodebase);
+
+		/* make machine code for patching */
+
+		disp = (ptrint) (rplp->outcode - rplp->pc) - 5;
+
+		rplp->mcode = 0xe9 | ((u8) disp << 8);
+
+		/* push address of `rplpoint` struct */
+			
+		M_MOV_IMM(rplp, REG_ITMP3);
+		M_PUSH(REG_ITMP3);
+
+		/* jump to replacement function */
+
+		M_MOV_IMM(asm_replacement_out, REG_ITMP3);
+		M_JMP(REG_ITMP3);
+	}
+}
+	
+
+/* emit_verbosecall_enter ******************************************************
+
+   Generates the code for the call trace.
+
+*******************************************************************************/
+
+#if !defined(NDEBUG)
+void emit_verbosecall_enter(jitdata *jd)
+{
+	methodinfo   *m;
+	codegendata  *cd;
+	registerdata *rd;
+	methoddesc   *md;
+	s4            i, j, k;
+
+	/* get required compiler data */
+
+	m  = jd->m;
+	cd = jd->cd;
+	rd = jd->rd;
+
+	md = m->parseddesc;
+
+	/* mark trace code */
+
+	M_NOP;
+
+	/* additional +1 is for 16-byte stack alignment */
+
+	M_LSUB_IMM((ARG_CNT + TMP_CNT + 1 + 1) * 8, REG_SP);
+
+	/* save argument registers */
+
+	for (i = 0; i < INT_ARG_CNT; i++)
+		M_LST(rd->argintregs[i], REG_SP, (1 + i) * 8);
+
+	for (i = 0; i < FLT_ARG_CNT; i++)
+		M_DST(rd->argfltregs[i], REG_SP, (1 + INT_ARG_CNT + i) * 8);
+
+	/* save temporary registers for leaf methods */
+
+	if (jd->isleafmethod) {
+		for (i = 0; i < INT_TMP_CNT; i++)
+			M_LST(rd->tmpintregs[i], REG_SP, (1 + ARG_CNT + i) * 8);
+
+		for (i = 0; i < FLT_TMP_CNT; i++)
+			M_DST(rd->tmpfltregs[i], REG_SP, (1 + ARG_CNT + INT_TMP_CNT + i) * 8);
+	}
+
+	/* show integer hex code for float arguments */
+
+	for (i = 0, j = 0; i < md->paramcount && i < INT_ARG_CNT; i++) {
+		/* If the paramtype is a float, we have to right shift all
+		   following integer registers. */
+	
+		if (IS_FLT_DBL_TYPE(md->paramtypes[i].type)) {
+			for (k = INT_ARG_CNT - 2; k >= i; k--)
+				M_MOV(rd->argintregs[k], rd->argintregs[k + 1]);
+
+			emit_movd_freg_reg(cd, rd->argfltregs[j], rd->argintregs[i]);
+			j++;
+		}
+	}
+
+	M_MOV_IMM(m, REG_ITMP2);
+	M_AST(REG_ITMP2, REG_SP, 0 * 8);
+	M_MOV_IMM(builtin_trace_args, REG_ITMP1);
+	M_CALL(REG_ITMP1);
+
+	/* restore argument registers */
+
+	for (i = 0; i < INT_ARG_CNT; i++)
+		M_LLD(rd->argintregs[i], REG_SP, (1 + i) * 8);
+
+	for (i = 0; i < FLT_ARG_CNT; i++)
+		M_DLD(rd->argfltregs[i], REG_SP, (1 + INT_ARG_CNT + i) * 8);
+
+	/* restore temporary registers for leaf methods */
+
+	if (jd->isleafmethod) {
+		for (i = 0; i < INT_TMP_CNT; i++)
+			M_LLD(rd->tmpintregs[i], REG_SP, (1 + ARG_CNT + i) * 8);
+
+		for (i = 0; i < FLT_TMP_CNT; i++)
+			M_DLD(rd->tmpfltregs[i], REG_SP, (1 + ARG_CNT + INT_TMP_CNT + i) * 8);
+	}
+
+	M_LADD_IMM((ARG_CNT + TMP_CNT + 1 + 1) * 8, REG_SP);
+
+	/* mark trace code */
+
+	M_NOP;
+}
+#endif /* !defined(NDEBUG) */
+
+
+/* emit_verbosecall_exit *******************************************************
+
+   Generates the code for the call trace.
+
+*******************************************************************************/
+
+#if !defined(NDEBUG)
+void emit_verbosecall_exit(jitdata *jd)
+{
+	methodinfo   *m;
+	codegendata  *cd;
+	registerdata *rd;
+
+	/* get required compiler data */
+
+	m  = jd->m;
+	cd = jd->cd;
+	rd = jd->rd;
+
+	/* mark trace code */
+
+	M_NOP;
+
+	M_ASUB_IMM(2 * 8, REG_SP);
+
+	M_LST(REG_RESULT, REG_SP, 0 * 8);
+	M_DST(REG_FRESULT, REG_SP, 1 * 8);
+
+	M_MOV_IMM(m, rd->argintregs[0]);
+	M_MOV(REG_RESULT, rd->argintregs[1]);
+	M_FLTMOVE(REG_FRESULT, rd->argfltregs[0]);
+	M_FLTMOVE(REG_FRESULT, rd->argfltregs[1]);
+
+	M_MOV_IMM(builtin_displaymethodstop, REG_ITMP1);
+	M_CALL(REG_ITMP1);
+
+	M_LLD(REG_RESULT, REG_SP, 0 * 8);
+	M_DLD(REG_FRESULT, REG_SP, 1 * 8);
+
+	M_AADD_IMM(2 * 8, REG_SP);
+
+	/* mark trace code */
+
+	M_NOP;
+}
+#endif /* !defined(NDEBUG) */
+
+
 /* code generation functions **************************************************/
 
 static void emit_membase(codegendata *cd, s4 basereg, s4 disp, s4 dreg)
@@ -331,17 +621,66 @@ static void emit_membase(codegendata *cd, s4 basereg, s4 disp, s4 dreg)
 }
 
 
-void emit_ishift(codegendata *cd, s4 shift_op, stackptr src, instruction *iptr)
+static void emit_membase32(codegendata *cd, s4 basereg, s4 disp, s4 dreg)
 {
-	s4 s1 = src->prev->regoff;
-	s4 s2 = src->regoff;
-	s4 d = iptr->dst->regoff;
-	s4 d_old;
+	if ((basereg == REG_SP) || (basereg == R12)) {
+		emit_address_byte(2, dreg, REG_SP);
+		emit_address_byte(0, REG_SP, REG_SP);
+		emit_imm32(disp);
+	}
+	else {
+		emit_address_byte(2, dreg, basereg);
+		emit_imm32(disp);
+	}
+}
+
+
+static void emit_memindex(codegendata *cd, s4 reg, s4 disp, s4 basereg, s4 indexreg, s4 scale)
+{
+	if (basereg == -1) {
+		emit_address_byte(0, reg, 4);
+		emit_address_byte(scale, indexreg, 5);
+		emit_imm32(disp);
+	}
+	else if ((disp == 0) && (basereg != RBP) && (basereg != R13)) {
+		emit_address_byte(0, reg, 4);
+		emit_address_byte(scale, indexreg, basereg);
+	}
+	else if (IS_IMM8(disp)) {
+		emit_address_byte(1, reg, 4);
+		emit_address_byte(scale, indexreg, basereg);
+		emit_imm8(disp);
+	}
+	else {
+		emit_address_byte(2, reg, 4);
+		emit_address_byte(scale, indexreg, basereg);
+		emit_imm32(disp);
+	}
+}
+
+
+void emit_ishift(jitdata *jd, s4 shift_op, instruction *iptr)
+{
+	s4 s1, s2, d, d_old;
+	varinfo *v_s1,*v_s2,*v_dst;
+	codegendata *cd;
+
+	/* get required compiler data */
+
+	cd = jd->cd;
+
+	v_s1  = VAROP(iptr->s1);
+	v_s2  = VAROP(iptr->sx.s23.s2);
+	v_dst = VAROP(iptr->dst);
+
+	s1 = v_s1->vv.regoff;
+	s2 = v_s2->vv.regoff;
+	d  = v_dst->vv.regoff;
 
 	M_INTMOVE(RCX, REG_ITMP1);                                    /* save RCX */
 
-	if (iptr->dst->flags & INMEMORY) {
-		if ((src->flags & INMEMORY) && (src->prev->flags & INMEMORY)) {
+	if (IS_INMEMORY(v_dst->flags)) {
+		if (IS_INMEMORY(v_s2->flags) && IS_INMEMORY(v_s1->flags)) {
 			if (s1 == d) {
 				M_ILD(RCX, REG_SP, s2 * 8);
 				emit_shiftl_membase(cd, shift_op, REG_SP, d * 8);
@@ -353,7 +692,7 @@ void emit_ishift(codegendata *cd, s4 shift_op, stackptr src, instruction *iptr)
 				M_IST(REG_ITMP2, REG_SP, d * 8);
 			}
 
-		} else if ((src->flags & INMEMORY) && !(src->prev->flags & INMEMORY)) {
+		} else if (IS_INMEMORY(v_s2->flags) && !IS_INMEMORY(v_s1->flags)) {
 			/* s1 may be equal to RCX */
 			if (s1 == RCX) {
 				if (s2 == d) {
@@ -373,7 +712,7 @@ void emit_ishift(codegendata *cd, s4 shift_op, stackptr src, instruction *iptr)
 
 			emit_shiftl_membase(cd, shift_op, REG_SP, d * 8);
 
-		} else if (!(src->flags & INMEMORY) && (src->prev->flags & INMEMORY)) {
+		} else if (!IS_INMEMORY(v_s2->flags) && IS_INMEMORY(v_s1->flags)) {
 			if (s1 == d) {
 				M_INTMOVE(s2, RCX);
 				emit_shiftl_membase(cd, shift_op, REG_SP, d * 8);
@@ -400,18 +739,18 @@ void emit_ishift(codegendata *cd, s4 shift_op, stackptr src, instruction *iptr)
 			d = REG_ITMP3;
 		}
 					
-		if ((src->flags & INMEMORY) && (src->prev->flags & INMEMORY)) {
+		if (IS_INMEMORY(v_s2->flags) && IS_INMEMORY(v_s1->flags)) {
 			M_ILD(RCX, REG_SP, s2 * 8);
 			M_ILD(d, REG_SP, s1 * 8);
 			emit_shiftl_reg(cd, shift_op, d);
 
-		} else if ((src->flags & INMEMORY) && !(src->prev->flags & INMEMORY)) {
+		} else if (IS_INMEMORY(v_s2->flags) && !IS_INMEMORY(v_s1->flags)) {
 			/* s1 may be equal to RCX */
 			M_INTMOVE(s1, d);
 			M_ILD(RCX, REG_SP, s2 * 8);
 			emit_shiftl_reg(cd, shift_op, d);
 
-		} else if (!(src->flags & INMEMORY) && (src->prev->flags & INMEMORY)) {
+		} else if (!IS_INMEMORY(v_s2->flags) && IS_INMEMORY(v_s1->flags)) {
 			M_INTMOVE(s2, RCX);
 			M_ILD(d, REG_SP, s1 * 8);
 			emit_shiftl_reg(cd, shift_op, d);
@@ -447,17 +786,28 @@ void emit_ishift(codegendata *cd, s4 shift_op, stackptr src, instruction *iptr)
 }
 
 
-void emit_lshift(codegendata *cd, s4 shift_op, stackptr src, instruction *iptr)
+void emit_lshift(jitdata *jd, s4 shift_op, instruction *iptr)
 {
-	s4 s1 = src->prev->regoff;
-	s4 s2 = src->regoff;
-	s4 d = iptr->dst->regoff;
-	s4 d_old;
+	s4 s1, s2, d, d_old;
+	varinfo *v_s1,*v_s2,*v_dst;
+	codegendata *cd;
+
+	/* get required compiler data */
+
+	cd = jd->cd;
+
+	v_s1  = VAROP(iptr->s1);
+	v_s2  = VAROP(iptr->sx.s23.s2);
+	v_dst = VAROP(iptr->dst);
+
+	s1 = v_s1->vv.regoff;
+	s2 = v_s2->vv.regoff;
+	d  = v_dst->vv.regoff;
 	
 	M_INTMOVE(RCX, REG_ITMP1);                                    /* save RCX */
 
-	if (iptr->dst->flags & INMEMORY) {
-		if ((src->flags & INMEMORY) && (src->prev->flags & INMEMORY)) {
+	if (IS_INMEMORY(v_dst->flags)) {
+		if (IS_INMEMORY(v_s2->flags) && IS_INMEMORY(v_s1->flags)) {
 			if (s1 == d) {
 				M_ILD(RCX, REG_SP, s2 * 8);
 				emit_shift_membase(cd, shift_op, REG_SP, d * 8);
@@ -469,7 +819,7 @@ void emit_lshift(codegendata *cd, s4 shift_op, stackptr src, instruction *iptr)
 				M_LST(REG_ITMP2, REG_SP, d * 8);
 			}
 
-		} else if ((src->flags & INMEMORY) && !(src->prev->flags & INMEMORY)) {
+		} else if (IS_INMEMORY(v_s2->flags) && !IS_INMEMORY(v_s1->flags)) {
 			/* s1 may be equal to RCX */
 			if (s1 == RCX) {
 				if (s2 == d) {
@@ -489,7 +839,7 @@ void emit_lshift(codegendata *cd, s4 shift_op, stackptr src, instruction *iptr)
 
 			emit_shift_membase(cd, shift_op, REG_SP, d * 8);
 
-		} else if (!(src->flags & INMEMORY) && (src->prev->flags & INMEMORY)) {
+		} else if (!IS_INMEMORY(v_s2->flags) && IS_INMEMORY(v_s1->flags)) {
 			if (s1 == d) {
 				M_INTMOVE(s2, RCX);
 				emit_shift_membase(cd, shift_op, REG_SP, d * 8);
@@ -516,18 +866,18 @@ void emit_lshift(codegendata *cd, s4 shift_op, stackptr src, instruction *iptr)
 			d = REG_ITMP3;
 		}
 
-		if ((src->flags & INMEMORY) && (src->prev->flags & INMEMORY)) {
+		if (IS_INMEMORY(v_s2->flags) && IS_INMEMORY(v_s1->flags)) {
 			M_ILD(RCX, REG_SP, s2 * 8);
 			M_LLD(d, REG_SP, s1 * 8);
 			emit_shift_reg(cd, shift_op, d);
 
-		} else if ((src->flags & INMEMORY) && !(src->prev->flags & INMEMORY)) {
+		} else if (IS_INMEMORY(v_s2->flags) && !IS_INMEMORY(v_s1->flags)) {
 			/* s1 may be equal to RCX */
 			M_INTMOVE(s1, d);
 			M_ILD(RCX, REG_SP, s2 * 8);
 			emit_shift_reg(cd, shift_op, d);
 
-		} else if (!(src->flags & INMEMORY) && (src->prev->flags & INMEMORY)) {
+		} else if (!IS_INMEMORY(v_s2->flags) && IS_INMEMORY(v_s1->flags)) {
 			M_INTMOVE(s2, RCX);
 			M_LLD(d, REG_SP, s1 * 8);
 			emit_shift_reg(cd, shift_op, d);
@@ -610,7 +960,7 @@ void emit_mov_membase_reg(codegendata *cd, s8 basereg, s8 disp, s8 reg) {
 void emit_mov_membase32_reg(codegendata *cd, s8 basereg, s8 disp, s8 reg) {
 	emit_rex(1,(reg),0,(basereg));
 	*(cd->mcodeptr++) = 0x8b;
-	emit_membase32((basereg),(disp),(reg));
+	emit_membase32(cd, (basereg),(disp),(reg));
 }
 
 
@@ -628,7 +978,7 @@ void emit_movl_membase32_reg(codegendata *cd, s8 basereg, s8 disp, s8 reg)
 {
 	emit_byte_rex((reg),0,(basereg));
 	*(cd->mcodeptr++) = 0x8b;
-	emit_membase32((basereg),(disp),(reg));
+	emit_membase32(cd, (basereg),(disp),(reg));
 }
 
 
@@ -642,7 +992,7 @@ void emit_mov_reg_membase(codegendata *cd, s8 reg, s8 basereg, s8 disp) {
 void emit_mov_reg_membase32(codegendata *cd, s8 reg, s8 basereg, s8 disp) {
 	emit_rex(1,(reg),0,(basereg));
 	*(cd->mcodeptr++) = 0x89;
-	emit_membase32((basereg),(disp),(reg));
+	emit_membase32(cd, (basereg),(disp),(reg));
 }
 
 
@@ -658,35 +1008,35 @@ void emit_movl_reg_membase(codegendata *cd, s8 reg, s8 basereg, s8 disp) {
 void emit_movl_reg_membase32(codegendata *cd, s8 reg, s8 basereg, s8 disp) {
 	emit_byte_rex((reg),0,(basereg));
 	*(cd->mcodeptr++) = 0x89;
-	emit_membase32((basereg),(disp),(reg));
+	emit_membase32(cd, (basereg),(disp),(reg));
 }
 
 
 void emit_mov_memindex_reg(codegendata *cd, s8 disp, s8 basereg, s8 indexreg, s8 scale, s8 reg) {
 	emit_rex(1,(reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x8b;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
 void emit_movl_memindex_reg(codegendata *cd, s8 disp, s8 basereg, s8 indexreg, s8 scale, s8 reg) {
 	emit_rex(0,(reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x8b;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
 void emit_mov_reg_memindex(codegendata *cd, s8 reg, s8 disp, s8 basereg, s8 indexreg, s8 scale) {
 	emit_rex(1,(reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x89;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
 void emit_movl_reg_memindex(codegendata *cd, s8 reg, s8 disp, s8 basereg, s8 indexreg, s8 scale) {
 	emit_rex(0,(reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x89;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
@@ -694,14 +1044,14 @@ void emit_movw_reg_memindex(codegendata *cd, s8 reg, s8 disp, s8 basereg, s8 ind
 	*(cd->mcodeptr++) = 0x66;
 	emit_rex(0,(reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x89;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
 void emit_movb_reg_memindex(codegendata *cd, s8 reg, s8 disp, s8 basereg, s8 indexreg, s8 scale) {
 	emit_byte_rex((reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x88;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
@@ -716,7 +1066,7 @@ void emit_mov_imm_membase(codegendata *cd, s8 imm, s8 basereg, s8 disp) {
 void emit_mov_imm_membase32(codegendata *cd, s8 imm, s8 basereg, s8 disp) {
 	emit_rex(1,0,0,(basereg));
 	*(cd->mcodeptr++) = 0xc7;
-	emit_membase32((basereg),(disp),0);
+	emit_membase32(cd, (basereg),(disp),0);
 	emit_imm32((imm));
 }
 
@@ -734,7 +1084,7 @@ void emit_movl_imm_membase(codegendata *cd, s8 imm, s8 basereg, s8 disp) {
 void emit_movl_imm_membase32(codegendata *cd, s8 imm, s8 basereg, s8 disp) {
 	emit_byte_rex(0,0,(basereg));
 	*(cd->mcodeptr++) = 0xc7;
-	emit_membase32((basereg),(disp),0);
+	emit_membase32(cd, (basereg),(disp),0);
 	emit_imm32((imm));
 }
 
@@ -782,7 +1132,7 @@ void emit_movswq_memindex_reg(codegendata *cd, s8 disp, s8 basereg, s8 indexreg,
 	emit_rex(1,(reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0xbf;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
@@ -790,7 +1140,7 @@ void emit_movsbq_memindex_reg(codegendata *cd, s8 disp, s8 basereg, s8 indexreg,
 	emit_rex(1,(reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0xbe;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
@@ -798,7 +1148,7 @@ void emit_movzwq_memindex_reg(codegendata *cd, s8 disp, s8 basereg, s8 indexreg,
 	emit_rex(1,(reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0xb7;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
@@ -806,7 +1156,7 @@ void emit_mov_imm_memindex(codegendata *cd, s4 imm, s4 disp, s4 basereg, s4 inde
 {
 	emit_rex(1,0,(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0xc7;
-	emit_memindex(0,(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, 0,(disp),(basereg),(indexreg),(scale));
 	emit_imm32((imm));
 }
 
@@ -815,7 +1165,7 @@ void emit_movl_imm_memindex(codegendata *cd, s4 imm, s4 disp, s4 basereg, s4 ind
 {
 	emit_rex(0,0,(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0xc7;
-	emit_memindex(0,(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, 0,(disp),(basereg),(indexreg),(scale));
 	emit_imm32((imm));
 }
 
@@ -825,7 +1175,7 @@ void emit_movw_imm_memindex(codegendata *cd, s4 imm, s4 disp, s4 basereg, s4 ind
 	*(cd->mcodeptr++) = 0x66;
 	emit_rex(0,0,(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0xc7;
-	emit_memindex(0,(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, 0,(disp),(basereg),(indexreg),(scale));
 	emit_imm16((imm));
 }
 
@@ -834,7 +1184,7 @@ void emit_movb_imm_memindex(codegendata *cd, s4 imm, s4 disp, s4 basereg, s4 ind
 {
 	emit_rex(0,0,(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0xc6;
-	emit_memindex(0,(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, 0,(disp),(basereg),(indexreg),(scale));
 	emit_imm8((imm));
 }
 
@@ -1543,7 +1893,7 @@ void emit_movd_reg_memindex(codegendata *cd, s8 reg, s8 disp, s8 basereg, s8 ind
 	emit_rex(0,(reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0x7e;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
@@ -1570,7 +1920,7 @@ void emit_movd_memindex_reg(codegendata *cd, s8 disp, s8 basereg, s8 indexreg, s
 	emit_rex(0,(dreg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0x6e;
-	emit_memindex((dreg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (dreg),(disp),(basereg),(indexreg),(scale));
 }
 
 
@@ -1635,7 +1985,7 @@ void emit_movss_reg_membase32(codegendata *cd, s8 reg, s8 basereg, s8 disp) {
 	emit_byte_rex((reg),0,(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0x11;
-	emit_membase32((basereg),(disp),(reg));
+	emit_membase32(cd, (basereg),(disp),(reg));
 }
 
 
@@ -1655,7 +2005,7 @@ void emit_movsd_reg_membase32(codegendata *cd, s8 reg, s8 basereg, s8 disp) {
 	emit_byte_rex((reg),0,(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0x11;
-	emit_membase32((basereg),(disp),(reg));
+	emit_membase32(cd, (basereg),(disp),(reg));
 }
 
 
@@ -1675,7 +2025,7 @@ void emit_movss_membase32_reg(codegendata *cd, s8 basereg, s8 disp, s8 dreg) {
 	emit_byte_rex((dreg),0,(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0x10;
-	emit_membase32((basereg),(disp),(dreg));
+	emit_membase32(cd, (basereg),(disp),(dreg));
 }
 
 
@@ -1713,7 +2063,7 @@ void emit_movsd_membase32_reg(codegendata *cd, s8 basereg, s8 disp, s8 dreg) {
 	emit_byte_rex((dreg),0,(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0x10;
-	emit_membase32((basereg),(disp),(dreg));
+	emit_membase32(cd, (basereg),(disp),(dreg));
 }
 
 
@@ -1742,7 +2092,7 @@ void emit_movss_reg_memindex(codegendata *cd, s8 reg, s8 disp, s8 basereg, s8 in
 	emit_rex(0,(reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0x11;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
@@ -1751,7 +2101,7 @@ void emit_movsd_reg_memindex(codegendata *cd, s8 reg, s8 disp, s8 basereg, s8 in
 	emit_rex(0,(reg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0x11;
-	emit_memindex((reg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (reg),(disp),(basereg),(indexreg),(scale));
 }
 
 
@@ -1760,7 +2110,7 @@ void emit_movss_memindex_reg(codegendata *cd, s8 disp, s8 basereg, s8 indexreg,
 	emit_rex(0,(dreg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0x10;
-	emit_memindex((dreg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (dreg),(disp),(basereg),(indexreg),(scale));
 }
 
 
@@ -1769,7 +2119,7 @@ void emit_movsd_memindex_reg(codegendata *cd, s8 disp, s8 basereg, s8 indexreg,
 	emit_rex(0,(dreg),(indexreg),(basereg));
 	*(cd->mcodeptr++) = 0x0f;
 	*(cd->mcodeptr++) = 0x10;
-	emit_memindex((dreg),(disp),(basereg),(indexreg),(scale));
+	emit_memindex(cd, (dreg),(disp),(basereg),(indexreg),(scale));
 }
 
 
