@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: md-abi.c 4488 2006-02-12 12:59:52Z christian $
+   $Id: md-abi.c 5634 2006-10-02 14:18:04Z edwin $
 
 */
 
@@ -89,24 +89,27 @@ void md_param_alloc(methoddesc *md)
 		case TYPE_LNG:
 			if (iarg < INT_ARG_CNT) {
 				pd->inmemory = false;
-				pd->regoff = iarg;
-			} else {
+				pd->regoff   = iarg;
+			}
+			else {
 				pd->inmemory = true;
-				pd->regoff = stacksize;
+				pd->regoff   = stacksize;
 			}
 			if (iarg < INT_ARG_CNT)
 				iarg++;
 			else
 				stacksize++;
 			break;
+
 		case TYPE_FLT:
 		case TYPE_DBL:
 			if (farg < FLT_ARG_CNT) {
 				pd->inmemory = false;
-				pd->regoff = farg;
-			} else {
+				pd->regoff   = farg;
+			}
+			else {
 				pd->inmemory = true;
-				pd->regoff = stacksize;
+				pd->regoff   = stacksize;
 			}
 			if (farg < FLT_ARG_CNT)
 				farg++;
@@ -116,8 +119,9 @@ void md_param_alloc(methoddesc *md)
 		}
 	}
 
-	/* Since XMM0 (==A0) is used for passing return values, this */
-	/* argument register usage has to be regarded, too           */
+	/* Since XMM0 (==A0) is used for passing return values, this
+	   argument register usage has to be regarded, too. */
+
 	if (IS_FLT_DBL_TYPE(md->returntype.type))
 		if (farg < 1)
 			farg = 1;
@@ -126,7 +130,7 @@ void md_param_alloc(methoddesc *md)
 
 	md->argintreguse = iarg;
 	md->argfltreguse = farg;
-	md->memuse = stacksize;
+	md->memuse       = stacksize;
 }
 
 
@@ -138,48 +142,55 @@ void md_param_alloc(methoddesc *md)
    return register.
 
    --- in
-   m:                       Methodinfo of current method
-   return_type:             Return Type of the Method (TYPE_INT.. TYPE_ADR)
-   							TYPE_VOID is not allowed!
+   jd:                      jitdata of the current method
    stackslot:               Java Stackslot to contain the Return Value
 
    --- out
    if precoloring was possible:
-   stackslot->varkind       =ARGVAR
-   			->varnum        =-1
-   			->flags         =0
-   			->regoff        =[REG_RESULT, (REG_RESULT2/REG_RESULT), REG_FRESULT]
+   VAR(stackslot->varnum)->flags     = PREALLOC
+   			             ->vv.regoff = [REG_RESULT|REG_FRESULT]
    rd->arg[flt|int]reguse   set to a value according the register usage
 
+   NOTE: Do not pass a LOCALVAR in stackslot->varnum.
+
 *******************************************************************************/
 
-void md_return_alloc(methodinfo *m, registerdata *rd, s4 return_type,
-					 stackptr stackslot)
+void md_return_alloc(jitdata *jd, stackptr stackslot)
 {
+	methodinfo   *m;
+	registerdata *rd;
+	methoddesc   *md;
+
+	/* get required compiler data */
+
+	m  = jd->m;
+	rd = jd->rd;
+
+	md = m->parseddesc;
+
 	/* precoloring only straightforward possible with flt/dbl types
 	   For Address/Integer/Long REG_RESULT == rax == REG_ITMP1 and so
 	   could be destroyed if the return value Stack Slot "lives too
 	   long" */
 
-	if (IS_FLT_DBL_TYPE(return_type)) {
+	if (IS_FLT_DBL_TYPE(md->returntype.type)) {
 		/* In Leafmethods Local Vars holding parameters are precolored
 		   to their argument register -> so leafmethods with
 		   paramcount > 0 could already use a00! */
 
-		if (!m->isleafmethod || (m->parseddesc->paramcount == 0)) {
+		if (!jd->isleafmethod || (md->paramcount == 0)) {
 			/* Only precolor the stackslot, if it is not a SAVEDVAR
 			   <-> has not to survive method invokations */
 
 			if (!(stackslot->flags & SAVEDVAR)) {
-				stackslot->varkind = ARGVAR;
-				stackslot->varnum = -1;
-				stackslot->flags = 0;
+
+				VAR(stackslot->varnum)->flags = PREALLOC;
 
 			    /* float/double */
 				if (rd->argfltreguse < 1)
 					rd->argfltreguse = 1;
 
-				stackslot->regoff = REG_FRESULT;
+				VAR(stackslot->varnum)->vv.regoff = REG_FRESULT;
 			}
 		}
 	}
