@@ -28,7 +28,7 @@
 
    Changes: Edwin Steiner
 
-   $Id: md.c 4921 2006-05-15 14:24:36Z twisti $
+   $Id: md.c 5233 2006-08-14 10:59:39Z twisti $
 
 */
 
@@ -43,6 +43,10 @@
 
 #include "vm/jit/x86_64/md-abi.h"
 
+#if defined(ENABLE_THREADS)
+# include "threads/native/threads.h"
+#endif
+
 #include "vm/exceptions.h"
 #include "vm/signallocal.h"
 #include "vm/jit/asmpart.h"
@@ -75,11 +79,11 @@ void md_init(void)
 
 void md_signal_handler_sigsegv(int sig, siginfo_t *siginfo, void *_p)
 {
-	ucontext_t  *_uc;
-	mcontext_t  *_mc;
-	u1          *sp;
-	u1          *ra;
-	u1          *xpc;
+	ucontext_t *_uc;
+	mcontext_t *_mc;
+	u1         *sp;
+	u1         *ra;
+	u1         *xpc;
 
 	_uc = (ucontext_t *) _p;
 	_mc = &_uc->uc_mcontext;
@@ -91,6 +95,12 @@ void md_signal_handler_sigsegv(int sig, siginfo_t *siginfo, void *_p)
 	xpc = (u1 *) _mc->gregs[REG_RIP];
 	ra  = xpc;                          /* return address is equal to xpc     */
 
+#if 0
+	/* check for StackOverflowException */
+
+	threads_check_stackoverflow(sp);
+#endif
+
 	_mc->gregs[REG_RAX] =
 		(ptrint) stacktrace_hardware_nullpointerexception(NULL, sp, ra, xpc);
 
@@ -132,15 +142,47 @@ void md_signal_handler_sigfpe(int sig, siginfo_t *siginfo, void *_p)
 }
 
 
+/* md_signal_handler_sigusr2 ***************************************************
+
+   Signal handler for profiling sampling.
+
+*******************************************************************************/
+
 #if defined(ENABLE_THREADS)
-void thread_restartcriticalsection(ucontext_t *uc)
+void md_signal_handler_sigusr2(int sig, siginfo_t *siginfo, void *_p)
 {
-	void *critical;
+	threadobject *t;
+	ucontext_t   *_uc;
+	mcontext_t   *_mc;
+	u1           *pc;
 
-	critical = critical_find_restart_point((void *) uc->uc_mcontext.gregs[REG_RIP]);
+	t = THREADOBJECT;
 
-	if (critical)
-		uc->uc_mcontext.gregs[REG_RIP] = (ptrint) critical;
+	_uc = (ucontext_t *) _p;
+	_mc = &_uc->uc_mcontext;
+
+	/* ATTENTION: Don't use CACAO's internal REG_* defines as they are
+	   different to the ones in <ucontext.h>. */
+
+	pc = (u1 *) _mc->gregs[REG_RIP];
+
+	t->pc = pc;
+}
+#endif
+
+
+#if defined(ENABLE_THREADS)
+void thread_restartcriticalsection(ucontext_t *_uc)
+{
+	mcontext_t *_mc;
+	void       *pc;
+
+	_mc = &_uc->uc_mcontext;
+
+	pc = critical_find_restart_point((void *) _mc->gregs[REG_RIP]);
+
+	if (pc != NULL)
+		_mc->gregs[REG_RIP] = (ptrint) pc;
 }
 #endif
 
@@ -172,7 +214,7 @@ u1 *md_stacktrace_get_returnaddress(u1 *sp, u4 framesize)
 
    INVOKESTATIC/SPECIAL:
 
-   49 ba 98 3a ed ab aa 2a 00 00    mov    $0x2aaaabed3a98,%r10
+   4d 8b 15 e2 fe ff ff             mov    -286(%rip),%r10
    49 ff d2                         rex64Z callq  *%r10
 
    INVOKEVIRTUAL:
@@ -206,14 +248,19 @@ u1 *md_get_method_patch_address(u1 *ra, stackframeinfo *sfi, u1 *mptr)
 
 	/* check for the different calls */
 
-	/* INVOKESTATIC/SPECIAL */
-
 	if (mcode == 0xd2) {
-		/* patch address is 8-bytes before the call instruction */
+		/* INVOKESTATIC/SPECIAL */
 
-		pa = ra - 8;
+		/* Get the offset from the instruction (the offset address is
+		   4-bytes before the call instruction). */
 
-	} else if (mcode == 0xd3) {
+		offset = *((s4 *) (ra - 4));
+
+		/* add the offset to the return address (IP-relative addressing) */
+
+		pa = ra + offset;
+	}
+	else if (mcode == 0xd3) {
 		/* INVOKEVIRTUAL/INTERFACE */
 
 		/* Get the offset from the instruction (the offset address is
@@ -224,8 +271,8 @@ u1 *md_get_method_patch_address(u1 *ra, stackframeinfo *sfi, u1 *mptr)
 		/* add the offset to the method pointer */
 
 		pa = mptr + offset;
-
-	} else {
+	}
+	else {
 		/* catch any problems */
 
 		assert(0);
@@ -235,20 +282,21 @@ u1 *md_get_method_patch_address(u1 *ra, stackframeinfo *sfi, u1 *mptr)
 }
 
 
-/* md_codegen_findmethod *******************************************************
+/* md_codegen_get_pv_from_pc ***************************************************
 
-   On this architecture just a wrapper function to codegen_findmethod.
+   On this architecture just a wrapper function to
+   codegen_get_pv_from_pc.
 
 *******************************************************************************/
 
-u1 *md_codegen_findmethod(u1 *ra)
+u1 *md_codegen_get_pv_from_pc(u1 *ra)
 {
 	u1 *pv;
 
-	/* the the start address of the function which contains this
-       address from the method table */
+	/* Get the start address of the function which contains this
+       address from the method table. */
 
-	pv = codegen_findmethod(ra);
+	pv = codegen_get_pv_from_pc(ra);
 
 	return pv;
 }
