@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: patcher.c 4530 2006-02-21 09:11:53Z twisti $
+   $Id: patcher.c 5233 2006-08-14 10:59:39Z twisti $
 
 */
 
@@ -36,10 +36,13 @@
 #include "config.h"
 #include "vm/types.h"
 
+#include "vm/jit/x86_64/codegen.h"
+
 #include "mm/memory.h"
 #include "native/native.h"
 #include "vm/builtin.h"
 #include "vm/class.h"
+#include "vm/exceptions.h"
 #include "vm/field.h"
 #include "vm/initialize.h"
 #include "vm/options.h"
@@ -48,6 +51,82 @@
 #include "vm/jit/patcher.h"
 
 
+/* patcher_wrapper *************************************************************
+
+   Wrapper for all patchers.  It also creates the stackframe info
+   structure.
+
+   If the return value of the patcher function is false, it gets the
+   exception object, clears the exception pointer and returns the
+   exception.
+
+*******************************************************************************/
+
+java_objectheader *patcher_wrapper(u1 *sp, u1 *pv, u1 *ra)
+{
+	stackframeinfo     sfi;
+	u1                *xpc;
+	java_objectheader *o;
+	functionptr        f;
+	bool               result;
+	java_objectheader *e;
+
+	/* define the patcher function */
+
+	bool (*patcher_function)(u1 *);
+
+	/* get stuff from the stack */
+
+	xpc = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	o   = (java_objectheader *) *((ptrint *) (sp + 4 * 8));
+	f   = (functionptr)         *((ptrint *) (sp + 0 * 8));
+
+	/* calculate and set the new return address */
+
+	xpc = xpc - PATCHER_CALL_SIZE;
+
+	*((ptrint *) (sp + 5 * 8)) = (ptrint) xpc;
+
+	/* cast the passed function to a patcher function */
+
+	patcher_function = (bool (*)(u1 *)) (ptrint) f;
+
+	/* enter a monitor on the patching position */
+
+	PATCHER_MONITORENTER;
+
+	/* create the stackframeinfo */
+
+	/* RA is passed as NULL, but the XPC is correct and can be used in
+	   stacktrace_create_extern_stackframeinfo for
+	   md_codegen_get_pv_from_pc. */
+
+	stacktrace_create_extern_stackframeinfo(&sfi, pv, sp + 6 * 8, xpc, xpc);
+
+	/* call the proper patcher function */
+
+	result = (patcher_function)(sp);
+
+	/* remove the stackframeinfo */
+
+	stacktrace_remove_stackframeinfo(&sfi);
+
+	/* check for return value and exit accordingly */
+
+	if (result == false) {
+		e = exceptions_get_and_clear_exception();
+
+		PATCHER_MONITOREXIT;
+
+		return e;
+	}
+
+	PATCHER_MARK_PATCHED_MONITOREXIT;
+
+	return NULL;
+}
+
+
 /* patcher_get_putstatic *******************************************************
 
    Machine code:
@@ -60,46 +139,29 @@
 
 bool patcher_get_putstatic(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-	u8                 mcode;
-	unresolved_field  *uf;
-	s4                 disp;
-	fieldinfo         *fi;
-	s4                 offset;
+	u1               *ra;
+	u8                mcode;
+	unresolved_field *uf;
+	s4                disp;
+	fieldinfo        *fi;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	uf    = (unresolved_field *)  *((ptrint *) (sp + 1 * 8));
-	disp  =                       *((s4 *)     (sp + 0 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)               *((ptrint *) (sp + 5 * 8));
+	mcode =                      *((u8 *)     (sp + 3 * 8));
+	uf    = (unresolved_field *) *((ptrint *) (sp + 2 * 8));
+	disp  =                      *((s4 *)     (sp + 1 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(fi = resolve_field_eager(uf))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(fi = resolve_field_eager(uf)))
 		return false;
-	}
 
 	/* check if the field's class is initialized */
 
-	if (!(fi->class->state & CLASS_INITIALIZED)) {
-		if (!initialize_class(fi->class)) {
-			PATCHER_MONITOREXIT;
-
+	if (!(fi->class->state & CLASS_INITIALIZED))
+		if (!initialize_class(fi->class))
 			return false;
-		}
-	}
 
 	/* patch back original code */
 
@@ -110,15 +172,9 @@ bool patcher_get_putstatic(u1 *sp)
 	if (opt_showdisassemble)
 		ra = ra + 5;
 
-	/* get RIP offset from machine instruction */
+	/* patch the field value's address */
 
-	offset = *((u4 *) (ra + 3));
-
-	/* patch the field value's address (+ 7: is the size of the RIP move) */
-
-	*((ptrint *) (ra + 7 + offset)) = (ptrint) &(fi->value);
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
+	*((ptrint *) (ra + 7 + disp)) = (ptrint) &(fi->value);
 
 	return true;
 }
@@ -135,34 +191,22 @@ bool patcher_get_putstatic(u1 *sp)
 
 bool patcher_get_putfield(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-	u8                 mcode;
-	unresolved_field  *uf;
-	fieldinfo         *fi;
-	u1                 byte;
+	u1               *ra;
+	u8                mcode;
+	unresolved_field *uf;
+	fieldinfo        *fi;
+	u1                byte;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	uf    = (unresolved_field *)  *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)               *((ptrint *) (sp + 5 * 8));
+	mcode =                      *((u8 *)     (sp + 3 * 8));
+	uf    = (unresolved_field *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(fi = resolve_field_eager(uf))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(fi = resolve_field_eager(uf)))
 		return false;
-	}
 
 	/* patch back original code (instruction code is smaller than 8 bytes) */
 
@@ -186,8 +230,8 @@ bool patcher_get_putfield(u1 *sp)
 			*((u4 *) (ra + 4)) = (u4) (fi->offset);
 		else
 			*((u4 *) (ra + 3)) = (u4) (fi->offset);
-
-	} else {
+	}
+	else {
 		/* check for special case: %rsp or %r12 as base register */
 
 		byte = *(ra + 5);
@@ -198,8 +242,6 @@ bool patcher_get_putfield(u1 *sp)
 			*((u4 *) (ra + 5)) = (u4) (fi->offset);
 	}
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -215,33 +257,21 @@ bool patcher_get_putfield(u1 *sp)
 
 bool patcher_putfieldconst(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-	u8                 mcode;
-	unresolved_field  *uf;
-	fieldinfo         *fi;
+	u1               *ra;
+	u8                mcode;
+	unresolved_field *uf;
+	fieldinfo        *fi;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	uf    = (unresolved_field *)  *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)               *((ptrint *) (sp + 5 * 8));
+	mcode =                      *((u8 *)     (sp + 3 * 8));
+	uf    = (unresolved_field *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(fi = resolve_field_eager(uf))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(fi = resolve_field_eager(uf)))
 		return false;
-	}
 
 	/* patch back original code */
 
@@ -258,15 +288,15 @@ bool patcher_putfieldconst(u1 *sp)
 		/* handle special case when the base register is %r12 */
 
 		if (*(ra + 2) == 0x84) {
-			*((u4 *) (ra + 4)) = (u4) (fi->offset);
+			*((u4 *) (ra + 4))      = (u4) (fi->offset);
 			*((u4 *) (ra + 12 + 4)) = (u4) (fi->offset + 4);
-
-		} else {
-			*((u4 *) (ra + 3)) = (u4) (fi->offset);
+		}
+		else {
+			*((u4 *) (ra + 3))      = (u4) (fi->offset);
 			*((u4 *) (ra + 11 + 3)) = (u4) (fi->offset + 4);
 		}
-
-	} else {
+	}
+	else {
 		/* handle special case when the base register is %r12 */
 
 		if (*(ra + 2) == 0x84)
@@ -275,8 +305,6 @@ bool patcher_putfieldconst(u1 *sp)
 			*((u4 *) (ra + 3)) = (u4) (fi->offset);
 	}
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -293,32 +321,20 @@ bool patcher_putfieldconst(u1 *sp)
 bool patcher_aconst(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
 	u8                 mcode;
 	constant_classref *cr;
 	classinfo         *c;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	mcode =                       *((u8 *)     (sp + 3 * 8));
+	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the classinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
@@ -333,8 +349,6 @@ bool patcher_aconst(u1 *sp)
 
 	*((ptrint *) (ra + 2)) = (ptrint) c;
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -355,32 +369,20 @@ bool patcher_aconst(u1 *sp)
 bool patcher_builtin_multianewarray(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
 	u8                 mcode;
 	constant_classref *cr;
 	classinfo         *c;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	mcode =                       *((u8 *)     (sp + 3 * 8));
+	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the classinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
@@ -395,12 +397,6 @@ bool patcher_builtin_multianewarray(u1 *sp)
 
 	*((ptrint *) (ra + 10 + 2)) = (ptrint) c;
 
-	/* patch new function address */
-
-	*((ptrint *) (ra + 10 + 10 + 3 + 2)) = (ptrint) BUILTIN_multianewarray;
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -419,32 +415,20 @@ bool patcher_builtin_multianewarray(u1 *sp)
 bool patcher_builtin_arraycheckcast(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
 	u8                 mcode;
 	constant_classref *cr;
 	classinfo         *c;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	mcode =                       *((u8 *)     (sp + 3 * 8));
+	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the classinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
@@ -459,8 +443,6 @@ bool patcher_builtin_arraycheckcast(u1 *sp)
 
 	*((ptrint *) (ra + 2)) = (ptrint) c;
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -478,32 +460,23 @@ bool patcher_builtin_arraycheckcast(u1 *sp)
 bool patcher_invokestatic_special(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
 	u8                 mcode;
 	unresolved_method *um;
+	s4                 disp;
 	methodinfo        *m;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	um    = (unresolved_method *) *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	mcode =                       *((u8 *)     (sp + 3 * 8));
+	um    = (unresolved_method *) *((ptrint *) (sp + 2 * 8));
+	disp  =                       *((s4 *)     (sp + 1 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(m = resolve_method_eager(um))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(m = resolve_method_eager(um)))
 		return false;
-	}
+
 	/* patch back original code */
 
 	*((u8 *) ra) = mcode;
@@ -515,9 +488,8 @@ bool patcher_invokestatic_special(u1 *sp)
 
 	/* patch stubroutine */
 
-	*((ptrint *) (ra + 2)) = (ptrint) m->stubroutine;
-
-	PATCHER_MARK_PATCHED_MONITOREXIT;
+/* 	*((ptrint *) (ra + 2)) = (ptrint) m->stubroutine; */
+	*((ptrint *) (ra + 7 + disp)) = (ptrint) m->stubroutine;
 
 	return true;
 }
@@ -537,32 +509,20 @@ bool patcher_invokestatic_special(u1 *sp)
 bool patcher_invokevirtual(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
 	u8                 mcode;
 	unresolved_method *um;
 	methodinfo        *m;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	um    = (unresolved_method *) *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	mcode =                       *((u8 *)     (sp + 3 * 8));
+	um    = (unresolved_method *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(m = resolve_method_eager(um))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(m = resolve_method_eager(um)))
 		return false;
-	}
 
 	/* patch back original code */
 
@@ -578,8 +538,6 @@ bool patcher_invokevirtual(u1 *sp)
 	*((s4 *) (ra + 3 + 3)) = (s4) (OFFSET(vftbl_t, table[0]) +
 								   sizeof(methodptr) * m->vftblindex);
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -599,32 +557,20 @@ bool patcher_invokevirtual(u1 *sp)
 bool patcher_invokeinterface(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
 	u8                 mcode;
 	unresolved_method *um;
 	methodinfo        *m;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	um    = (unresolved_method *) *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	mcode =                       *((u8 *)     (sp + 3 * 8));
+	um    = (unresolved_method *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(m = resolve_method_eager(um))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(m = resolve_method_eager(um)))
 		return false;
-	}
 
 	/* patch back original code */
 
@@ -645,8 +591,6 @@ bool patcher_invokeinterface(u1 *sp)
 	*((s4 *) (ra + 3 + 7 + 3)) =
 		(s4) (sizeof(methodptr) * (m - m->class->methods));
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -665,32 +609,20 @@ bool patcher_invokeinterface(u1 *sp)
 bool patcher_checkcast_instanceof_flags(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
 	u8                 mcode;
 	constant_classref *cr;
 	classinfo         *c;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	mcode =                       *((u8 *)     (sp + 3 * 8));
+	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
@@ -705,8 +637,6 @@ bool patcher_checkcast_instanceof_flags(u1 *sp)
 
 	*((s4 *) (ra + 2)) = (s4) c->flags;
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -727,32 +657,20 @@ bool patcher_checkcast_instanceof_flags(u1 *sp)
 bool patcher_checkcast_instanceof_interface(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
 	u8                 mcode;
 	constant_classref *cr;
 	classinfo         *c;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	mcode =                       *((u8 *)     (sp + 3 * 8));
+	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
@@ -771,8 +689,6 @@ bool patcher_checkcast_instanceof_interface(u1 *sp)
 		(s4) (OFFSET(vftbl_t, interfacetable[0]) -
 			  c->index * sizeof(methodptr*));
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -793,32 +709,20 @@ bool patcher_checkcast_instanceof_interface(u1 *sp)
 bool patcher_checkcast_class(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
 	u8                 mcode;
 	constant_classref *cr;
 	classinfo         *c;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	mcode =                       *((u8 *)     (sp + 3 * 8));
+	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
@@ -834,8 +738,6 @@ bool patcher_checkcast_class(u1 *sp)
 	*((ptrint *) (ra + 2)) = (ptrint) c->vftbl;
 	*((ptrint *) (ra + 10 + 7 + 7 + 3 + 2)) = (ptrint) c->vftbl;
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -852,32 +754,20 @@ bool patcher_checkcast_class(u1 *sp)
 bool patcher_instanceof_class(u1 *sp)
 {
 	u1                *ra;
-	java_objectheader *o;
 	u8                 mcode;
 	constant_classref *cr;
 	classinfo         *c;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	cr    = (constant_classref *) *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)                *((ptrint *) (sp + 5 * 8));
+	mcode =                       *((u8 *)     (sp + 3 * 8));
+	cr    = (constant_classref *) *((ptrint *) (sp + 2 * 8));
 
 	/* get the fieldinfo */
 
-	if (!(c = resolve_classref_eager(cr))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(c = resolve_classref_eager(cr)))
 		return false;
-	}
 
 	/* patch back original code */
 
@@ -892,8 +782,6 @@ bool patcher_instanceof_class(u1 *sp)
 
 	*((ptrint *) (ra + 2)) = (ptrint) c->vftbl;
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -912,41 +800,26 @@ bool patcher_instanceof_class(u1 *sp)
 
 bool patcher_clinit(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-	u8                 mcode;
-	classinfo         *c;
+	u1        *ra;
+	u8         mcode;
+	classinfo *c;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	c     = (classinfo *)         *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)        *((ptrint *) (sp + 5 * 8));
+	mcode =               *((u8 *)     (sp + 3 * 8));
+	c     = (classinfo *) *((ptrint *) (sp + 2 * 8));
 
 	/* check if the class is initialized */
 
-	if (!(c->state & CLASS_INITIALIZED)) {
-		if (!initialize_class(c)) {
-			PATCHER_MONITOREXIT;
-
+	if (!(c->state & CLASS_INITIALIZED))
+		if (!initialize_class(c))
 			return false;
-		}
-	}
 
 	/* patch back original code */
 
 	*((u8 *) ra) = mcode;
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 
@@ -962,40 +835,26 @@ bool patcher_clinit(u1 *sp)
 #ifdef ENABLE_VERIFIER
 bool patcher_athrow_areturn(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-	u8                 mcode;
-	unresolved_class  *uc;
-	classinfo         *c;
+	u1               *ra;
+	u8                mcode;
+	unresolved_class *uc;
+	classinfo        *c;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	uc    = (unresolved_class *)  *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)               *((ptrint *) (sp + 5 * 8));
+	mcode =                      *((u8 *)     (sp + 3 * 8));
+	uc    = (unresolved_class *) *((ptrint *) (sp + 2 * 8));
 
 	/* resolve the class */
 
-	if (!resolve_class(uc, resolveEager, false, &c)) {
-		PATCHER_MONITOREXIT;
-
+	if (!resolve_class(uc, resolveEager, false, &c))
 		return false;
-	}
 
 	/* patch back original code */
 
 	*((u8 *) ra) = mcode;
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 #endif /* ENABLE_VERIFIER */
@@ -1014,33 +873,21 @@ bool patcher_athrow_areturn(u1 *sp)
 #if !defined(WITH_STATIC_CLASSPATH)
 bool patcher_resolve_native(u1 *sp)
 {
-	u1                *ra;
-	java_objectheader *o;
-	u8                 mcode;
-	methodinfo        *m;
-	functionptr        f;
+	u1          *ra;
+	u8           mcode;
+	methodinfo  *m;
+	functionptr  f;
 
 	/* get stuff from the stack */
 
-	ra    = (u1 *)                *((ptrint *) (sp + 4 * 8));
-	o     = (java_objectheader *) *((ptrint *) (sp + 3 * 8));
-	mcode =                       *((u8 *)     (sp + 2 * 8));
-	m     = (methodinfo *)        *((ptrint *) (sp + 1 * 8));
-
-	/* calculate and set the new return address */
-
-	ra = ra - 5;
-	*((ptrint *) (sp + 4 * 8)) = (ptrint) ra;
-
-	PATCHER_MONITORENTER;
+	ra    = (u1 *)         *((ptrint *) (sp + 5 * 8));
+	mcode =                *((u8 *)     (sp + 3 * 8));
+	m     = (methodinfo *) *((ptrint *) (sp + 2 * 8));
 
 	/* resolve native function */
 
-	if (!(f = native_resolve_function(m))) {
-		PATCHER_MONITOREXIT;
-
+	if (!(f = native_resolve_function(m)))
 		return false;
-	}
 
 	/* patch back original code */
 
@@ -1055,8 +902,6 @@ bool patcher_resolve_native(u1 *sp)
 
 	*((ptrint *) (ra + 2)) = (ptrint) f;
 
-	PATCHER_MARK_PATCHED_MONITOREXIT;
-
 	return true;
 }
 #endif /* !defined(WITH_STATIC_CLASSPATH) */
