@@ -32,7 +32,7 @@
             Edwin Steiner
             Christian Thalinger
 
-   $Id: linker.c 4973 2006-05-29 15:21:46Z edwin $
+   $Id: linker.c 5868 2006-10-30 11:21:36Z edwin $
 
 */
 
@@ -45,7 +45,13 @@
 
 #include "mm/memory.h"
 #include "native/native.h"
-#include "vm/builtin.h"
+
+#if defined(ENABLE_THREADS)
+# include "threads/native/lock.h"
+#else
+# include "threads/none/lock.h"
+#endif
+
 #include "vm/class.h"
 #include "vm/classcache.h"
 #include "vm/exceptions.h"
@@ -56,6 +62,8 @@
 #include "vm/stringlocal.h"
 #include "vm/access.h"
 #include "vm/rt-timing.h"
+#include "vm/vm.h"
+#include "vm/jit/asmpart.h"
 
 
 /* global variables ***********************************************************/
@@ -189,6 +197,9 @@ bool linker_init(void)
 	if (!link_class(class_java_lang_ThreadGroup))
 		return false;
 
+	if (!link_class(class_java_lang_VMSystem))
+		return false;
+
 	if (!link_class(class_java_lang_VMThread))
 		return false;
 
@@ -364,23 +375,17 @@ classinfo *link_class(classinfo *c)
 
 	RT_TIMING_GET_TIME(time_start);
 
-	if (!c) {
+	if (c == NULL) {
 		exceptions_throw_nullpointerexception();
 		return NULL;
 	}
 
-#if defined(ENABLE_THREADS)
-	/* enter a monitor on the class */
-
-	builtin_monitorenter((java_objectheader *) c);
-#endif
+	LOCK_MONITOR_ENTER(c);
 
 	/* maybe the class is already linked */
 
 	if (c->state & CLASS_LINKED) {
-#if defined(ENABLE_THREADS)
-		builtin_monitorexit((java_objectheader *) c);
-#endif
+		LOCK_MONITOR_EXIT(c);
 
 		return c;
 	}
@@ -414,11 +419,7 @@ classinfo *link_class(classinfo *c)
 		compilingtime_start();
 #endif
 
-#if defined(ENABLE_THREADS)
-	/* leave the monitor */
-
-	builtin_monitorexit((java_objectheader *) c);
-#endif
+	LOCK_MONITOR_EXIT(c);
 
 	RT_TIMING_GET_TIME(time_end);
 
@@ -444,13 +445,13 @@ static classinfo *link_class_intern(classinfo *c)
 	s4 vftbllength;               /* vftbllength of current class             */
 	s4 interfacetablelength;      /* interface table length                   */
 	vftbl_t *v;                   /* vftbl of current class                   */
-	s4 i,j;                       /* interface/method/field counter           */
+	s4 i;                         /* interface/method/field counter           */
 	arraydescriptor *arraydesc;   /* descriptor for array classes             */
 #if defined(ENABLE_RT_TIMING)
 	struct timespec time_start, time_resolving, time_compute_vftbl,
 					time_abstract, time_compute_iftbl, time_fill_vftbl,
 					time_offsets, time_fill_iftbl, time_finalizer,
-					time_exceptions, time_subclasses;
+					time_subclasses;
 #endif
 
 	RT_TIMING_GET_TIME(time_start);
@@ -660,10 +661,11 @@ static classinfo *link_class_intern(classinfo *c)
 	RT_TIMING_GET_TIME(time_compute_vftbl);
 
 
-	/* Check all interfaces of an abtract class (maybe be an interface
-	   too) for unimplemented methods.  Such methods are called
-	   miranda-methods and are marked with the ACC_MIRANDA flag.
-	   VMClass.getDeclaredMethods does not return such methods. */
+	/* Check all interfaces of an abstract class (maybe be an
+	   interface too) for unimplemented methods.  Such methods are
+	   called miranda-methods and are marked with the ACC_MIRANDA
+	   flag.  VMClass.getDeclaredMethods does not return such
+	   methods. */
 
 	if (c->flags & ACC_ABSTRACT) {
 		classinfo  *ic;
@@ -674,7 +676,7 @@ static classinfo *link_class_intern(classinfo *c)
 
 		abstractmethodscount = 0;
 
-		/* check all interfaces of the abtract class */
+		/* check all interfaces of the abstract class */
 
 		for (i = 0; i < c->interfacescount; i++) {
 			ic = c->interfaces[i].cls;
@@ -757,14 +759,14 @@ static classinfo *link_class_intern(classinfo *c)
 	/* compute interfacetable length */
 
 	interfacetablelength = 0;
-	tc = c;
-	while (tc) {
+
+	for (tc = c; tc != NULL; tc = tc->super.cls) {
 		for (i = 0; i < tc->interfacescount; i++) {
 			s4 h = class_highestinterface(tc->interfaces[i].cls) + 1;
+
 			if (h > interfacetablelength)
 				interfacetablelength = h;
 		}
-		tc = tc->super.cls;
 	}
 	RT_TIMING_GET_TIME(time_compute_iftbl);
 
@@ -775,11 +777,12 @@ static classinfo *link_class_intern(classinfo *c)
 							  sizeof(methodptr*) * (interfacetablelength - (interfacetablelength > 0)));
 	v = (vftbl_t *) (((methodptr *) v) +
 					 (interfacetablelength - 1) * (interfacetablelength > 1));
-	c->vftbl = v;
-	v->class = c;
-	v->vftbllength = vftbllength;
+
+	c->vftbl                = v;
+	v->class                = c;
+	v->vftbllength          = vftbllength;
 	v->interfacetablelength = interfacetablelength;
-	v->arraydesc = arraydesc;
+	v->arraydesc            = arraydesc;
 
 	/* store interface index in vftbl */
 
@@ -790,30 +793,57 @@ static classinfo *link_class_intern(classinfo *c)
 
 	for (i = 0; i < supervftbllength; i++) 
 		v->table[i] = super->vftbl->table[i];
-	
+
+	/* Fill the remaining vftbl slots with the AbstractMethodError
+	   stub (all after the super class slots, because they are already
+	   initialized). */
+
+	for (; i < vftbllength; i++) {
+#if defined(ENABLE_JIT)
+# if defined(ENABLE_INTRP)
+		if (opt_intrp)
+			v->table[i] = (methodptr) (ptrint) &intrp_asm_abstractmethoderror;
+		else
+# endif
+			v->table[i] = (methodptr) (ptrint) &asm_abstractmethoderror;
+#else
+		v->table[i] = (methodptr) (ptrint) &intrp_asm_abstractmethoderror;
+#endif
+	}
+
 	/* add method stubs into virtual function table */
 
 	for (i = 0; i < c->methodscount; i++) {
 		methodinfo *m = &(c->methods[i]);
 
-		/* Methods in ABSTRACT classes from interfaces maybe already
-		   have a stubroutine. */
+		assert(m->stubroutine == NULL);
+
+		/* Don't create a compiler stub for abstract methods as they
+		   throw an AbstractMethodError with the default stub in the
+		   vftbl.  This entry is simply copied by sub-classes. */
+
+		if (m->flags & ACC_ABSTRACT)
+			continue;
 
-		if (!m->stubroutine) {
 #if defined(ENABLE_JIT)
 # if defined(ENABLE_INTRP)
-			if (opt_intrp)
-				m->stubroutine = intrp_createcompilerstub(m);
-			else
-#endif
-				m->stubroutine = createcompilerstub(m);
-#else
+		if (opt_intrp)
 			m->stubroutine = intrp_createcompilerstub(m);
+		else
+#endif
+			m->stubroutine = createcompilerstub(m);
+#else
+		m->stubroutine = intrp_createcompilerstub(m);
 #endif
-		}
 
-		if (!(m->flags & ACC_STATIC))
-			v->table[m->vftblindex] = (methodptr) (ptrint) m->stubroutine;
+		/* static methods are not in the vftbl */
+
+		if (m->flags & ACC_STATIC)
+			continue;
+
+		/* insert the stubroutine into the vftbl */
+
+		v->table[m->vftblindex] = (methodptr) (ptrint) m->stubroutine;
 	}
 	RT_TIMING_GET_TIME(time_fill_vftbl);
 
@@ -832,9 +862,9 @@ static classinfo *link_class_intern(classinfo *c)
 			/* header structs like java_lang_Double must match the offsets */
 			/* of the Java fields (eg. java.lang.Double.value).            */
 #if defined(__I386__)
-			c->instancesize = ALIGN(c->instancesize, 4);
+			c->instancesize = MEMORY_ALIGN(c->instancesize, 4);
 #else
-			c->instancesize = ALIGN(c->instancesize, dsize);
+			c->instancesize = MEMORY_ALIGN(c->instancesize, dsize);
 #endif
 
 			f->offset = c->instancesize;
@@ -879,23 +909,6 @@ static classinfo *link_class_intern(classinfo *c)
 	}
 	RT_TIMING_GET_TIME(time_finalizer);
 
-	/* resolve exception class references */
-
-	for (i = 0; i < c->methodscount; i++) {
-		methodinfo *m = &(c->methods[i]);
-
-		for (j = 0; j < m->exceptiontablelength; j++) {
-			if (!m->exceptiontable[j].catchtype.any)
-				continue;
-			if (!resolve_classref_or_classinfo(NULL,
-											   m->exceptiontable[j].catchtype,
-											   resolveEager, true, false,
-											   &(m->exceptiontable[j].catchtype.cls)))
-				return NULL;
-		}
-	}
-	RT_TIMING_GET_TIME(time_exceptions);
-	
 	/* final tasks */
 
 	linker_compute_subclasses(c);
@@ -919,8 +932,7 @@ static classinfo *link_class_intern(classinfo *c)
 	RT_TIMING_TIME_DIFF(time_fill_vftbl   ,time_offsets      ,RT_TIMING_LINK_OFFSETS);
 	RT_TIMING_TIME_DIFF(time_offsets      ,time_fill_iftbl   ,RT_TIMING_LINK_F_IFTBL);
 	RT_TIMING_TIME_DIFF(time_fill_iftbl   ,time_finalizer    ,RT_TIMING_LINK_FINALIZER);
-	RT_TIMING_TIME_DIFF(time_finalizer    ,time_exceptions   ,RT_TIMING_LINK_EXCEPTS);
-	RT_TIMING_TIME_DIFF(time_exceptions   ,time_subclasses   ,RT_TIMING_LINK_SUBCLASS);
+	RT_TIMING_TIME_DIFF(time_finalizer    ,time_subclasses   ,RT_TIMING_LINK_SUBCLASS);
 
 	/* just return c to show that we didn't had a problem */
 
@@ -1148,29 +1160,31 @@ static void linker_compute_class_values(classinfo *c)
 
 static bool linker_addinterface(classinfo *c, classinfo *ic)
 {
-	s4     j, m;
-	s4     i   = ic->index;
-	vftbl_t *v = c->vftbl;
+	s4          j, k;
+	vftbl_t    *v;
+	s4          i;
+	classinfo  *sc;
+	methodinfo *m;
 
-	if (i >= v->interfacetablelength) {
-		log_text("Inernal error: interfacetable overflow");
-		assert(0);
-	}
+	v = c->vftbl;
+	i = ic->index;
+
+	if (i >= v->interfacetablelength)
+		vm_abort("Internal error: interfacetable overflow");
 
 	/* if this interface has already been added, return immediately */
 
-	if (v->interfacetable[-i])
+	if (v->interfacetable[-i] != NULL)
 		return true;
 
 	if (ic->methodscount == 0) {  /* fake entry needed for subtype test */
 		v->interfacevftbllength[i] = 1;
-		v->interfacetable[-i] = MNEW(methodptr, 1);
-		v->interfacetable[-i][0] = NULL;
-
+		v->interfacetable[-i]      = MNEW(methodptr, 1);
+		v->interfacetable[-i][0]   = NULL;
 	}
 	else {
 		v->interfacevftbllength[i] = ic->methodscount;
-		v->interfacetable[-i] = MNEW(methodptr, ic->methodscount);
+		v->interfacetable[-i]      = MNEW(methodptr, ic->methodscount);
 
 #if defined(ENABLE_STATISTICS)
 		if (opt_stat)
@@ -1179,17 +1193,16 @@ static bool linker_addinterface(classinfo *c, classinfo *ic)
 #endif
 
 		for (j = 0; j < ic->methodscount; j++) {
-			classinfo *sc = c;
+			for (sc = c; sc != NULL; sc = sc->super.cls) {
+				for (k = 0; k < sc->methodscount; k++) {
+					m = &(sc->methods[k]);
 
-			while (sc) {
-				for (m = 0; m < sc->methodscount; m++) {
-					methodinfo *mi = &(sc->methods[m]);
-
-					if (method_canoverwrite(mi, &(ic->methods[j]))) {
-						/* method mi overwrites the (abstract) method    */
+					if (method_canoverwrite(m, &(ic->methods[j]))) {
+						/* method m overwrites the (abstract) method */
 #if defined(ENABLE_VERIFIER)
-						/* Add loading constraints (for the more general */
-						/* types of the method ic->methods[j]).          */
+						/* Add loading constraints (for the more
+						   general types of the method
+						   ic->methods[j]).  */
 						if (!classcache_add_constraints_for_params(
 									c->classloader, ic->classloader,
 									&(ic->methods[j])))
@@ -1197,12 +1210,38 @@ static bool linker_addinterface(classinfo *c, classinfo *ic)
 							return false;
 						}
 #endif
-						v->interfacetable[-i][j] = v->table[mi->vftblindex];
+
+						/* XXX taken from gcj */
+						/* check for ACC_STATIC: IncompatibleClassChangeError */
+
+						/* check for !ACC_PUBLIC: IllegalAccessError */
+
+						/* check for ACC_ABSTRACT: AbstracMethodError,
+						   not sure about that one */
+
+						v->interfacetable[-i][j] = v->table[m->vftblindex];
 						goto foundmethod;
 					}
 				}
-				sc = sc->super.cls;
 			}
+
+			/* If no method was found, insert the AbstractMethodError
+			   stub. */
+
+#if defined(ENABLE_JIT)
+# if defined(ENABLE_INTRP)
+			if (opt_intrp)
+				v->interfacetable[-i][j] =
+					(methodptr) (ptrint) &intrp_asm_abstractmethoderror;
+			else
+# endif
+				v->interfacetable[-i][j] =
+					(methodptr) (ptrint) &asm_abstractmethoderror;
+#else
+			v->interfacetable[-i][j] =
+				(methodptr) (ptrint) &intrp_asm_abstractmethoderror;
+#endif
+
 		foundmethod:
 			;
 		}
@@ -1215,6 +1254,7 @@ static bool linker_addinterface(classinfo *c, classinfo *ic)
 			return false;
 
 	/* everything ok */
+
 	return true;
 }
 
