@@ -32,7 +32,7 @@
             Edwin Steiner
             Christian Thalinger
 
-   $Id: loader.c 4921 2006-05-15 14:24:36Z twisti $
+   $Id: loader.c 5845 2006-10-28 12:53:24Z edwin $
 
 */
 
@@ -74,19 +74,8 @@
 #include "vm/jit/codegen-common.h"
 #include "vm/rt-timing.h"
 
-
-/******************************************************************************/
-/* DEBUG HELPERS                                                              */
-/******************************************************************************/
-
-#ifndef NDEBUG
-#define LOADER_DEBUG
-#endif
-
-#ifdef LOADER_DEBUG
-#define LOADER_ASSERT(cond)  assert(cond)
-#else
-#define LOADER_ASSERT(cond)
+#if defined(ENABLE_JVMTI)
+#include "native/jvmti/cacaodbg.h"
 #endif
 
 
@@ -184,6 +173,10 @@ bool loader_init(void)
 		  load_class_bootstrap(utf_java_lang_ThreadGroup)))
 		return false;
 
+	if (!(class_java_lang_VMSystem =
+		  load_class_bootstrap(utf_new_char("java/lang/VMSystem"))))
+		return false;
+
 	if (!(class_java_lang_VMThread =
 		  load_class_bootstrap(utf_new_char("java/lang/VMThread"))))
 		return false;
@@ -676,7 +669,7 @@ static bool load_constantpool(classbuffer *cb, descriptor_pool *descpool)
 
 			/* link the class later, because we cannot link the class currently
 			   loading */
-			list_addfirst(&unlinkedclasses, tc);
+			list_add_first(&unlinkedclasses, tc);
 		}
 
 		/* the classref is created later */
@@ -872,19 +865,32 @@ static bool load_field(classbuffer *cb, fieldinfo *f, descriptor_pool *descpool)
 	f->class  = c;
 
 	switch (f->type) {
-	case TYPE_INT:     f->value.i = 0; break;
-	case TYPE_FLOAT:   f->value.f = 0.0; break;
-	case TYPE_DOUBLE:  f->value.d = 0.0; break;
-	case TYPE_ADDRESS: f->value.a = NULL;
-					   if (!(f->flags & ACC_STATIC))
-						   c->flags |= ACC_CLASS_HAS_POINTERS;
-					   break;
-	case TYPE_LONG:
+	case TYPE_INT:
+		f->value.i = 0;
+		break;
+
+	case TYPE_FLT:
+		f->value.f = 0.0;
+		break;
+
+	case TYPE_DBL:
+		f->value.d = 0.0;
+		break;
+
+	case TYPE_ADR:
+		f->value.a = NULL;
+		if (!(f->flags & ACC_STATIC))
+			c->flags |= ACC_CLASS_HAS_POINTERS;
+		break;
+
+	case TYPE_LNG:
 #if U8_AVAILABLE
-		f->value.l = 0; break;
+		f->value.l = 0;
 #else
-		f->value.l.low = 0; f->value.l.high = 0; break;
+		f->value.l.low  = 0;
+		f->value.l.high = 0;
 #endif
+		break;
 	}
 
 	/* read attributes */
@@ -930,7 +936,7 @@ static bool load_field(classbuffer *cb, fieldinfo *f, descriptor_pool *descpool)
 			}
 			break;
 					
-			case TYPE_LONG: {
+			case TYPE_LNG: {
 				constant_long *cl; 
 
 				if (!(cl = class_getconstant(c, pindex, CONSTANT_Long)))
@@ -940,7 +946,7 @@ static bool load_field(classbuffer *cb, fieldinfo *f, descriptor_pool *descpool)
 			}
 			break;
 
-			case TYPE_FLOAT: {
+			case TYPE_FLT: {
 				constant_float *cf;
 
 				if (!(cf = class_getconstant(c, pindex, CONSTANT_Float)))
@@ -950,7 +956,7 @@ static bool load_field(classbuffer *cb, fieldinfo *f, descriptor_pool *descpool)
 			}
 			break;
 											
-			case TYPE_DOUBLE: {
+			case TYPE_DBL: {
 				constant_double *cd;
 
 				if (!(cd = class_getconstant(c, pindex, CONSTANT_Double)))
@@ -960,7 +966,7 @@ static bool load_field(classbuffer *cb, fieldinfo *f, descriptor_pool *descpool)
 			}
 			break;
 						
-			case TYPE_ADDRESS:
+			case TYPE_ADR:
 				if (!(u = class_getconstant(c, pindex, CONSTANT_String)))
 					return false;
 
@@ -1162,33 +1168,33 @@ static bool load_method(classbuffer *cb, methodinfo *m, descriptor_pool *descpoo
 			if (!suck_check_classbuffer_size(cb, 2))
 				return false;
 
-			m->exceptiontablelength = suck_u2(cb);
-			if (!suck_check_classbuffer_size(cb, (2 + 2 + 2 + 2) * m->exceptiontablelength))
+			m->rawexceptiontablelength = suck_u2(cb);
+			if (!suck_check_classbuffer_size(cb, (2 + 2 + 2 + 2) * m->rawexceptiontablelength))
 				return false;
 
-			m->exceptiontable = MNEW(exceptiontable, m->exceptiontablelength);
+			m->rawexceptiontable = MNEW(raw_exception_entry, m->rawexceptiontablelength);
 
 #if defined(ENABLE_STATISTICS)
 			if (opt_stat) {
 				count_vmcode_len += m->jcodelength + 18;
 				count_extable_len +=
-					m->exceptiontablelength * sizeof(exceptiontable);
+					m->rawexceptiontablelength * sizeof(raw_exception_entry);
 			}
 #endif
 
-			for (j = 0; j < m->exceptiontablelength; j++) {
+			for (j = 0; j < m->rawexceptiontablelength; j++) {
 				u4 idx;
-				m->exceptiontable[j].startpc = suck_u2(cb);
-				m->exceptiontable[j].endpc = suck_u2(cb);
-				m->exceptiontable[j].handlerpc = suck_u2(cb);
+				m->rawexceptiontable[j].startpc = suck_u2(cb);
+				m->rawexceptiontable[j].endpc = suck_u2(cb);
+				m->rawexceptiontable[j].handlerpc = suck_u2(cb);
 
 				idx = suck_u2(cb);
 				if (!idx) {
-					m->exceptiontable[j].catchtype.any = NULL;
+					m->rawexceptiontable[j].catchtype.any = NULL;
 
 				} else {
 					/* the classref is created later */
-					if (!(m->exceptiontable[j].catchtype.any =
+					if (!(m->rawexceptiontable[j].catchtype.any =
 						  (utf*)class_getconstant(c, idx, CONSTANT_Class)))
 						return false;
 				}
@@ -1392,9 +1398,9 @@ classinfo *load_class_from_sysloader(utf *name)
 	java_objectheader *cl;
 	classinfo         *c;
 
-	LOADER_ASSERT(class_java_lang_Object);
-	LOADER_ASSERT(class_java_lang_ClassLoader);
-	LOADER_ASSERT(class_java_lang_ClassLoader->state & CLASS_LINKED);
+	assert(class_java_lang_Object);
+	assert(class_java_lang_ClassLoader);
+	assert(class_java_lang_ClassLoader->state & CLASS_LINKED);
 	
 	m = class_resolveclassmethod(class_java_lang_ClassLoader,
 								 utf_getSystemClassLoader,
@@ -1443,7 +1449,7 @@ classinfo *load_class_from_classloader(utf *name, java_objectheader *cl)
 
 	RT_TIMING_GET_TIME(time_start);
 
-	LOADER_ASSERT(name);
+	assert(name);
 
 	/* lookup if this class has already been loaded */
 
@@ -1528,7 +1534,7 @@ classinfo *load_class_from_classloader(utf *name, java_objectheader *cl)
 			}
 		}
 		
-		LOADER_ASSERT(class_java_lang_Object);
+		assert(class_java_lang_Object);
 
 		lc = class_resolveclassmethod(cl->vftbl->class,
 									  utf_loadClass,
@@ -1596,6 +1602,12 @@ classinfo *load_class_from_classloader(utf *name, java_objectheader *cl)
 			printf("]\n");
 		}
 
+#if defined(ENABLE_JVMTI)
+		/* fire Class Load JVMTI event */
+		if (jvmti) jvmti_ClassLoadPrepare(false, c);
+#endif
+
+
 		return c;
 	} 
 
@@ -1636,7 +1648,7 @@ classinfo *load_class_bootstrap(utf *name)
 
 	/* for debugging */
 
-	LOADER_ASSERT(name);
+	assert(name);
 
 	/* lookup if this class has already been loaded */
 
@@ -1661,7 +1673,7 @@ classinfo *load_class_bootstrap(utf *name)
 		c = load_newly_created_array(c, NULL);
 		if (c == NULL)
 			return NULL;
-		LOADER_ASSERT(c->state & CLASS_LOADED);
+		assert(c->state & CLASS_LOADED);
 
 		RT_TIMING_GET_TIME(time_array);
 		RT_TIMING_TIME_DIFF(time_start,time_array,RT_TIMING_LOAD_BOOT_ARRAY);
@@ -2094,12 +2106,12 @@ classinfo *load_class_from_classbuffer(classbuffer *cb)
 		if (!m->parseddesc)
 			goto return_exception;
 
-		for (j = 0; j < m->exceptiontablelength; j++) {
-			if (!m->exceptiontable[j].catchtype.any)
+		for (j = 0; j < m->rawexceptiontablelength; j++) {
+			if (!m->rawexceptiontable[j].catchtype.any)
 				continue;
-			if ((m->exceptiontable[j].catchtype.ref =
+			if ((m->rawexceptiontable[j].catchtype.ref =
 				 descriptor_pool_lookup_classref(descpool,
-						(utf *) m->exceptiontable[j].catchtype.any)) == NULL)
+						(utf *) m->rawexceptiontable[j].catchtype.any)) == NULL)
 				goto return_exception;
 		}
 
@@ -2252,9 +2264,9 @@ classinfo *load_class_from_classbuffer(classbuffer *cb)
 
 #if defined(ENABLE_STATISTICS)
 	if (opt_stat) {
-		count_class_infos += sizeof(classinfo*) * c->interfacescount;
-		count_class_infos += sizeof(fieldinfo)  * c->fieldscount;
-		count_class_infos += sizeof(methodinfo) * c->methodscount;
+		size_classinfo  += sizeof(classinfo*) * c->interfacescount;
+		size_fieldinfo  += sizeof(fieldinfo)  * c->fieldscount;
+		size_methodinfo += sizeof(methodinfo) * c->methodscount;
 	}
 #endif
 
@@ -2266,12 +2278,12 @@ classinfo *load_class_from_classbuffer(classbuffer *cb)
 	if (!load_attributes(cb, suck_u2(cb)))
 		goto return_exception;
 
-#if 0
-	/* Pre java 1.5 version don't check this. This implementation is like
-	   java 1.5 do it: for class file version 45.3 we don't check it, older
+	/* Pre Java 1.5 version don't check this. This implementation is like
+	   Java 1.5 do it: for class file version 45.3 we don't check it, older
 	   versions are checked.
 	 */
-	if ((ma == 45 && mi > 3) || ma > 45) {
+
+	if (((ma == 45) && (mi > 3)) || (ma > 45)) {
 		/* check if all data has been read */
 		s4 classdata_left = ((cb->data + cb->size) - cb->pos);
 
@@ -2280,7 +2292,6 @@ classinfo *load_class_from_classbuffer(classbuffer *cb)
 			goto return_exception;
 		}
 	}
-#endif
 
 	RT_TIMING_GET_TIME(time_attrs);
 
@@ -2292,6 +2303,13 @@ classinfo *load_class_from_classbuffer(classbuffer *cb)
 
 	c->state = (c->state & ~CLASS_LOADING) | CLASS_LOADED;
 
+#if defined(ENABLE_JVMTI)
+	/* fire Class Prepare JVMTI event */
+
+	if (jvmti)
+		jvmti_ClassLoadPrepare(true, c);
+#endif
+
 #if !defined(NDEBUG)
 	if (loadverbose)
 		log_message_class("Loading done class: ", c);
@@ -2374,7 +2392,7 @@ classinfo *load_newly_created_array(classinfo *c, java_objectheader *loader)
 		if (!(comp = load_class_from_classloader(u, loader)))
 			return NULL;
 
-		LOADER_ASSERT(comp->state & CLASS_LOADED);
+		assert(comp->state & CLASS_LOADED);
 
 		if (opt_eager)
 			if (!link_class(c))
@@ -2399,7 +2417,7 @@ classinfo *load_newly_created_array(classinfo *c, java_objectheader *loader)
 		if (!(comp = load_class_from_classloader(u, loader)))
 			return NULL;
 
-		LOADER_ASSERT(comp->state & CLASS_LOADED);
+		assert(comp->state & CLASS_LOADED);
 
 		if (opt_eager)
 			if (!link_class(c))
@@ -2424,9 +2442,9 @@ classinfo *load_newly_created_array(classinfo *c, java_objectheader *loader)
 		c->classloader = NULL;
 	}
 
-	LOADER_ASSERT(class_java_lang_Object);
-	LOADER_ASSERT(class_java_lang_Cloneable);
-	LOADER_ASSERT(class_java_io_Serializable);
+	assert(class_java_lang_Object);
+	assert(class_java_lang_Cloneable);
+	assert(class_java_io_Serializable);
 
 	/* setup the array class */
 
@@ -2439,13 +2457,13 @@ classinfo *load_newly_created_array(classinfo *c, java_objectheader *loader)
 		classinfo *tc;
 
 		tc = class_java_lang_Cloneable;
-		LOADER_ASSERT(tc->state & CLASS_LOADED);
-		list_addfirst(&unlinkedclasses, tc);
+		assert(tc->state & CLASS_LOADED);
+		list_add_first(&unlinkedclasses, tc);
 		c->interfaces[0].cls = tc;
 
 		tc = class_java_io_Serializable;
-		LOADER_ASSERT(tc->state & CLASS_LOADED);
-		list_addfirst(&unlinkedclasses, tc);
+		assert(tc->state & CLASS_LOADED);
+		list_add_first(&unlinkedclasses, tc);
 		c->interfaces[1].cls = tc;
 
 	} else {
@@ -2463,7 +2481,7 @@ classinfo *load_newly_created_array(classinfo *c, java_objectheader *loader)
 	/* create descriptor for clone method */
 	/* we need one paramslot which is reserved for the 'this' parameter */
 	clonedesc = NEW(methoddesc);
-	clonedesc->returntype.type = TYPE_ADDRESS;
+	clonedesc->returntype.type = TYPE_ADR;
 	clonedesc->returntype.classref = classrefs + 1;
 	clonedesc->returntype.arraydim = 0;
 	/* initialize params to "empty", add real params below in
