@@ -29,7 +29,7 @@
    Changes: Christian Thalinger
             Edwin Steiner
 
-   $Id: method.h 4758 2006-04-12 17:51:10Z edwin $
+   $Id: method.h 5785 2006-10-15 22:25:54Z edwin $
 */
 
 
@@ -38,7 +38,7 @@
 
 /* forward typedefs ***********************************************************/
 
-typedef struct exceptiontable exceptiontable;
+typedef struct raw_exception_entry raw_exception_entry;
 typedef struct lineinfo lineinfo; 
 
 #include "config.h"
@@ -62,8 +62,6 @@ struct methodinfo {                 /* method structure                       */
 	utf          *descriptor;       /* JavaVM descriptor string of method     */
 	methoddesc   *parseddesc;       /* parsed descriptor                      */
 			     
-	bool          isleafmethod;     /* does method call subroutines           */ /* XXX */
-			     
 	classinfo    *class;            /* class, the method belongs to           */
 	s4            vftblindex;       /* index of method in virtual function    */
 	                                /* table (if it is a virtual method)      */
@@ -71,22 +69,9 @@ struct methodinfo {                 /* method structure                       */
 	s4            maxlocals;        /* maximum number of local variables      */
 	s4            jcodelength;      /* length of JavaVM code                  */
 	u1           *jcode;            /* pointer to JavaVM code                 */
-			     
-	s4            basicblockcount;  /* number of basic blocks                 */
-	basicblock   *basicblocks;      /* points to basic block array            */
-	s4           *basicblockindex;  /* a table which contains for every byte  */
-	                                /* of JavaVM code a basic block index if  */
-	                                /* at this byte is the start of a basic   */
-	                                /* block                                  */
 
-	s4            instructioncount; /* number of JavaVM instructions          */
-	instruction  *instructions;     /* points to intermediate code instr.     */
-
-	s4            stackcount;       /* number of stack elements               */
-	stackelement *stack;            /* points to stack elements               */
-
-	s4            exceptiontablelength; /* exceptiontable length              */
-	exceptiontable *exceptiontable; /* the exceptiontable                     */
+	s4            rawexceptiontablelength;  /* exceptiontable length          */
+	raw_exception_entry *rawexceptiontable; /* the exceptiontable             */
 
 	u2            thrownexceptionscount; /* number of exceptions attribute    */
 	classref_or_classinfo *thrownexceptions; /* except. a method may throw    */
@@ -94,39 +79,24 @@ struct methodinfo {                 /* method structure                       */
 	u2            linenumbercount;  /* number of linenumber attributes        */
 	lineinfo     *linenumbers;      /* array of lineinfo items                */
 
-	int       c_debug_nr;           /* a counter to number all BB with an     */
-	                                /* unique value                           */
-
 	u1           *stubroutine;      /* stub for compiling or calling natives  */
 	codeinfo     *code;             /* current code of this method            */
 
 #if defined(ENABLE_LSRA)
 	s4            maxlifetimes;     /* helper for lsra                        */
 #endif
-
-	u4            frequency;        /* number of method invocations           */ /* XXX */
-	u4           *bbfrequency; /* XXX */
-
-	s8            cycles;           /* number of cpu cycles                   */ /* XXX */
 };
 
 
-/* exceptiontable *************************************************************/
+/* raw_exception_entry ********************************************************/
 
-struct exceptiontable {         /* exceptiontable entry in a method           */
-	s4              startpc;    /* start pc of guarded area (inclusive)       */
-	basicblock     *start;
+/* exception table entry read by the loader */
 
-	s4              endpc;      /* end pc of guarded area (exklusive)         */
-	basicblock     *end;
-
-	s4              handlerpc;  /* pc of exception handler                    */
-	basicblock     *handler;
-
-	classref_or_classinfo catchtype; /* catchtype of exc. (NULL == catchall)  */
-	exceptiontable *next;       /* used to build a list of exception when     */
-	                            /* loops are copied */
-	exceptiontable *down;       /* instead of the old array, a list is used   */
+struct raw_exception_entry {    /* exceptiontable entry in a method           */
+	classref_or_classinfo catchtype; /* catchtype of exc. (0 == catchall)     */
+	u2              startpc;    /* start pc of guarded area (inclusive)       */
+	u2              endpc;      /* end pc of guarded area (exklusive)         */
+	u2              handlerpc;  /* pc of exception handler                    */
 };
 
 
