@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: options.c 4942 2006-05-23 08:42:46Z twisti $
+   $Id: options.c 5842 2006-10-27 10:41:02Z twisti $
 
 */
 
@@ -49,6 +49,8 @@
 s4    opt_index = 0;            /* index of processed arguments               */
 char *opt_arg;                  /* this one exports the option argument       */
 
+bool opt_foo = false;           /* option for development                     */
+
 bool opt_jar = false;
 
 #if defined(ENABLE_JIT)
@@ -66,6 +68,7 @@ s4   opt_heapstartsize = 0;     /* initial heap size                          */
 s4   opt_stacksize     = 0;     /* thread stack size                          */
 
 bool opt_verbose = false;
+bool opt_debugcolor = false;	/* use ANSI terminal sequences 		      */
 bool compileall = false;
 
 bool loadverbose = false;
@@ -119,8 +122,10 @@ bool opt_verify  = true;       /* true if classfiles should be verified      */
 #endif
 bool opt_eager   = false;
 
+#if defined(ENABLE_PROFILING)
 bool opt_prof    = false;
 bool opt_prof_bb = false;
+#endif
 
 
 /* optimization options *******************************************************/
@@ -129,7 +134,7 @@ bool opt_prof_bb = false;
 bool opt_ifconv = false;
 #endif
 
-#if defined(ENABLE_LSRA)
+#if defined(ENABLE_LSRA) || defined(ENABLE_SSA)
 bool opt_lsra = false;
 #endif
 
@@ -196,6 +201,10 @@ s4 options_get(opt_struct *opts, JavaVMInitArgs *vm_args)
 			} else {
 				/* parameter and option have no space between */
 
+				/* FIXME: this assumption is plain wrong, hits you if there is a
+				 * parameter with no argument starting with same letter as param with argument
+				 * but named after that one, ouch! */
+
 				size_t l = strlen(opts[i].name);
 
 				if (strlen(option + 1) > l) {
@@ -213,31 +222,6 @@ s4 options_get(opt_struct *opts, JavaVMInitArgs *vm_args)
 }
 
 
-/* options_prepare *************************************************************
-
-   Prepare the JavaVMInitArgs.
-
-*******************************************************************************/
-
-JavaVMInitArgs *options_prepare(int argc, char **argv)
-{
-	JavaVMInitArgs *vm_args;
-	s4              i;
-
-	vm_args = NEW(JavaVMInitArgs);
-
-	vm_args->version            = JNI_VERSION_1_2;
-	vm_args->nOptions           = argc - 1;
-	vm_args->options            = MNEW(JavaVMOption, argc);
-	vm_args->ignoreUnrecognized = JNI_FALSE;
-
-	for (i = 1; i < argc; i++)
-		vm_args->options[i - 1].optionString = argv[i];
-
-	return vm_args;
-}
-
-
 /*
  * These are local overrides for various environment variables in Emacs.
  * Please do not remove this and leave it at the end of the file, where
