@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: options.h 4942 2006-05-23 08:42:46Z twisti $
+   $Id: options.h 5842 2006-10-27 10:41:02Z twisti $
 
 */
 
@@ -67,6 +67,8 @@ struct opt_struct {
 extern s4    opt_index;
 extern char *opt_arg;
 
+extern bool opt_foo;
+
 extern bool opt_jit;
 extern bool opt_intrp;
 
@@ -78,6 +80,7 @@ extern s4   opt_heapstartsize;
 extern s4   opt_stacksize;
 
 extern bool opt_verbose;
+extern bool opt_debugcolor;
 extern bool compileall;
 
 extern bool loadverbose;         /* Print debug messages during loading */
@@ -131,8 +134,10 @@ extern bool opt_verify;
 #endif
 extern bool opt_eager;
 
+#if defined(ENABLE_PROFILING)
 extern bool opt_prof;
 extern bool opt_prof_bb;
+#endif
 
 
 /* optimization options *******************************************************/
@@ -141,7 +146,7 @@ extern bool opt_prof_bb;
 extern bool opt_ifconv;
 #endif
 
-#if defined(ENABLE_LSRA)
+#if defined(ENABLE_LSRA) || defined(ENABLE_SSA)
 extern bool opt_lsra;
 #endif
 
@@ -161,7 +166,6 @@ extern bool vm_debug;
 /* function prototypes ********************************************************/
 
 s4 options_get(opt_struct *opts, JavaVMInitArgs *vm_args);
-JavaVMInitArgs *options_prepare(int argc, char **argv);
 
 #endif /* _OPTIONS_H */
 
