@@ -28,7 +28,7 @@
 
    Changes:
 
-   $Id: signal.c 4921 2006-05-15 14:24:36Z twisti $
+   $Id: signal.c 5809 2006-10-20 13:09:54Z twisti $
 
 */
 
@@ -52,9 +52,9 @@
 
 /* function prototypes ********************************************************/
 
-void signal_handler_sigquit(int sig, siginfo_t *siginfo, void *_p);
+void signal_handler_sighup(int sig, siginfo_t *siginfo, void *_p);
 void signal_handler_sigint(int sig, siginfo_t *siginfo, void *_p);
-void signal_handler_sigusr1(int sig, siginfo_t *siginfo, void *_p);
+void signal_handler_sigquit(int sig, siginfo_t *siginfo, void *_p);
 
 
 /* signal_init *****************************************************************
@@ -65,13 +65,18 @@ void signal_handler_sigusr1(int sig, siginfo_t *siginfo, void *_p);
 
 void signal_init(void)
 {
+#if !defined(__CYGWIN__)
 	struct sigaction act;
 
+	/* Allocate something so the garbage collector's signal handlers
+	   are installed. */
+
+	(void) GCNEW(u1);
+
 	/* install signal handlers we need to convert to exceptions */
 
 	sigemptyset(&act.sa_mask);
 
-
 #if defined(ENABLE_JIT)
 # if defined(ENABLE_INTRP)
 	if (!opt_intrp) {
@@ -80,7 +85,7 @@ void signal_init(void)
 
 		if (!checknull) {
 			act.sa_sigaction = md_signal_handler_sigsegv;
-			act.sa_flags = SA_NODEFER | SA_SIGINFO;
+			act.sa_flags     = SA_NODEFER | SA_SIGINFO;
 
 #if defined(SIGSEGV)
 			sigaction(SIGSEGV, &act, NULL);
@@ -91,12 +96,11 @@ void signal_init(void)
 #endif
 		}
 
-
 		/* catch ArithmeticException */
 
 #if defined(__I386__) || defined(__X86_64__)
 		act.sa_sigaction = md_signal_handler_sigfpe;
-		act.sa_flags = SA_NODEFER | SA_SIGINFO;
+		act.sa_flags     = SA_NODEFER | SA_SIGINFO;
 		sigaction(SIGFPE, &act, NULL);
 #endif
 # if defined(ENABLE_INTRP)
@@ -104,36 +108,61 @@ void signal_init(void)
 # endif
 #endif /* !defined(ENABLE_INTRP) */
 
+#if defined(ENABLE_THREADS)
+	/* catch SIGHUP for threads_thread_interrupt */
+
+	act.sa_sigaction = signal_handler_sighup;
+	act.sa_flags     = 0;
+	sigaction(SIGHUP, &act, NULL);
+#endif
 
 	/* catch SIGINT for exiting properly on <ctrl>-c */
 
 	act.sa_sigaction = signal_handler_sigint;
-	act.sa_flags = SA_NODEFER | SA_SIGINFO;
+	act.sa_flags     = SA_NODEFER | SA_SIGINFO;
 	sigaction(SIGINT, &act, NULL);
 
-
-
+#if defined(ENABLE_THREADS)
 	/* catch SIGQUIT for thread dump */
 
-#if defined(ENABLE_THREADS)
-#if !defined(__FREEBSD__)
+# if !defined(__FREEBSD__)
 	act.sa_sigaction = signal_handler_sigquit;
-	act.sa_flags = SA_SIGINFO;
+	act.sa_flags     = SA_SIGINFO;
 	sigaction(SIGQUIT, &act, NULL);
-
-	/* XXX boehm uses SIGUSR1 for suspend on freebsd */
-
-	act.sa_sigaction = signal_handler_sigusr1;
-	act.sa_flags = SA_SIGINFO;
-	sigaction(SIGUSR1, &act, NULL);
+# endif
 #endif
+
+#if defined(ENABLE_THREADS) && defined(ENABLE_PROFILING)
+	/* install signal handler for profiling sampling */
+
+	act.sa_sigaction = md_signal_handler_sigusr2;
+	act.sa_flags     = SA_SIGINFO;
+	sigaction(SIGUSR2, &act, NULL);
 #endif
+
+#endif /* !defined(__CYGWIN__) */
 }
 
 
+/* signal_handler_sighup *******************************************************
+
+   This handler is required by threads_thread_interrupt and does
+   nothing.
+
+*******************************************************************************/
+
+#if defined(ENABLE_THREADS)
+void signal_handler_sighup(int sig, siginfo_t *siginfo, void *_p)
+{
+	/* do nothing */
+}
+#endif
+
+
 /* signal_handler_sigquit ******************************************************
 
-   XXX
+   Handle for SIGQUIT (<ctrl>-\) which print a stacktrace for every
+   running thread.
 
 *******************************************************************************/
 
@@ -169,22 +198,6 @@ void signal_handler_sigint(int sig, siginfo_t *siginfo, void *_p)
 }
 
 
-/* signal_handler_sigusr1 ******************************************************
-
-   XXX
-
-*******************************************************************************/
-
-#if defined(ENABLE_THREADS)
-void signal_handler_sigusr1(int sig, siginfo_t *siginfo, void *_p)
-{
-	/* call stacktrace function */
-
-	stacktrace_dump_trace();
-}
-#endif
-
-
 /*
  * These are local overrides for various environment variables in Emacs.
  * Please do not remove this and leave it at the end of the file, where
