@@ -31,7 +31,7 @@
    Changes: Christian Thalinger
    			Edwin Steiner
 
-   $Id: string.c 4921 2006-05-15 14:24:36Z twisti $
+   $Id: string.c 5823 2006-10-24 23:24:19Z edwin $
 
 */
 
@@ -46,6 +46,14 @@
 
 #include "mm/memory.h"
 #include "native/include/java_lang_String.h"
+
+#if defined(ENABLE_THREADS)
+# include "threads/native/lock.h"
+#else
+# include "threads/none/lock.h"
+#endif
+
+#include "vm/builtin.h"
 #include "vm/exceptions.h"
 #include "vm/loader.h"
 #include "vm/options.h"
@@ -314,6 +322,63 @@ java_lang_String *javastring_new_from_utf_buffer(const char *buffer, u4 blength)
 }
 
 
+/* javastring_safe_new_from_utf8 ***********************************************
+
+   Create a new object of type java/lang/String with the text from
+   the specified UTF-8 string. This function is safe for invalid UTF-8.
+   (Invalid characters will be replaced by U+fffd.)
+
+   IN:
+      text.........the UTF-8 string, zero-terminated.
+
+   RETURN VALUE:
+      the java.lang.String object, or
+      NULL if an exception has been thrown
+
+*******************************************************************************/
+
+java_lang_String *javastring_safe_new_from_utf8(const char *text)
+{
+	java_lang_String *s;            /* result-string                          */
+	java_chararray *a;
+	s4 nbytes;
+	s4 len;
+
+	assert(text);
+
+	/* Get number of bytes. We need this to completely emulate the messy */
+	/* behaviour of the RI. :(                                           */
+
+	nbytes = strlen(text);
+
+	/* calculate number of Java characters */
+
+	len = utf8_safe_number_of_u2s(text, nbytes);
+
+	/* allocate the String object and the char array */
+
+	s = (java_lang_String *) builtin_new(class_java_lang_String);
+	a = builtin_newarray_char(len);
+
+	/* javastring or character-array could not be created? */
+
+	if (!a || !s)
+		return NULL;
+
+	/* decompress UTF-8 string */
+
+	utf8_safe_convert_to_u2s(text, nbytes, a->data);
+
+	/* set fields of the String object */
+
+	s->value  = a;
+	s->offset = 0;
+	s->count  = len;
+
+	return s;
+}
+
+
 /* javastring_new_from_utf_string **********************************************
 
    Create a new object of type java/lang/String with the text from
@@ -521,11 +586,12 @@ char *javastring_tochar(java_objectheader *so)
 
 *******************************************************************************/
 
-utf *javastring_toutf(java_lang_String *string, bool isclassname)
+utf *javastring_toutf(java_lang_String *s, bool isclassname)
 {
-	java_lang_String *str = (java_lang_String *) string;
+	if (s == NULL)
+		return utf_null;
 
-	return utf_new_u2(str->value->data + str->offset, str->count, isclassname);
+	return utf_new_u2(s->value->data + s->offset, s->count, isclassname);
 }
 
 
@@ -537,7 +603,7 @@ utf *javastring_toutf(java_lang_String *string, bool isclassname)
 
 s4 javastring_strlen(java_lang_String *s)
 {
-	if (!s)
+	if (s == NULL)
 		return 0;
 
 	return s->count;
@@ -563,9 +629,7 @@ java_objectheader *literalstring_u2(java_chararray *a, u4 length, u4 offset,
     u4                slot;
     u2                i;
 
-#if defined(ENABLE_THREADS)
-	builtin_monitorenter(lock_hashtable_string);
-#endif
+	LOCK_MONITOR_ENTER(lock_hashtable_string);
 
     /* find location in hashtable */
 
@@ -588,9 +652,7 @@ java_objectheader *literalstring_u2(java_chararray *a, u4 length, u4 offset,
 			if (!copymode)
 				mem_free(a, sizeof(java_chararray) + sizeof(u2) * (length - 1) + 10);
 
-#if defined(ENABLE_THREADS)
-			builtin_monitorexit(lock_hashtable_string);
-#endif
+			LOCK_MONITOR_EXIT(lock_hashtable_string);
 
 			return (java_objectheader *) js;
 		}
@@ -624,7 +686,7 @@ java_objectheader *literalstring_u2(java_chararray *a, u4 length, u4 offset,
 	/* if we use eager loading, we have to check loaded String class */
 
 	if (opt_eager)
-		list_addfirst(&unlinkedclasses, class_java_lang_String);
+		list_add_first(&unlinkedclasses, class_java_lang_String);
 
 	/* create new javastring */
 
@@ -691,9 +753,7 @@ java_objectheader *literalstring_u2(java_chararray *a, u4 length, u4 offset,
 		hashtable_string = newhash;
 	}
 
-#if defined(ENABLE_THREADS)
-	builtin_monitorexit(lock_hashtable_string);
-#endif
+	LOCK_MONITOR_EXIT(lock_hashtable_string);
 
 	return (java_objectheader *) js;
 }
