@@ -28,7 +28,7 @@
 
    Changes: Edwin Steiner
 
-   $Id: utf8.h 4887 2006-05-05 19:31:39Z edwin $
+   $Id: utf8.h 5823 2006-10-24 23:24:19Z edwin $
 
 */
 
@@ -77,12 +77,14 @@ extern utf *utf_java_io_Serializable;
 extern utf *utf_java_lang_Throwable;
 extern utf *utf_java_lang_VMThrowable;
 extern utf *utf_java_lang_Error;
-extern utf *utf_java_lang_NoClassDefFoundError;
+extern utf *utf_java_lang_AbstractMethodError;
 extern utf *utf_java_lang_LinkageError;
+extern utf *utf_java_lang_NoClassDefFoundError;
 extern utf *utf_java_lang_NoSuchMethodError;
 extern utf *utf_java_lang_OutOfMemoryError;
 
 extern utf *utf_java_lang_Exception;
+extern utf *utf_java_lang_ClassCastException;
 extern utf *utf_java_lang_ClassNotFoundException;
 extern utf *utf_java_lang_IllegalArgumentException;
 extern utf *utf_java_lang_IllegalMonitorStateException;
@@ -121,6 +123,7 @@ extern utf *utf_run;
 
 extern utf *utf_add;
 extern utf *utf_remove;
+extern utf *utf_removeThread;
 extern utf *utf_put;
 extern utf *utf_get;
 extern utf *utf_value;
@@ -155,10 +158,11 @@ extern utf *utf_void__java_lang_Throwable;
 extern utf *utf_java_lang_Object__java_lang_Object;
 extern utf *utf_java_lang_String__void;
 extern utf *utf_java_lang_String__java_lang_Class;
+extern utf *utf_java_lang_Thread__V;
 extern utf *utf_java_lang_Throwable__void;
 
 extern utf *utf_not_named_yet;
-
+extern utf *utf_null;
 extern utf *array_packagename;
 
 
@@ -188,7 +192,11 @@ u4 utf_bytes(utf *u);
 /* get next unicode character of a utf-string */
 u2 utf_nextu2(char **utf);
 
-/* get number of unicode characters of a utf string */
+/* get (number of) unicode characters of a utf string (safe) */
+s4 utf8_safe_number_of_u2s(const char *text, s4 nbytes);
+void utf8_safe_convert_to_u2s(const char *text, s4 nbytes, u2 *buffer);
+
+/* get (number of) unicode characters of a utf string (UNSAFE!) */
 u4 utf_get_number_of_u2s(utf *u);
 u4 utf_get_number_of_u2s_for_buffer(const char *buffer, u4 blength);
 
